#Set the serial port to a baud rate of 115200
proc initializeSerials {} {
    global gvar

    foreach tty $gvar(ttyList) {
	#puts "Initialize $tty -- pid: $gvar(pid)"
	if {$opSystem == "HP-UX"} {
	    exec stty raw $baud cs8 cread crts clocal rtsxoff ctsxon < $tty
	} elseif {$opSystem == "SunOS"} {
	    exec stty raw 38400 cs8 cread crtscts clocal crtsxoff < $tty
	} else {
	    #...
	}
    }
}

#Open the serial port and set it to a baud rate of 115200
proc openPorts {} {
    global gvar

    if {[info exists gvar(portSerial)] || \
	    $gvar(automatic) == 1} {
	Debug "open port $gvar(tty)" 1
	set tty $gvar(tty)
	set gvar($tty) [open $tty r+]

	Debug "Speed is $gvar(baud)" 1
	set baud $gvar(baud)
	fconfigure $gvar($tty)  -mode $baud,n,8,1
	set opSystem [exec uname]
	
	if {$opSystem == "HP-UX"} {
	    Debug "Configure port for HP-UX" 3
	    exec stty raw $baud cs8 cread crts clocal rtsxoff ctsxon < $tty
	} elseif {$opSystem == "SunOS"} {
	    Debug "Configure port for Solaris" 3
	    exec stty raw $baud cs8 cread crtscts clocal crtsxoff < $tty
	} else {
	    #Use the same of Solaris...for the moment...
	    exec stty raw $baud cs8 cread crtscts clocal crtsxoff < $tty
	}
    } else {
	set gvar(tty) $gvar(portParallel)
    }
}

#Send a string to the robot via the serial port.
proc sendMessage {action} {
    global gvar 

    Debug "Send message: puts $gvar(tty) $action" 3
    puts $gvar($gvar(tty)) $action
    flush $gvar($gvar(tty))
}

#Open the printer drawer.
proc openPrinter {} {
    global gvar

    Log "Open Printer"
    sendMessage $gvar(openPrinter)
    readPort 6
}

#Close the printer drawer.
proc closePrinter {} {
    global gvar 

    Log "Close Printer"
    sendMessage $gvar(closePrinter)
    readPort 6
}

#Switch on the fan. There is only one fan for two writers.
proc fanOn {} {
    global gvar
    
    write_file $gvar(reqDir)/fan-$gvar(id)-$gvar(ttyTail).romanic ""
    Log "Switch on fan"
    sendMessage $gvar(fanOn)
    readPort 6 
}

#Switch off the fan.
proc fanOff {} {
    global gvar
    
    Log "Switch off fan"
    sendMessage $gvar(fanOff)
    readPort 6
}

#Send an enquiry message to the robot.
proc romanicPilotEnquiry {} {
    global gvar
    
    Debug "Enquiry" 3
    sendMessage $gvar(enquiryString)
    readPort 6
}

#Query all the robots connected and wait for an answer.
proc romanicRequestSense {} {
    global gvar
    
	set tty $gvar(tty)
	Debug "Enquiry robot $tty" 2
	if {[waitForOk]} {
	    puts "Cannot communicate with the Robot connected to $tty"
	    puts "Verify that the robot:"
	    puts "- Is powered on"
	    puts "- Has its door closed"
	    exit 1
	}
}

#Query the robot up to 3 times. If the output is not OK 
#generate an error.
proc waitForOk {} {
    global gvar
    
    set i 0
    while {[romanicRequestSense] != "Ok --- Ok"} {
	Debug "Robot is not ok. Try again." 1
	if {$i == 3} {
	    return 1
	}
	after 3000
	incr i
    }
    return 0
}

#Enquiry the robot if the previous command gave an error.
#It gives more information about an error.
proc romanicRequestSense {} {
    global gvar 

    sendMessage $gvar(requestSenseString)
    set sense [readPort 11]
    set senseData1 [string range $sense 4 6]
    set senseData2 [string range $sense 7 9]

    Debug "sense: $sense" 3
    Debug "Sense data 1: $senseData1" 3
    Debug "Sense data 2: $senseData2" 3
    
    switch  $senseData1 {
	"000" {set sense1 "Ok"}
	"001" {set sense1 "Recovered Error (Retry was required)"}
	"002" {set sense1 "Not Ready; Check Condition. The unit was powered-on"}
	"003" {set sense1 "Medium Error."}
	"004" {set sense1 "Medium Hardware Error. Indicates the unit had problems gripping the media or the media is not present. Perform request element status for more information."}
	"005" {set sense1 "Illegal Request. Indicates there is an illegal parameter in the command."}
	"006" {set sense1 "Unit Attention. Indicates the operator access is currently open or was opened during operation."}
	"007" {set sense1 "No Recorder Detected. Indicates the Drawer Detect sensor could not locate an open recorder drawer."}
	"008" {set sense1 "No carousel."}
	"009" {set sense1 "No Printer or Printer Communication Failure. Cannot communicate with the printer."}
	"010" {set sense1 "Carousel Failure. Cannot position the carousel."}
	"011" {set sense1 "Lif Failure. Cannot position the lift."}
	"012" {set sense1 "Drawer Detect sensor is not functioning."}
	"013" {set sense1 "Reserved."}
	"014" {set sense1 "Reserved."}
	"015" {set sense1 "Host Communication Failure."}
	"016" {set sense1 "Program Flash failure."}
	"017" {set sense1 "Rotate medium failure."}
	"018" {set sense1 "Mailbox Failure."}
	"019" {set sense1 "Perfect Print Failure."}
	default { set sense1 "Wrong Sense." }
    }

    switch $senseData2 {
	"000" {set sense2 "Ok"}
	"001" {set sense2 "Ribbon out."}
	"002" {set sense2 "Busy Printing Image (the image is being printed)."}
	"003" {set sense2 "Lower Print Head Failure."}
	"004" {set sense2 "Raise Print Head Failure."}
	"005" {set sense2 "Position Printer Drawer Home (out) Failure."}
	"006" {set sense2 "Position Printer Drawer In Failure."}
	"007" {set sense2 "Position Printer Drawer Out Failure."}
	"008" {set sense2 "Corrupt/Invalid Data in Command."}
	"009" {set sense2 "Data Mismatch."}
	"010" {set sense2 "Left Margin Length Invalid."}
	"011" {set sense2 "No Margin Specified When RLE is Disabled."}
	"012" {set sense2 "CD Detection - A print command was received without a disc being present."}
	"013" {set sense2 "Color change Error - The printer is unable to detect the requested ribbon Color."}
	default { set sense2 "Wrong Sense." }
    }

    Debug  "Codes: $sense1 --- $sense2" 3
    return "$sense1 --- $sense2"
}

#####  Move arm commands ###

#Get a disk from magazine 1
proc getBlank {} {
    global gvar 
    
    Log "getBlank"
    
    set result [ positionMedium $gvar(blankAddress) $gvar(takeMedia) ]
    if {$result != ""} { 
	switch -glob $result {
	    "Medium Error.*"    {
		puts "Magazine 1 is empty. Nothing to do."
		puts "Exit."
		exit 2
	    }
	    "Unit Attention.*" {
		if {[waitForOk]} {
		    waitForFixError "Door open or robot $gvar(ttyTail) powered off"
		}
	    }
	} 
	getBlank
    }
}

#Get a disk from magazine 2
proc getWritten {} {
    global gvar 

    Log "getWritten" 
    set result [ positionMedium  $gvar(writtenAddress) $gvar(takeMedia) ]
    if {$result != ""} { 
	switch -glob $result {
	    "Medium Error.*"    {waitForFixError "Written magazine of robot $gvar(ttyTail) is empty"}
	    "Unit Attention.*" {
		if {[waitForOk]} {
		    waitForFixError "Door open or robot $gvar(ttyTail) powered off"
		}
	    }
	} 
	getWritten
    }
}

#Get a disk from magazine 3
proc getBad {} {
    global gvar 

    Log "getBad"

    set result [ positionMedium $gvar(badAddress) $gvar(takeMedia) ]
    if {$result != ""} { 
	switch -glob $result {
	    "Medium Error.*"    {waitForFixError "Bad magazine of robot $gvar(ttyTail) is empty"}
	    "Unit Attention.*" {
		if {[waitForOk]} {
		    waitForFixError "Door open or robot $gvar(ttyTail) powered off"
		}
	    }
	} 
	getBad
    }
}

#Take a CD from the CD writer.
proc getMedia {} {
    global gvar 
    
    Log "getMedia"
    set result [ positionMedium $gvar(cdAddress) $gvar(takeMedia) ]
    
    if {$result != ""} { 
	if {![info exists gvar(id)]} { return 1 }
	if {[waitForOk]} {
	    waitForFixError "Error getting CD from writer of robot $gvar(ttyTail)"
	}
	getMedia
    }
}

#Get a disk from a predefined "input magazine" (can be 1 or 2)
#It's used in the efficient mode, where the input can be magazine 1 or 2
proc getInputCd {} {
    global gvar

    
    set gvar(inputMagazine) [read_file -nonewline $gvar(reqDir)/currentMagazine-$gvar(ttyTail).romanic]
    
    switch $gvar(inputMagazine) {
	"1" {set result [positionMedium $gvar(blankAddress) $gvar(takeMedia)]
	    set gvar(outputMagazine) 3
	}
	"2" {set result [positionMedium $gvar(writtenAddress) $gvar(takeMedia)]
	    set gvar(outputMagazine) 1
	}
    }
    
    if {$result != ""} {
	if {$gvar(inputMagazine) == 1} { 
	    Log "Switch to second magazine"
	    set gvar(inputMagazine)  2
	    set gvar(outputMagazine) 1
	    write_file $gvar(reqDir)/currentMagazine-$gvar(ttyTail).romanic $gvar(inputMagazine)
	    getInputCd 
	} else {
	    switch -glob $result {
		"Medium Error.*"    {waitForFixError "Magazine 2\nOf robot $gvar(ttyTail) is empty.\nInsert new CDs in magazine\n1 AND 2"}
		"Unit Attention.*" {
		    if {[waitForOk]} {
			waitForFixError "Door open or robot $gvar(ttyTail) powered off"
		    }
		    
		}
	    }
	    set gvar(inputMagazine)  1
	    set gvar(outputMagazine) 3
	    write_file $gvar(reqDir)/currentMagazine-$gvar(ttyTail).romanic $gvar(inputMagazine)
	    getInputCd
	}
    }
}

#Put a disk in the output magazine (can be 3 or 1). Used in efficient mode.
proc putOutputCd {} {
    global gvar

    Log "Put CD to magazine: $gvar(outputMagazine)"
    
    switch $gvar(outputMagazine) {
	"3" {set result [positionMedium $gvar(badAddress) $gvar(putMedia)]}
	"1" {set result [positionMedium $gvar(blankAddress) $gvar(putMedia)]}
    }
    
	if {$result != ""} {
	    Log "Error putting CD to magazine $gvar(outputMagazine)"
	    Log "Reason: $result"
    	if {[waitForOk]} {
	    waitForFixError "Error putting CD to magazine $gvar(outputMagazine)\nof robot $gvar(ttyTail)"
	}
	putOutputCd
    }
}

#Put a CD in a CD writer slot.
proc putMedia {} {
    global gvar 
    
    Log "put CD to writer $gvar(cdAddress)"
    set result [ positionMedium $gvar(cdAddress) $gvar(putMedia) ]
    
    if {$result != ""} { 
	if {[waitForOk]} {
	    waitForFixError "Error putting CD to writer of robot $gvar(ttyTail)"
	}
	putMedia
    }
}

#Reset a specific CD writer.
proc resetCD {device} {
    
    Log "Reset CD: $device"
    catch {exec cdrecord dev=$device -reset}
}

#Open the CD tray.
proc ejectCD {device} {
    
    Log "Eject CD: $device"
    catch {exec cdrecord dev=$device -eject}
}

#Close the CD tray.
proc loadCD {device} {

    Log "Load CD: $device"
    catch {exec cdrecord dev=$device -load} output
 
    if {[string match "*cdrecord: No disk / Wrong disk!*" $output] ||
	[string match "*cdrecord: CD/DVD-Recorder not ready*" $output]} {
	puts "Writer tray empty ($device)"
	return "error"
    } else {
	puts "Disk in the tray ($device)"
	return "ok"
    }
}

#Execute cdrecord.
proc writeCD {id} {
    global gvar

    Log "writeCD: $gvar($id-writerAddress)"
    set cdrecordOptions "-v dev=$gvar($id-writerAddress) speed=$gvar($id-writerSpeed)"
    if {$gvar($id-dummyMode)} {
	append cdrecordOptions " -dummy"
    }
    
    Log "Execute: cdrecord $cdrecordOptions $gvar($id-volumeFile).vol*"
    #puts "Volume file: $gvar($id-volumeFile)"
    if {[catch { eval "exec sh -c \"cdrecord $cdrecordOptions $gvar($id-volumeFile).vol* > $gvar(reqDir)/cdrecord-$id.romanic\" &"} msg ]} {
	Log "Error writing cdrom: $msg" 
	close $logFile
	return "error"
    }
    
    set logFile [open "$gvar(reqDir)/cdrecord-$id.romanic" w+]
    set line [split [read $logFile]]
    while {[string match "*Track 01: Total bytes read/written*" $line] == 0} { 
	
	set line [split [read $logFile]]
	if {[string match "*error*" $line] || 
	    [string match "*Illegal Request*" $line]} {
	    Log "Error writing the CD" 
	    close $logFile
	    return "error"}
	
	regsub -all "{" $line "" line
	regsub -all "}" $line "" line
	if {[lindex $line 0] != "Track"} { 
	    continue
	} else {
	    write_file $gvar(reqDir)/count-$id.romanic  [lrange $line 2 4]
	}
    }

    while {[string match "*Fixating time:*" $line] == 0} {
	if {[string match "*error*" $line]} {
	    Log "Error writing the CD" 
	    close $logFile
	    return "error"}
	set line [split [read $logFile]]
	after 2000
    }
    
    Log  "CD $gvar($id-volumeFile) written."
    return ""
}

######

#Put a CD in the magazine 1.
proc putBlank {} {
    global gvar 

    Log "putBlank"
    set result [ positionMedium $gvar(blankAddress) $gvar(putMedia) ]
    
    if {$result != ""} { 
	Log "Error putting CD to blank magazine of $gvar(tty) robot." 
	Log "Reason: $result" 
    	if {[waitForOk]} {
	    waitForFixError "Error putting CD to blank magazine\nof robot $gvar(ttyTail)"
	}
	putBlank
    }

}

#Put a CD in magazine 2
proc putWritten {} {
    global gvar 

    Log "putWritten"
    set result [ positionMedium $gvar(writtenAddress) $gvar(putMedia) ]

    if {$result != ""} { 
	Log "Error putting CD to written magazine of $gvar(tty) robot."
	Log "Reason: $result"
	if {[waitForOk]} {
	    waitForFixError "Error putting CD to written magazine\nof robot $gvar(ttyTail)."
	}
	putWritten
    }
}

#Put a CD in magazine 3
proc putBad {} {
    global gvar 

    Log "putBad"
    set result [ positionMedium $gvar(badAddress) $gvar(putMedia) ]
    
    if {$result != ""} { 
	Log "Error putting CD to bad magazine of $gvar(tty) robot."
	Log "Reason: $result"
	if {[waitForOk]} {
	    waitForFixError "Error putting CD to bad magazine\nof robot $gvar(ttyTail)."
	}
	putBad
    }
}

#Take a CD from the printer drawer.
proc getPrinter {} {
    global gvar 
    
    Log "getPrinter"
    set result [ positionMedium $gvar(printerAddress) $gvar(takeMedia) ]
    
    if {$result != ""} { 
	if {![info exists gvar(id)]} { return 1 }
	
	set inc 0
	while {$inc < 3 && $result != "" } {
	    incr inc
	    getPrinter
	    set result [ positionMedium $gvar(printerAddress) $gvar(putMedia) ]
	}
        
	if {$inc == 3} {
	    Log "Error taking CD from printer of $gvar(tty) robot."
	    Log "Reason: $result"
	    waitForFixError "Error taking CD from printer\nof robot $gvar(tty)."
	    getPrinter
	}
    }
}

#Put a CD on the printer drawer.
proc putPrinter {} {
    global gvar 

    Log "putPrinter" 
    set result [ positionMedium $gvar(printerAddress) $gvar(putMedia) ]
    if {$result != ""} { 
	set inc 0
	while {$inc < 3 && $result != "" } {
	    incr inc
	    getPrinter
	    set result [ positionMedium $gvar(printerAddress) $gvar(putMedia) ]
	}
        
	if {$inc == 3} {
	    Log "Error putting CD to printer of $gvar(tty) robot."
	    Log "Reason: $result"
	    waitForFixError "Error putting CD to printer\nof robot $gvar(ttyTail)."
	    putPrinter
	    
	}
    }
}

#print a label.
proc printLabel {type volumeName} {
    global gvar 
    

    if {$type == "goodLabel"} { 
	set label [findVLTFile bitmaps/good-cd.ps]
    } else {
	set label [findVLTFile bitmaps/bad-cd.ps]
    }

    set psFile  [open $label r]
    set outFile [open "$gvar(reqDir)/cdLabel-$volumeName.ps" w]
    set userName $volumeName
    set date [clock format [clock seconds] -format "%d-%b-%Y"]
    
    while {[gets $psFile line] > -1 } { 
        if {[string match "*CD-LABEL-HERE*" $line]} {
	    lvarpop line 3 ($userName)
	}
	
	if {[string match "*DD-MMM-YYYY*" $line]} {
	    lvarpop line 3 ($date)
	}
	
	puts $outFile $line
    }

    close $psFile
    close $outFile
    
    catch {exec ps2pcl $gvar(reqDir)/cdLabel-$volumeName.ps $gvar(reqDir)/cdLabel-$volumeName.rim}

    if {$gvar(automatic) == 0} {
	puts "Open drawer insert a medium and press Return"
	gets stdin
    	puts "Printing $gvar(volumeName)..."
    }
    
    Log "Printing... exec cat $gvar(reqDir)/cdLabel-$volumeName.rim > $gvar(tty)"
    catch {exec cat $gvar(reqDir)/cdLabel-$volumeName.rim > $gvar(tty)}
    after 5000
    file delete $gvar(reqDir)/cdLabel-$volumeName.rim 
    file delete $gvar(reqDir)/cdLabel-$volumeName.ps
}


#Put the arm on top. It's kind of reset.
proc armOnTop {} {
    global gvar 

    Log "Arm on top"
    set result [ positionMedium $gvar(printerAddress) $gvar(noAction) ]
    
    if {$result != ""} { 
	Log "Error putting arm on top of $gvar(tty) robot."
	Log "Reason: $result"
	if {[waitForOk]} {
	    waitForFixError "Error putting arm on top of robot $gvar(ttyTail)"
	}
	armOnTop
    }
}

#Move the arm to a specific position. 
proc positionMedium {address action} {
    global gvar
    

    Debug "Position medium: $gvar(tty) $address $action" 1
    set commandString "$gvar(escape)$gvar(header1)$address$action$gvar(trail)"
    set expectedAnswer "\x1B\x21\x66\x31\x31\x43"
    
    sendMessage $commandString
    set status [readPort 6]
    if {$status != $expectedAnswer} {
	Debug "I expected $expectedAnswer" 1
	Debug "But got: $status" 1
	set requestSense [romanicRequestSense]
	Debug "And the sense is: $requestSense" 1
	return $requestSense
    }
    return ""
}

#Read the serial port. This operation MUST be done even if
#we don't care about the output, otherwise the root gets "confused"
proc readPort {bytes} {
    global gvar
    
    #set tty [open /dev/tty1p0 r]
    
    if {[catch {set buffer [read $gvar($gvar(tty)) $bytes]}]} {
	Log "An error occoured, while reading serial port: "
	after 3000
	readPort $bytes
    }	
    return $buffer
}

#Close all the channels.
proc romanicPilotQuit {} {
    global gvar
    
    foreach tty $gvar(ttyList) {
	close $gvar($tty)
    }
    exit
}

proc mountDisk {} {
    global gvar
    
    set opSystem [exec uname]
    if {$opSystem == "HP-UX"} {
	if {[catch {exec pfs_mount $gvar(mountPoint)} msg]} {
	    puts "Cannot mount media under $gvar(mountPoint) : $msg"
	    exit 1
	}
    } else {
	#Fake the mount on Solaris.
	#Operations on Solaris will be implemented later.
    }
}


proc unmountDisk {} {
    global gvar
    
    set opSystem [exec uname]
    if {$opSystem == "HP-UX"} {
	if {[catch {exec pfs_umount $gvar(mountPoint)} msg]} {
	    puts "Cannot un-mount media from $gvar(mountPoint) : $msg"
	    exit 1
	}
    } else {
	#Fake the mount on Solaris.
	#Operations on Solaris will be implemented later.
    }
}


proc getCDname {} {
    global gvar 

    set opSystem [exec uname]
    if {$opSystem == "HP-UX"} {

	set deviceName [getDeviceName $gvar(mountPoint)]
	puts "Device name: $deviceName"

	if {$deviceName == "error"} {
	    return "error"
	}
    } else {
	set deviceName "/dev/dsk/c1t3d0s2"
    }	

    set volumeName [getVolumeName $deviceName]
    return $volumeName
   
}

proc getVolumeName {deviceName} {
    
    if {[catch {set deviceFile [open $deviceName r]}]} {
	puts "Cannot open $deviceName"
	return "error"
    }

    if {[catch {seek $deviceFile 0100050 start}]} {
	puts "Cannot seek medium"
	close $deviceFile
	return "error"
    }

    if {[catch {set volumeName [read $deviceFile 32]}]} {
	puts "Cannot read volume name"
	close $deviceFile
	return "error"
    }
        
    if {$volumeName == ""} {
	puts "Volume name not valid"
	close $deviceFile
	return "error"
    } else {
	set volumeName [string trimright $volumeName " "]
    }
    
    close $deviceFile
    return $volumeName
}


proc getDeviceName {mountPoint} {

    if {[catch {set mnttabFile [open /etc/mnttab r]}]} {
	puts "I cannot open /etc/mnttab"
	return "error"
    }
    
    set deviceName ""
    while {[gets $mnttabFile line] > 0} {
	if {[string match "*$mountPoint*" $line]} {
	    set deviceName [lindex $line 0]
	}
    }
    
    close $mnttabFile
    if {$deviceName == ""} {
	return "error"
    } else {
	return $deviceName
    }
}

#Parse the command line options and set some variables.
proc romanicSetOptions {} {
    global gvar
    
    regsub -all "{" $gvar(args) "" gvar(args)
    regsub -all "}" $gvar(args) "" gvar(args)
    
    for { set i 0} {$i <[llength $gvar(args)]} {incr i} {

        switch -- [lindex $gvar(args) $i] {
	    -info {
		explainAll
	    } 
	    -debug {
		incr i
		set level [lindex $gvar(args) $i]
		if {$level == "" || $level > 3 || $level < 1} {
		    explain
		}
		set gvar(debugMode) $level
	    }
	    -log {
		incr i
		set logFile [lindex $gvar(args) $i]
		if {[catch {set gvar(logFile) [open $logFile a]} var]} {
		    puts "Cannot open $logFile"
		    exit }
		if {![info exists gvar(debugMode)]} { set gvar(debugMode) 1 }
	    } 
	    -automatic {
		set gvar(automatic) 1
	    }
	    -numMedia {
		incr i
		set gvar(numMedia) [lindex $gvar(args) $i]
	    }
	    -mountPoint {
		incr i
		set gvar(mountPoint) [lindex $gvar(args) $i]
	    }
	    -volumeName {
		incr i
		set gvar(volumeName) [lindex $gvar(args) $i]
	    }
	    -portSerial {
		incr i
		set gvar(portSerial) [lindex $gvar(args) $i]
	    }
	    -speed {
		incr i
		set gvar(baud) [lindex $gvar(args) $i]
	    }
	    -portParallel {
		incr i
		set gvar(portParallel) [lindex $gvar(args) $i]
	    }
	    default {
		puts "Unknown option: [lindex $gvar(args) $i]"
		explain
	    }
	}
    }
    checkArguments
}

#Be sure that the options specified are correct.
proc checkArguments {} {
    global gvar

    if {$gvar(automatic) == 0} {
	if {![info exists gvar(volumeName)]} {
	    puts "You have to specify a volume name"
	    explain
	}
	if {![info exists gvar(portSerial)] && \
		![info exists gvar(portParallel)]} {
	    puts "You have to specify a serial or parallel port"
	    explain
	}
	
	if {[info exists gvar(portSerial)] &&
	    (![file exists $gvar(portSerial)] || \
		 ![file readable $gvar(portSerial)] || \
		 ![file writable $gvar(portSerial)]) } {
	    puts "Serial port $gvar(portSerial) doesn't exist or"
	    puts "it's not readable/writable"
	    exit 1
	} 
	
	if {[info exists gvar(portSerial)]} {
	    if {![info exists gvar(baud)]} {
		puts "You must specify the speed of the serial port."
		puts "Use the option -speed <speed>"
		exit 1
	    }
	    if {[checkBaud $gvar(baud)] == "error"} {
		puts "Speed $gvar(baud) not supported."
		puts "Valid values are:"
	        puts "1200 2400	4800 9600 19200	38400 57600 115200 230400" 
		exit 1
	    }
	}
	
	if {[info exists gvar(portParallel)] &&
	    (![file exists $gvar(portParallel)] || \
		 ![file readable $gvar(portParallel)] || \
		 ![file writable $gvar(portParallel)]) } {
	    puts "Serial port $gvar(portParallel) doesn't exist or"
	    puts "it's not readable/writable"
	    exit 1
	}
	if {[info exists gvar(portSerial)]} {
	    set gvar(tty) $gvar(portSerial)
	} else {
	    set gvar(tty) $gvar(portParallel)
	}
	
    } else {
	if {[info exists gvar(numMedia)]} {
	    if {$gvar(numMedia) <= 0} {
		puts "numMedia must be greater than 0"
		exit 1
	    }
	    if {$gvar(numMedia) > 60} {
		puts "numMedia must be less than 60"
		exit 1
	    }
	}
	if {![info exists gvar(mountPoint)]} {
	    puts "Specify a directory where you want to temporary mount your media"
	    explain
	} else {
	    if {![file exists $gvar(mountPoint)]} {
		puts "Directory $gvar(mountPoint) doesn't exist"
		exit 1
	    }
	}
    }
}

proc checkBaud {baud} {

    switch $baud   { 
	"1200"     { set returnCode "ok" }
	"2400"     { set returnCode "ok" }
	"4800"     { set returnCode "ok" }
	"9600"     { set returnCode "ok" }
	"19200"    { set returnCode "ok" }
	"38400"    { set returnCode "ok" }
	"57600"    { set returnCode "ok" }
	"115200"   { set returnCode "ok" }
	"230400"   { set returnCode "ok" }
	default    { set returnCode "error"}
    }

    return $returnCode
}



#I won't explain explain =;-)
proc explain {} {
    global gvar

    puts "Usage: printMedia -info\n-debug \[1-3\]\n-log <log file>\n\[-automatic -numMedia <num> -mountPoint <directory>\]\n-volumeName <volume name>\n\[-portSerial <port> -speed <speed>\] | -portParallel <port>"
    exit 1
}


proc explainAll {} {
    puts "-info                   : This message"
    puts "-log <filename>         : Puts all the messages in the specified file"
    puts "                        : If you don't specify -debug the verbose level"
    puts "                        : will be set to 1"
    puts "-debug \[1-3\]          : Define the verbose level"
    puts "-automatic              : Use the robot to pick up media, read the volume and print it."
    puts "-numMedia <number>      : Specify how many media you want to print"
    puts "                        : (automatic mode only)"
    puts "-mountPoint <directory> : Directory under which the medium will be mounted and read."
    puts "                        : (automatic mode only)"
    puts "-volumeName <name>      : Actual string to be printed on the medium"
    puts "-portSerial <port>      : Serial port to be used to communicate with the printer"
    puts "-speed <speed>          : Speed of the serial port"
    puts "                        : (used only when -portSerial is defined)"
    puts "-portParallel <port>    : Parallel port to be used to communicate with the printer"
    exit 1
}

#Loop looking for requests under the specified directory.
proc romanicGlobLoop { time cmd } {
    global gvar
    
    if {$gvar(cancelGlobLoop) == 1} {
	set gvar(cancelGlobLoop) 0
	after cancel $gvar(globLoopId)
	Debug  "Loop - $gvar(globLoopId) - $cmd - killed " 1
    } else {
	eval $cmd
	set gvar(globLoopId) [after $time romanicGlobLoop $time \"$cmd\" ]
	Debug "LoopID = $gvar(globLoopId) -- cmd = $cmd" 3
    }
}

#Serve requests in queue.
proc romanicServeRequestsLoop { time cmd } {
    global gvar
    
    if {$gvar(cancelServeRequestsLoop) == 1} {
	set gvar(cancelServeRequestsLoop) 0
	after cancel $gvar(serveRequestsLoopId)
	Debug " Loop - $gvar(serveRequestsLoopId) - $cmd - killed " 1
    } else {
	eval $cmd
	set gvar(serveRequestsLoopId) [after $time romanicServeRequestsLoop $time \"$cmd\" ]
	Debug "LoopID = $gvar(serveRequestsLoopId) -- cmd = $cmd" 3
    }
}

#Create a child process to serve two requests in parallel.
proc forkProcess {command} {
    global gvar

    signal default SIGCHLD
    Debug  "Fork process with command: $command" 1
    set pid [ fork ]
    Debug "fork id: $pid" 2
    set gvar(pid) $pid
    
    # This is the father
    if { $pid != 0} {
	set getSignal [signal get SIGCHLD]
	#puts "Status of my signal for $command is: $getSignal"
        Debug "I'm the father. Return" 2
	signal trap SIGCHLD reaper
	return
    } else {
    	# This is the son
    	Debug "I'm the son, execute: eval $command" 2
	set gvar(cancelServeRequestsLoop) 1
	set gvar(cancelGlobLoop) 1
	eval $command
	#initializeSerials
	# set txt "/dev/tty1p0"
# 	puts "initialize $txt"
# 	set getSignal [signal get SIGCHLD]
# 	puts "Status of my signal for $command is: $getSignal"
# 	exec stty raw 115200 cs8 cread crts clocal rtsxoff ctsxon < $txt
	exit
    }
}

#Used by fork procedure.
proc reaper {} {
    wait -nohang
    #signal trap SIGCHLD reaper
}


#Lock the arm. Other arm operations are frozen.
proc lockArm {tty} {
    global gvar 

    write_file $gvar(reqDir)/$tty-$gvar(id).lock ""
}

#Unlock the arm.
proc releaseArm {tty} {
    global gvar

    file delete $gvar(reqDir)/$tty-$gvar(id).lock
    file delete $gvar(reqDir)/$tty-$gvar(id).mylock
}

#Wait for the arm to be unlocked.
proc justWait {tty} {
    global gvar
    
#    while {[file exists $gvar(reqDir)/$tty.lock]}#
    while {[glob -nocomplain $gvar(reqDir)/$tty-*.lock] != ""} {
	if {![file exists $gvar(reqDir)/request-$gvar(id).romanic]} {
	    Log "Request cancelled"
	    cleanUpQueue $gvar(id)
	    exit 1
	}
	after 3000}
}

#Write the operation to be executed to a log file.
proc Log {txt} {
    global gvar 
    
    if {![info exists gvar(id)]} { return }
    set file [open $gvar(reqDir)/log-$gvar(id)-[file tail $gvar(ttyList)].romanic a+]
    puts $file "$txt \[$gvar(tty)\]"
    close $file
    if {[info exists gvar(debugMode)]} { Debug "$txt \[$gvar(tty)\]" 1 }


#### Temporary log procedure
    
    set file [open $gvar(reqDir)/log-[file tail $gvar(ttyList)].romanic a+]
    puts $file "$txt \[$gvar(tty)\]"
    close $file
    if {[info exists gvar(debugMode)]} { Debug "$txt \[$gvar(tty)\]" 1 }

}

#Print debug messages.
proc Debug {text level} {
    global gvar

    #if {![info exists gvar(HtVerbose)] || $gvar(HtVerbose) == "0" } { return }
    if {![info exists gvar(debugMode)]} { return }

    if { $gvar(debugMode) <  $level } { return }
    set date [clock format [clock seconds] -format "%Y-%m-%d %T"]

    if {[info exists gvar(logFile)]} {puts $gvar(logFile) "Debug>\[$date\] $text"
    } else { puts "Debug>\[$date\] $text" }
}

proc findVLTFile { file { confirm "yes" } } {
    global env
     
    #
    # Take the only file name and check in the current directory
    #
    set myFile  [file tail $file] 
    set myDir   [pwd]

    if {[file exist $myFile]} { return  ${myDir}/${myFile} }

    #
    # Does the file exist in the current sub-directory?
    #
    
    set fileName ${myDir}/$file
    if {[file exist $fileName]} { return  $fileName }
  
    #
    # Does the file exist in the current module?
    # (the current module is assumed to be the parent directory 
    #  of the current pwd)
    
    set fileName ../$file
    if {[file exist $fileName]} { return  $fileName }
    #
    # Does the file exist in the current integration area, if any?
    #
    if {[info exists env(INTROOT)]} {
        set fileName $env(INTROOT)/$file
        if {[file exist $fileName]} { return  $fileName }
    }
    #
    # Does the file exist in the current VLT data area, if any?
    #
    if {[info exists env(VLTDATA)]} {
        set fileName $env(VLTDATA)/$file
        if {[file exist $fileName]} { return  $fileName }
    }
    #
    # Does the file exist in the current VLT root area, if any?
    #
    if {[info exists env(VLTROOT)]} {
        set fileName $env(VLTROOT)/$file
        if {[file exist $fileName]} { return  $fileName }
    }

    #
    # File does not exist.
    #
    if { $confirm == "yes" } {
       set tmp "File  $myFile  NOT FOUND"
       puts $tmp
    }

    return ""
}

proc waitForFixError {msg} {
    
    puts $msg
    exit 1

}


##########################################
