package org.eso.dfs.gui;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

import org.eso.dfs.util.SharedList;

/**
 * @author Nick Kornweibel
 * 
 * Test the RecallingComboBox widget and associated models.
 * Create a panel with a vertical column of 4 combo boxes and a display field showing
 * the latest selection from any of the four combo boxes.
 * The four combo boxes are:
 * RecallingComboBox
 * RecallingComboBox with a sorted list
 * RecallingComboBox with a shared list
 * RecallingComboBox with a shared list - sharing list with third combo box.
 * 
 * To Run:
 * java org.eso.dfs.gui.RecallingComboBoxDemo
 */

public class RecallingComboBoxDemo extends JPanel implements ActionListener {

    private JLabel result;

    public RecallingComboBoxDemo() {
        setLayout(new BoxLayout(this, BoxLayout.PAGE_AXIS));

        JComboBox comboBox1 = new RecallingComboBox(false, 10);
        JComboBox comboBox2 = new RecallingComboBox(true, 5);
        comboBox1.addActionListener(this);
        comboBox2.addActionListener(this);
        
        SharedList sharedList = new SharedList();
        sharedList.setMaxCapacity(10);
        sharedList.setSortList(true);
        JComboBox comboBox3 = new RecallingComboBox(new SharedListComboBoxModel(sharedList));
        JComboBox comboBox4 = new RecallingComboBox(new SharedListComboBoxModel(sharedList));        
        comboBox3.addActionListener(this);
        comboBox4.addActionListener(this);

        // Create the UI for displaying result.
        JLabel resultLabel = new JLabel("Current Selection", JLabel.LEADING);
        //== LEFT
        result = new JLabel(" ");
        result.setForeground(Color.black);
        result.setBorder(
            BorderFactory.createCompoundBorder(
                BorderFactory.createLineBorder(Color.black),
                BorderFactory.createEmptyBorder(5, 5, 5, 5)));

        // Lay out everything.
        JPanel panel = new JPanel();
        panel.setLayout(
            new BoxLayout(panel, BoxLayout.PAGE_AXIS));
        panel.add(new JLabel("Enter something new or select one from the list:"));

        // lazy coding to add a seperator.
        panel.add(new JLabel("         "));
        
        panel.add(new JLabel("Standard RecallingComboBox (max capacity 10):"));
        comboBox1.setAlignmentX(Component.LEFT_ALIGNMENT);
        panel.add(comboBox1);

        // lazy coding to add a seperator.
        panel.add(new JLabel("         "));        
        panel.add(new JLabel("Sorted RecallingComboBox (max capacity 5):"));
        comboBox2.setAlignmentX(Component.LEFT_ALIGNMENT);
        panel.add(comboBox2);
        
        // lazy coding to add a seperator.
        panel.add(new JLabel("         "));
        panel.add(new JLabel("Shared, sorted RecallingComboBox (1 of 2):"));
        comboBox3.setAlignmentX(Component.LEFT_ALIGNMENT);
        panel.add(comboBox3);
        
        // lazy coding to add a seperator.
        panel.add(new JLabel("         "));
        panel.add(new JLabel("Shared, sorted RecallingComboBox (2 of 2):"));
        comboBox4.setAlignmentX(Component.LEFT_ALIGNMENT);
        panel.add(comboBox4);

        JPanel resultPanel = new JPanel(new GridLayout(0, 1));
        resultPanel.add(resultLabel);
        resultPanel.add(result);

        panel.setAlignmentX(Component.LEFT_ALIGNMENT);
        resultPanel.setAlignmentX(Component.LEFT_ALIGNMENT);

        add(panel);
        add(Box.createRigidArea(new Dimension(0, 10)));
        add(resultPanel);

        setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));

        newEntry(" ");
    }

    public void actionPerformed(ActionEvent e) {
        JComboBox cb = (JComboBox) e.getSource();
        // Due to multiple events being fired for editable comboboxes, the following
        // check is made:
        // If the selected index is -1, the event does not indicate the end
        // of an item selection....so do nothing until a valid item is selected.
        if (cb.getSelectedIndex() > -1) {
            String newSelection = cb.getSelectedItem().toString();
            newEntry(newSelection);
        }
    }

    /** 
     * Formats and displays today's date. 
     */
    public void newEntry(String entry) {
        result.setText(entry);
    }

    /**
     * Create the GUI and show it.  For thread safety,
     * this method should be invoked from the
     * event-dispatching thread.
     */
    private static void createAndShowGUI() {
        // Make sure we have nice window decorations.
        JFrame.setDefaultLookAndFeelDecorated(true);

        // Create and set up the window.
        JFrame frame = new JFrame("RecallingComboBoxDemo");
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        // Create and set up the content pane.
        JComponent newContentPane = new RecallingComboBoxDemo();
        //newContentPane.setOpaque(true); //content panes must be opaque
        frame.setContentPane(newContentPane);

        // Display the window.
        frame.pack();
        frame.setVisible(true);
    }

    public static void main(String[] args) {
        // Schedule a job for the event-dispatching thread:
        // creating and showing this application's GUI.
        javax.swing.SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                createAndShowGUI();
            }
        });
    }
}
