package org.eso.dfs.gui;


import java.io.File;

import javax.swing.JFileChooser;
import javax.swing.plaf.FileChooserUI;
import javax.swing.plaf.basic.BasicFileChooserUI;
/**
 * 
 * @author Nick Kornweibel
 * 
 * A SwingFileChooser wraps a JFileChooser. 
 * This class had to written to get around the multiple file selection problem in Swing.
 * If and when org.eso.ohs.core is combined with this package (org.eso.dfs) then
 * this class can be merged with the SwingFileChooser from OHS core - it is essentially
 * a complete subset.
 *
 */
public class SwingFileChooser extends JFileChooser {

    /**
     * Default Constructor
     */

    public SwingFileChooser() {
        super();
    }

    /**
     * Constructs a file chooser given a certaing directory
     * @param directory Directory for file Chooser
     */
    public SwingFileChooser(String directory) {
        super(directory);  
    }

    /**
     * Constructs a file chooser given a certain path
     * @param directory Directory for file Chooser
     */
    public SwingFileChooser(File path) {
        super(path);  
    }

    /**
     * Set the current Directory.
     * This had to be overridden to get around another bug in Swing.
     * in Directory only selection mode.
     * @param file Set the current directory
     */

    public void setCurrentDirectory(File file) {
        // Let the action take place, then override the 'Name' field.
        // This is equivalent to the user typing "." in the name field.
        // stdlog_.debug("CLASS SwingFileChooser - setCurrentDirectory: File = "+file);
        super.setCurrentDirectory(file);
        if (isDirectorySelectionEnabled()) {
            setFilename(".");
        }
    }  
    
    /**
     * Set filename
     * @param filename
     */

    public void setFilename(String filename) {
        FileChooserUI ui = getUI();
        if (ui instanceof BasicFileChooserUI) {
            ((BasicFileChooserUI) ui).setFileName(filename);
        }
    }
}
