/**
 * These classes have been developed for the DFSLog system.
 *
 * @author N. Kornweibel nkornwei@eso.org
 *
 * TO DO:
 * need @revision, and SNIFF info here.
 * 
 */
package org.eso.dfslog;


import java.io.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

// JDBC
import java.sql.*;

/**
 * RetryDialog:
 * 
 * A simple dialog to display a countdown and perform periodic connection
 * attempts on the ConnectionServer passed in the constructor.
 * If the user selects quit (the only option other than waiting for
 * a successful connection), the calling application may determine this
 * by calling quitSelected().
 */

public class RetryDialog extends JDialog implements ActionListener {

    private ConnectionServer dbServer;
    private int updateInterval = 5000;
    private Timer retryTimer;
    private int retryCounter;
    private JLabel retryField;
    private JLabel contactField;
    private JButton quitButton;
    private boolean quit = false;

    public RetryDialog(Frame frm, String title, boolean modal,
            ConnectionServer m) {
        super(frm, title, modal);
        dbServer = m;
        retryTimer = new Timer(updateInterval, this);
        retryTimer.start();
        retryCounter = 0;
        setupDialog();
        retryField.setText("");
        contactField.setText("Contacting server...");
    }




    public boolean quitSelected() {
        return quit;
    }




    public void actionPerformed(ActionEvent a) {
        if (a.getSource() == retryTimer) {
            // normally stop/restart the timer here while the event is
            // processed, however it is done in retryUpdate().
            retryUpdate();
        } else if (a.getSource() == quitButton) {
            quit = true;
            setVisible(false);
            retryTimer.stop();
            dispose();
            return;
        }
    }




    private void retryUpdate() {
        if (retryCounter == 0) {
            retryField.setText("");
            contactField.setText("Contacting server...");
            try {
                retryTimer.stop();
                dbServer.connectToServer();
            } catch (SQLException sqlx) {
                contactField.setText(
                    "Contacting server...failed.");
                // System.out.println("Connection Failed:"+sqlx.toString());
            }
            if (dbServer.isValid()) {
                setVisible(false);
                retryTimer.stop();
                dispose();
                return;
            }
            retryTimer.restart();
            retryCounter = 6;
        }
        retryField.setText("Retry in "+(5*retryCounter)+" seconds");
        retryCounter--;
    }




    void setupDialog() {
        retryField = new JLabel();
        contactField = new JLabel();
        quitButton = new JButton("exit");
        quitButton.addActionListener(this);
        GridLayout g = new GridLayout(3,1);
        JPanel p = new JPanel(g);
        g.setHgap(5);
        p.add(contactField);
        p.add(retryField);
        getContentPane().add(p, "Center");
        getContentPane().add(quitButton, "South");
    }
}
// end of file RetryDialog.java.
