/**
 * These classes have been developed for the DFSLog system.
 *
 * @author N. Kornweibel nkornwei@eso.org
 *
 * TO DO:
 * need @revision, and SNIFF info here.
 * 
 */
package org.eso.dfslog;


import java.io.*;
import java.awt.*;
import javax.swing.*;
import java.util.*;
import java.awt.event.*;


/**
 * SimplePropertyEditor:
 *
 * The SimplePropertyEditor presents a JDialog for editing the set of
 * properties passed in the constructor.
 * The "ok" button indicates that the changes should be acknowledged,
 * while "cancel" indicates that the changes should be ignored.
 */

public class SimplePropertyEditor extends JDialog implements ActionListener {

    private JButton cancelButton = null;
    private JButton okButton = null;
    private JTextField[] fields = null;
    private String[] keys = null;
    private Properties props;
    private int nKeys = 0;
    private boolean okSel = false;

    public SimplePropertyEditor(Frame frm, String title, 
            boolean modal, Properties p){
        super(frm, title, modal);
        props = p;
        setupDialog();
    }



    public Properties getProperties() {
        return props;
    }



    public boolean okSelected() {
        return okSel;
    }

    private void setupDialog() {
        nKeys = props.size();
       
        GridLayout g = new GridLayout(nKeys, 2);
        g.setHgap(5);
        g.setVgap(5);
        JPanel p = new JPanel(g);
        boolean putLabel = true;
        
        fields = new JTextField[nKeys];
        keys = new String[nKeys];
        Enumeration en = props.propertyNames();

        for (int i = 0; i < nKeys*2; i++) {
            if (putLabel) {
                try {
                    keys[i/2] = (String)en.nextElement();
                } catch (NoSuchElementException e) {
                    keys[i/2] = null;
                    break;
                }
                p.add(new JLabel(keys[i/2], JLabel.RIGHT));
                putLabel = false;
            } else if (keys[i/2] != null) {
                JTextField f = new JTextField(
                    props.getProperty(keys[i/2]));
                fields[i/2] = f;
                p.add(f);
                putLabel = true;
            } else {
                fields[i/2] = null;
                putLabel = true;
            }
        } // end of for loop.
        getContentPane().add(p, "Center");
        JPanel p1 = new JPanel(new FlowLayout());
         
        okButton = new JButton("ok");
        okButton.addActionListener(this);
        p1.add(okButton);
        cancelButton = new JButton("cancel");
        cancelButton.addActionListener(this);
        p1.add(cancelButton);
        getContentPane().add(p1, "South");
    } // end of setupDialog().


    // implement the ActionListener interface.
    public void actionPerformed(ActionEvent a) {
        if (a.getSource() == cancelButton) {
            // do nothing - we dispose() of the window later.
            setVisible(false);
            dispose();
        }
        else if (a.getSource() == okButton) {
            // save the edited fields to the property.
            okSel = true;
            props.clear();
            for (int i = 0; i < nKeys; i++) {
                if (keys[i] != null) {
                    try {
                        props.put(keys[i], fields[i].getText());
                    } catch (NullPointerException e) {
                        System.out.println(
                            "Attempt to enter null property key");
                    }
                }
            }

            setVisible(false);
            dispose();
        }
    } // end of actionPerformed().

} // end of class SimplePropertyEditor

// end of file SimplePropertyEditor.java
