/**
 * @author Michele Zamparelli
 * @version 1.0
 * A JTree widget for displaying a DBTreeModel object.
 * 
 * TODO: DFSTreeTable, ResultsTreeTable and DBTreeTable all have 
 * a CustomListener - these should be extracted out to a common type.
 */

package org.eso.gasgano;

import java.awt.Component;
import java.awt.Container;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.MouseEvent;
import java.util.Enumeration;
import java.util.Vector;

import javax.swing.JFrame;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JTree;
import javax.swing.SwingUtilities;
import javax.swing.event.MouseInputAdapter;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

import org.eso.gasgano.datamodel.database.DBContainer;
import org.eso.gasgano.datamodel.gui.DBOBComponent;
import org.eso.gasgano.datamodel.gui.FitsFileOBComponent;
import org.eso.gasgano.gui.DBTreeTableModel;
import org.eso.gasgano.gui.DFSFileTreeCellRenderer;
import org.eso.gasgano.gui.JTreeTable;
import org.eso.gasgano.gui.JTreeTable.ListToTreeSelectionModelWrapper;
import org.eso.gasgano.properties.PropertyDB;
import org.eso.gasgano.tools.Exec;

public class DBTreeTable extends JTreeTable implements ActionListener {

    private Vector selectionVector = new Vector();
    protected String selectedDir = null;
    private JPopupMenu filePopUpMenu = null;
    private JPopupMenu containerPopUpMenu = null;
    private Object popUpDBComponent = null;
    private GasganoDisplay mainDisplay = null;
    // some labels
    private String retrieveLabel = "Retrieve file(s)";

    public DBTreeTable(DBTreeTableModel model, GasganoDisplay mainDisplay) {
        super(model);
        getTree().getSelectionModel().setSelectionMode(
            TreeSelectionModel.DISCONTIGUOUS_TREE_SELECTION);
        getTree().setCellRenderer(new DFSFileTreeCellRenderer());
        getTree().addTreeExpansionListener(model);
        model.setTree(getTree());
        setIntercellSpacing(new Dimension(5, 1));
        setColumnWidths();
        setupPopUpMenu();
        this.mainDisplay = mainDisplay;
    }

    public void setColumnWidths() {
        TableColumnModel tcm = getColumnModel();
        TableColumn tCol = tcm.getColumn(0);
        tCol.setMinWidth(370);
    }

    /**
     * @return Return a Vector of String of paths names of the 
     * selected files - or an empty Vector is nothing is selected.
     */
    public Vector getSelectedFiles() {
        Vector rc = selectionVector;
        rc.removeAllElements();
        TreePath paths[] = getTree().getSelectionPaths();

        if (paths != null) {
            for (int i = 0; i < paths.length; i++) {
                Object obj = paths[i].getLastPathComponent();
                if (obj instanceof DBOBComponent) {
                    DBOBComponent dbobc = (DBOBComponent) obj;
                    rc.addElement(dbobc.getId());
                }
            }
        }
        return rc;
    }

    private void setupPopUpMenu() {
        filePopUpMenu = new JPopupMenu();
        containerPopUpMenu = new JPopupMenu();

        JMenuItem item = null;
        filePopUpMenu.add(
            item =
                new JMenuItem(
                    PropertyDB.getInstance().getProperty("MENU_CUSTOM1_LBL")));
        item.addActionListener(this);
        filePopUpMenu.add(
            item =
                new JMenuItem(
                    PropertyDB.getInstance().getProperty("MENU_CUSTOM2_LBL")));
        item.addActionListener(this);
        filePopUpMenu.addSeparator();
        filePopUpMenu.add(item = new JMenuItem(retrieveLabel));
        item.addActionListener(this);

        containerPopUpMenu.add(
            item =
                new JMenuItem(
                    PropertyDB.getInstance().getProperty("MENU_CUSTOM3_LBL")));
        item.addActionListener(this);
        containerPopUpMenu.add(
            item =
                new JMenuItem(
                    PropertyDB.getInstance().getProperty("MENU_CUSTOM4_LBL")));
        item.addActionListener(this);
    }

    public Vector getSelectedOBComponents() {
        Vector rc = selectionVector;
        rc.removeAllElements();
        TreePath paths[] = getTree().getSelectionPaths();

        if (paths != null) {
            for (int i = 0; i < paths.length; i++) {
                Object obj = paths[i].getLastPathComponent();
                if (obj instanceof DBOBComponent) {
                    rc.addElement(obj);
                }
            }
        }
        return rc;
    }

    public String getSelectedInstrument() {
        Vector selVect = getSelectedOBComponents();
        String rc = null;
        int c = 0;
        for (int i = 0; i < selVect.size(); i++) {
            if (selVect.elementAt(i) instanceof FitsFileOBComponent) {
                FitsFileOBComponent ffoc =
                    (FitsFileOBComponent) selVect.elementAt(i);
                if (c == 0) {
                    rc = ffoc.getInstrument();
                    c++;
                } else {
                    c++;
                    if (ffoc.getInstrument() != null
                        && !rc.equals(ffoc.getInstrument())) {
                        rc = null;
                        return rc;
                    }
                }
            }
        }
        return rc;
    }

/*
    public class CustomListener extends MouseInputAdapter {
        Toolkit toolkit = null;
        Container contentPane = null;
        JFrame display = null;
        int yOffset = 0;
        private boolean debug = false;

        public CustomListener(
            Container contentPane,
            JFrame display,
            int yOffset) {
            toolkit = Toolkit.getDefaultToolkit();
            this.contentPane = contentPane;
            this.display = display;
            this.yOffset = yOffset;
        }

        public void setOffset(int yOffset) {
            this.yOffset = yOffset;
        }

        public void mouseMoved(MouseEvent e) {
            //System.out.print("Moved  ");
            redispatchMouseEvent(e);
        }

        public void mouseDragged(MouseEvent e) {
            // System.out.print("Dragged ");
            redispatchMouseEvent(e);
        }

        public void mouseClicked(MouseEvent e) {
            // System.out.print("Clicked ");
            redispatchMouseEvent(e);
        }

        public void mouseEntered(MouseEvent e) {
            // System.out.print("Entered\n");
            redispatchMouseEvent(e);
        }

        public void mouseExited(MouseEvent e) {
            redispatchMouseEvent(e);
        }

        public void mousePressed(MouseEvent e) {
            // System.out.print("Pressed ");
            redispatchMouseEvent(e);
        }

        public void mouseReleased(MouseEvent e) {
            // System.out.print("Released");
            redispatchMouseEvent(e);
        }

        void redispatchMouseEvent(MouseEvent e) {
            Component component = null;
            Point treeTablePoint = e.getPoint();
            int eventID = e.getID();
            int modifiers = e.getModifiers();

            treeTablePoint.y += yOffset;

            component =
                SwingUtilities.getDeepestComponentAt(
                    contentPane,
                    treeTablePoint.x,
                    treeTablePoint.y);

            //component = contentPane.getComponentAt(treeTablePoint.x, treeTablePoint.y  );
            if (component != null) {
                if (debug) {
                    System.out.println(
                        component.getClass().getName()
                            + " "
                            + treeTablePoint.x
                            + " "
                            + treeTablePoint.y);
                }
                Point componentPoint =
                    SwingUtilities.convertPoint(
                        contentPane,
                        treeTablePoint,
                        component);

                if (component instanceof JTreeTable
                    && eventID == MouseEvent.MOUSE_PRESSED) {
                    JTree tree = getTree();
                    ListToTreeSelectionModelWrapper selectionModel =
                        (ListToTreeSelectionModelWrapper) tree
                            .getSelectionModel();

                    // Java 1.1.8: int row = componentPoint.y / ( tree.getRowHeight() +1  ) ; 
                    // follow works better for Java 1.4:
                    int row = componentPoint.y / tree.getRowHeight();
                    DBTreeTableModel model = (DBTreeTableModel) tree.getModel();

                    // it is not a leaf
                    if (!tree
                        .getModel()
                        .isLeaf(
                            tree.getPathForRow(row).getLastPathComponent())) {
                        Cursor c = display.getCursor();
                        display.setCursor(new Cursor(Cursor.WAIT_CURSOR));
                        Toolkit toolkit = Toolkit.getDefaultToolkit();

                        // it's  a normal single click
                        if (modifiers == InputEvent.BUTTON1_MASK) {
                            if (tree.isCollapsed(row)) {
                                tree.expandPath(tree.getPathForRow(row));
                            } else {
                                tree.collapsePath(tree.getPathForRow(row));
                            }
                            display.setCursor(c);
                            return;
                        }

                        // middle button spawns popupMenu on Programs
                        if ((modifiers & InputEvent.BUTTON2_MASK) != 0) {
                            Object parent =
                                tree.getPathForRow(row).getLastPathComponent();
                            if (model.isPopupSuitable(parent)) {
                                containerPopUpMenu.setLocation(
                                    treeTablePoint.x,
                                    treeTablePoint.y + 5);
                                containerPopUpMenu.show(
                                    contentPane,
                                    treeTablePoint.x,
                                    treeTablePoint.y + 5);
                                popUpDBComponent =
                                    tree
                                        .getPathForRow(row)
                                        .getLastPathComponent();
                                return;
                            }
                        }

                        if ((modifiers & InputEvent.BUTTON3_MASK) != 0) {
                            TreePath[] kids = null;
                            // it's a right mouse button click
                            Object parent =
                                tree.getPathForRow(row).getLastPathComponent();
                            if (parent instanceof DBContainer
                                && ((DBContainer) parent).isObservation()
                                && tree.isExpanded(tree.getPathForRow(row))) {

                                kids =
                                    new TreePath[model.getChildCount(parent)];
                                for (int i = 0; i < kids.length; i++) {
                                    kids[i] =
                                        new TreePath(
                                            model.getPathToRoot(
                                                model.getChild(parent, i)));
                                }

                            } else {
                                int i = 0;
                                model.expandAllTheNodes(row);
                                Vector allLeaves =
                                    model.getAllTheLeaves(parent);
                                kids = new TreePath[allLeaves.size()];

                                for (Enumeration ee = allLeaves.elements();
                                    ee.hasMoreElements();
                                    ) {
                                    // this is a very unpleasant particularization
                                    // which I would have liked to avoid.....
                                    Object tmpNode = ee.nextElement();
                                    kids[i] =
                                        new TreePath(
                                            model.getPathToRoot(
                                                (DBOBComponent) tmpNode));
                                    i++;
                                }
                            }

                            if (debug) {
                                System.out.println(
                                    "Right Mouse Button "
                                        + kids.length
                                        + " on "
                                        + parent);
                            }

                            if (e.isControlDown()) {
                                // try to determine if they are all already selected
                                boolean allSelected = true;
                                for (int l = 0; l < kids.length; l++) {
                                    if (!selectionModel
                                        .isPathSelected(kids[l])) {
                                        allSelected = false;
                                        break;
                                    }
                                }
                                if (allSelected) {
                                    selectionModel.removeSelectionPaths(kids);
                                } else {
                                    selectionModel.addSelectionPaths(kids);
                                }
                            } else {
                                selectionModel.setSelectionPaths(kids);
                            }
                            display.setCursor(c);
                            kids = null;
                            return;
                        }
                    } else { // it is a leaf!
                        if ((modifiers & InputEvent.BUTTON3_MASK) != 0) {
                            // it's a right mouse button click
                            if (tree.isPathSelected(tree.getPathForRow(row))) {
                                popUpDBComponent =
                                    tree
                                        .getPathForRow(row)
                                        .getLastPathComponent();
                                filePopUpMenu.setLocation(
                                    treeTablePoint.x,
                                    treeTablePoint.y + 5);
                                filePopUpMenu.show(
                                    contentPane,
                                    treeTablePoint.x,
                                    treeTablePoint.y + 5);
                                return;
                            }
                        }
                    }
                } // component was a JTreeTable

                // this is here merely to handle a grace exit from the Popup Menu
                if (component instanceof JTreeTable
                    && eventID == MouseEvent.MOUSE_ENTERED) {
                    if (filePopUpMenu.isVisible()) {
                        filePopUpMenu.setVisible(false);
                    }
                    if (containerPopUpMenu.isVisible()) {
                        containerPopUpMenu.setVisible(false);
                    }
                }

                component.dispatchEvent(
                    new MouseEvent(
                        component,
                        e.getID(),
                        e.getWhen(),
                        e.getModifiers(),
                        componentPoint.x,
                        componentPoint.y,
                        e.getClickCount(),
                        e.isPopupTrigger()));
                if (debug) {
                    System.out.println("Redispatched");
                }
            }
        }
    }
*/
    // actionListener interface for the PopUp Menu

    public void actionPerformed(ActionEvent e) {
        StringBuffer commandLong = null;
        // System.out.println( e.getActionCommand() + " " + popUpDBComponent );
        if (e.getActionCommand().toString().equals(retrieveLabel)) {
            mainDisplay.retrieveFilesFromDb(getSelectedFiles().elements());
            return;
        }
        for (int i = 0; i < 4; i++) {
            String tmpString = "MENU_CUSTOM" + String.valueOf(i + 1) + "_LBL";
            if (e
                .getActionCommand()
                .toString()
                .equals(PropertyDB.getInstance().getProperty(tmpString))) {
                String command =
                    PropertyDB.getInstance().getProperty(
                        "MENU_CUSTOM" + String.valueOf(i + 1));
                if (i > 1) {
                    System.out.println(command + " " + popUpDBComponent);
                    Exec.runCommand(command + " " + popUpDBComponent);
                } else {
                    commandLong = new StringBuffer(command + " ");
                    for (Enumeration ee = getSelectedFiles().elements();
                        ee.hasMoreElements();
                        ) {
                        commandLong.append(ee.nextElement().toString() + " ");
                    }
                    System.out.println(commandLong);
                    Exec.runCommand(commandLong.toString());
                }
                return;
            }
        }
    }
}
// end of file DBTreeTable.java
