/**
 * A class used to communicate an event in the data model.
 * It is not necessary to include the affectedFile when communicating
 * an event and the application needs to handle the event when there is
 * a null affectedFile.
 */

package org.eso.gasgano.datamodel;

public class DataModelEvent {

    static public final int UNKNOWN_EVENT = -1; 
    static public final int FILE_DELETED = 0; 
    static public final int FILE_ADDED = 1; 
    static public final int FILE_UPDATED = 2; 
    static public final int PROGRAM_DELETED = 3; 
    static public final int PROGRAM_ADDED = 4; 
    static public final int OBSERVATION_DELETED = 5; 
    static public final int OBSERVATION_ADDED = 6; 
    static public final int DIRECTORY_DELETED = 7; 
    static public final int DIRECTORY_ADDED = 8; 
    static public final int TELESCOPE_DELETED = 9; 
    static public final int TELESCOPE_ADDED = 10; 
    static public final int INTERMEDIATE_DELETED = 11; 
    static public final int INTERMEDIATE_ADDED = 12; 

    static public final int TREE_GENERAL = -1 ; 

    private int eventType = UNKNOWN_EVENT;
    private Object affectedFile = null;
    private int treeLevel = TREE_GENERAL  ; 

    /**
     * @param e The event type: UNKNOWN_EVENT, FILE_DELETED, FILE_ADDED,
     * FILE_UPDATED.
     */
    public DataModelEvent(int e) {
        init(e, null);
    }

    /**
     * @param e The event type: UNKNOWN_EVENT, FILE_DELETED, FILE_ADDED,
     * FILE_UPDATED.
     * @param f The file associated with the event.
     */
    public DataModelEvent(int e, Object f) {
        init(e, f);
    }

    /**
     * @param e The event type: UNKNOWN_EVENT, FILE_DELETED, FILE_ADDED,
     * FILE_UPDATED.
     * @param f The file associated with the event.
     */
    private void init(int e, Object  f) {
        eventType = e;
        affectedFile = f;
    }

    /**
     * @return The event type: UNKNOWN_EVENT, FILE_DELETED, FILE_ADDED,
     * FILE_UPDATED.
     */
    public int getEvent() {
        return eventType;
    }

    /**
     * @return The file associated with the event, possibly null.
     */
    public Object getFile() {
        return affectedFile;
    }

    /**
     * @return: the tree representation level for which this event is meaningful <p>
     * @see: DFSFileTreeTableModel
     */
    public int  getTreeType() {
        return treeLevel ;
    }

    /**
     * @param:  the tree representation level for which this event is meaningful <p>
     * @see: DFSFileTreeTableModel
     */

    public void   setTreeType(int tt ) {
        treeLevel = tt ;
    }
}
// end of file DataModelEvent.java
