/**
 * @author N. Kornweibel
 * @version 1.0
 */

package org.eso.gasgano.gui;

import java.awt.Menu;
import java.awt.MenuBar;
import java.awt.MenuItem;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.util.Enumeration;
import java.util.Vector;

import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JFrame;

import org.eso.dfs.gui.SwingFileChooser;
import org.eso.gasgano.datamodel.filesystem.DFSFile;
import org.eso.gasgano.datamodel.filesystem.FitsFile;
import org.eso.gasgano.datamodel.gui.FitsFileDisplay;
import org.eso.gasgano.datamodel.gui.FitsTableModel;
import org.eso.gasgano.keyword.Keyword;
import org.eso.gasgano.keyword.RuleTable;

public class ClassBldrTest implements ActionListener {

    private Vector fitsFiles = new Vector();
    private ClassificationBuilder classBldr = null;
    private RuleTable rt = null;
    private String filePromptDir = new String(".");

    public static void main(String args[]) {
        String ruleTablePath = null;

        if (args.length < 1) {
            System.out.println("usage: <path to rule table>");
            System.exit(0);
        } else {
            ruleTablePath = args[0];
        }
        ClassBldrTest cb = new ClassBldrTest(ruleTablePath);
    }

    public ClassBldrTest(String rulePath) {
        boolean rtNotFound = false;
        rt = RuleTable.readFile(rulePath);
        if (rt == null) {
            rt = new RuleTable(rulePath);
            rtNotFound = true;
        }
        classBldr = new ClassificationBuilder(rt);
        classBldr.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                System.exit(0);
            }
        });
 
        classBldr.pack();
        classBldr.setSize(800, 800);
        // required by my window manager at least...
        classBldr.setLocation(100, 300);
        classBldr.setVisible(true);
        addMenu();
        if (rtNotFound) {
            classBldr.userMessage("Failed to read/locate: "+rulePath
                +" starting with an empty table.");
        }
    }

    private void addMenu() {

        MenuBar mb = new MenuBar();
        Menu m = new Menu("File");
 
        MenuItem mi = null;

        mi = new MenuItem("Open Fits File");
        mi.addActionListener(this);
        m.add(mi);
 
        mi = new MenuItem("Reclassify");
        mi.addActionListener(this);
        m.add(mi);
 
        mi = new MenuItem("Quit");
        mi.addActionListener(this);
        m.add(mi);
 
        mb.add(m);
        classBldr.setMenuBar(mb);
    }

    private void addFile() {
        boolean promptAgain = true;
        JFileChooser fc = new SwingFileChooser(filePromptDir);
        fc.setFileSelectionMode(JFileChooser.FILES_ONLY);
        fc.setMultiSelectionEnabled(false);
        String selectedFile = null;
        fc.setDialogTitle("Select FITS file");
        fc.setDialogType(JFileChooser.OPEN_DIALOG);
        fc.setApproveButtonText("Load");
        do {
            if (fc.showOpenDialog(classBldr) == JFileChooser.APPROVE_OPTION) {
                if (fc.getSelectedFile() != null) {
                    selectedFile = fc.getSelectedFile().getPath();
                    filePromptDir = fc.getSelectedFile().getParent();
                    promptAgain = true;
                } else {
                    break;
                }
            } else {
                break;
            }
            if (selectedFile.endsWith(".fits") ||
                    selectedFile.endsWith(".tfits")) {
                FitsFileWindow ffile = new FitsFileWindow(selectedFile, rt);
                if (ffile.getFitsFile().getStatus() == DFSFile.FILE_VALID) {
                    fitsFiles.addElement(ffile);
                    promptAgain = false;
                    ffile.pack();
                    ffile.setSize(1060, 380);
                    ffile.setLocation(50+(80*fitsFiles.size()),
                        50+(80*fitsFiles.size()));
                    ffile.setVisible(true);
                } else {
                    classBldr.userMessage("Invalid FITS File: "+selectedFile);
                    promptAgain = true;
                }
            } else {
                classBldr.userMessage("Selected file is not a FITS File");
                promptAgain = true;
            }
        } while (promptAgain);
    }

    private void reclassify() {
        Enumeration e = fitsFiles.elements();
        while (e.hasMoreElements()) {
            FitsFileWindow f = (FitsFileWindow)e.nextElement();
            if (f != null) {
                f.classify(rt);
            }
        }
    }

    // implementation of the ActionListener interface
    public void actionPerformed(ActionEvent a) {
        if (a.getActionCommand() == "Quit") {
            System.exit(0);
        } else if (a.getActionCommand() == "Open Fits File") {
            addFile();
        } else if (a.getActionCommand() == "Reclassify") {
            reclassify();
        }
    }

    class FitsFileWindow extends JFrame implements WindowListener {
        private FitsFile ffile = null;
        private String fileAddress = null;

        FitsFileWindow(String fAddress, RuleTable r) {
            super(fAddress);
            fileAddress = fAddress;
            ffile = new FitsFile(fileAddress, r, null);
            getContentPane().add(getDisplayComponent());
            setTitle();
        }

    public JComponent getDisplayComponent() {
        JComponent rc = null;
        boolean freeMemory = true;
        if (ffile.headerInMemory()) {
            freeMemory = false;
        } else {
            ffile.readFitsHeaderAndExtensions(FitsFileDisplay.isShowExtensions());
        }
        String title = null;
        Keyword origFile = ffile.getKeyword(Keyword.ORIGFILE);
        rc = new FitsFileDisplay(fileAddress, new FitsTableModel(ffile));
        if (freeMemory) {
            ffile.freeHeaderMemory();
        }
        return rc;
    }


        FitsFile getFitsFile() {
            return ffile;
        }
 
        private void setTitle() {
            setTitle(fileAddress+": "+ffile.getClassification());
        }
 
        void classify(RuleTable r) {
             ffile.classify(rt);
             setTitle();
        }
 
        // implement the WindowListener interface.
        public void windowOpened(WindowEvent e) {
        }

        public void windowClosing(WindowEvent e) {
            fitsFiles.removeElement(this);
        }

        public void windowClosed(WindowEvent e) {
        }

        public void windowIconified(WindowEvent e) {
        }

        public void windowDeiconified(WindowEvent e) {
        }

        public void windowActivated(WindowEvent e) {
        }

        public void windowDeactivated(WindowEvent e) {
        }
    }
}
// end of file ClassBldrTest.java
