package org.eso.gasgano.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.InputEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.AbstractAction;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.border.EmptyBorder;
import javax.swing.border.TitledBorder;
import javax.swing.table.TableColumn;

import org.eso.gasgano.properties.GasProp;
import org.eso.gasgano.properties.MenuTableModel;

/**
 * @author nkornwei
 *
 * Panel for editing menu definitions database tree display keywords
 */
public class DBDisplayPanel extends PropertyEditorSubPanel {
    
    private GasProp gpi;
    private JTable menuTable;
    private MenuTableModel menuTableModel;
    
    // database view display keywords.
    private JTable dbDispKeys;
    private KeywordListAdaptor dbDispKeysModel;

    /**
     * Constructor.
     * @param gp
     */
    public DBDisplayPanel(GasProp gp) {
        JPanel jpTable1;
        JPanel jpTable2;

        menuTable = new JTable();
        menuTableModel = new MenuTableModel(gp);
        menuTable.setModel(menuTableModel);
        menuTable.setDefaultEditor(
            menuTable.getColumnClass(1),
            new PropertyCellEditor());

        menuTable.setCellSelectionEnabled(true);
        menuTable.setPreferredScrollableViewportSize(new Dimension(240, 100));
 
        JScrollPane scrollPane = new JScrollPane(menuTable);
        jpTable2 = new JPanel(new BorderLayout());
        jpTable2.setBorder(new TitledBorder("Database pull-down menus associations"));
        jpTable2.add(scrollPane, BorderLayout.CENTER);

        setBorder(new EmptyBorder(3, 3, 3, 3));
        setLayout(new BorderLayout());

        jpTable1 = setupDBDisplayColumnsTable(gp);  
        loadProperties(gp);

        // nothing is visible if this is not the safe environment.
        if (GasProp.isEnvironmentSafe()) {
            add(jpTable1, BorderLayout.NORTH);
            add(jpTable2, BorderLayout.SOUTH);
        }
    }

    /**
     * Utility method to reduce code in the constructor - initializes the
     * panel for editing DB display columns keywords.
     * Must be called before loadProperties().
     * @return JPanel
     */
    private JPanel setupDBDisplayColumnsTable(GasProp gp) {
        JButton insertRow;
        JButton deleteRow;
        String dbHeaderLabel = new String("Database - Displayed Keywords");
        
        insertRow = new JButton("Insert Row");
        insertRow.setToolTipText(
            "Left-click for insert below, right-click for insert above.");
        insertRow.addMouseListener(new MouseAdapter() {
            public void mousePressed(MouseEvent e) {
                if ((e.getModifiers() & InputEvent.BUTTON3_MASK) != 0) {
                    insertBlankRow(dbDispKeys, false);
                }
                if ((e.getModifiers() & InputEvent.BUTTON1_MASK) != 0) {
                    insertBlankRow(dbDispKeys, true);
                }
            }

        });
        deleteRow = new JButton("Delete Row");
        deleteRow.setToolTipText("Delete selected row.");
        deleteRow.addActionListener(new AbstractAction() {
            public void actionPerformed(ActionEvent e) {
                deleteSelectedRow(dbDispKeys);
            }});
            
        // Table area
        dbDispKeys = new JTable();
        dbDispKeys.setCellSelectionEnabled(true);
        dbDispKeys.setPreferredScrollableViewportSize(new Dimension(240, 140));
        
        dbDispKeysModel = new KeywordListAdaptor(gp.dbDisplayColumns, dbHeaderLabel, true);
        dbDispKeys.setModel(dbDispKeysModel);
        dbDispKeys.setDefaultEditor(
            dbDispKeys.getColumnClass(0),
            new PropertyCellEditor());
            
        TableColumn mColumn = dbDispKeys.getColumn(dbHeaderLabel);
        mColumn.setMinWidth(210);
        mColumn.setResizable(false);
        
        JScrollPane scrollPane1 = new JScrollPane(dbDispKeys);
        scrollPane1.setVerticalScrollBarPolicy(
            JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
         
        JPanel jpLower = new JPanel();
        jpLower.add(insertRow);
        jpLower.add(deleteRow);

        JPanel rc = new JPanel(new BorderLayout());
        rc.setBorder(new TitledBorder("Database - Displayed Keywords"));
        rc.add(scrollPane1, BorderLayout.CENTER);
        rc.add(jpLower, BorderLayout.SOUTH);
        
        return rc;
    }

    /**
     * Load the properties and reset the GUI.
     * @see org.eso.gasgano.gui.PropertyEditorSubPanel#loadProperties(GasProp)
     */
    public void loadProperties(GasProp gp) {
        gpi = gp;        
        dbDispKeysModel.setData(gp.dbDisplayColumns);
        menuTable
            .getDefaultEditor(menuTable.getColumnClass(0))
            .cancelCellEditing();

        menuTableModel.loadProperties(gpi);
    }

    /**
     * Save the properties from the GUI to the properties database.
     * 
     * @return String null on success, non-null on error - the returned
     * String is the error message.
     */
    public String saveProperties() {
        String rc = null;
        dbDispKeys
            .getDefaultEditor(dbDispKeys.getColumnClass(0))
            .stopCellEditing();
            
        for (int i = 0; i < gpi.dbDisplayColumns.size(); i++) {
            if (gpi.dbDisplayColumns.elementAt(i).toString().equals("")
                || gpi.dbDisplayColumns.elementAt(i).toString().equals(
                    "*INVALID*")
                || gpi.dbDisplayColumns.elementAt(i) == null) {
                gpi.dbDisplayColumns.removeElementAt(i);
            }
        }
        
        Object[][] tableContent = menuTableModel.getTableContent();
        if (tableContent[0][1].equals("")) {
            tableContent[0][1] = GasProp.MENU_CUSTOM1_LBL;
        }
        if (tableContent[1][1].equals("")) {
            tableContent[1][1] = GasProp.MENU_CUSTOM2_LBL;
        }
        if (tableContent[2][1].equals("")) {
            tableContent[2][1] = GasProp.MENU_CUSTOM3_LBL;
        }
        if (tableContent[3][1].equals("")) {
            tableContent[3][1] = GasProp.MENU_CUSTOM4_LBL;
        }
        gpi.put("MENU_CUSTOM1_LBL", tableContent[0][1]);
        gpi.put("MENU_CUSTOM2_LBL", tableContent[1][1]);
        gpi.put("MENU_CUSTOM3_LBL", tableContent[2][1]);
        gpi.put("MENU_CUSTOM4_LBL", tableContent[3][1]);
        gpi.put("MENU_CUSTOM1", tableContent[0][2]);
        gpi.put("MENU_CUSTOM2", tableContent[1][2]);
        gpi.put("MENU_CUSTOM3", tableContent[2][2]);
        gpi.put("MENU_CUSTOM4", tableContent[3][2]);
        gpi.put("MENU_RETRIEVE", tableContent[4][2]);
        menuTableModel.setTableContent(tableContent);
        return rc;
    }

    /**
     * Is a restart required due to property changes - in this
     * case never, but the call is used to stop editing
     * on the widget - not the cirrect place for it....
     */
    public int restartRequired() {
        menuTable
            .getDefaultEditor(menuTable.getColumnClass(0))
            .stopCellEditing();

        return NOACTION_ON_SAVE;
    }
    
    /**
     * Insert a row into the table. Assumes that the table model is
     * KeywordListAdaptor.
     * TODO: same method appears in DisplayPanel - needs refactoring.
     * @param table
     * @param after
     */
    private void insertBlankRow(JTable table, boolean after) {
        int selRow = table.getSelectedRow();
        if (selRow < 0) {
            // insert at beginning or append to end as no specific row was selected.
            selRow = after ? table.getRowCount() - 1 : 0;
        }
        if (isDebug()) {
            System.out.println("selected row: " + selRow);
        }
        if (after && table.getRowCount() != 0) {
            ((ListAdaptor) table.getModel()).insertRow(selRow + 1, "");
            table.setRowSelectionInterval(selRow + 1, selRow + 1);
        } else {
            ((ListAdaptor) table.getModel()).insertRow(selRow, "");
            table.setRowSelectionInterval(selRow, selRow);
        }
    }

    /**
     * Delete a row into the table. Assumes that the table model is
     * KeywordListAdaptor.
     * TODO: same method appears in other property editing panels - needs refactoring.
     */
    private void deleteSelectedRow(JTable table) {
        int selRow = table.getSelectedRow();
        if (selRow < 0) {
            return;
        }
        if (isDebug()) {
            System.out.println("selected row: " + selRow);
        }
        ((ListAdaptor) table.getModel()).removeRow(selRow);
    }
}