package org.eso.gasgano.gui;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.util.Vector;

import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.border.EmptyBorder;
import javax.swing.border.EtchedBorder;
import javax.swing.border.TitledBorder;

import org.eso.gasgano.GasPropEditor;
import org.eso.gasgano.properties.GasProp;

/**
 * @author nkornwei
 *
 * Panel for editing properties associated with the file system settings.
 */
public class DataPanel extends PropertyEditorSubPanel {
    private GasProp gpi;
    private String selectedFile;

    private JTextField keyList;
    private JTextField scriptDir;
    private JTextField filterRule;
    private JTextField classificationRule;

    // verbose.
    private JComboBox updateTimer;
    private JTextField verboseLevel;
    private JRadioButton shortenPaths;

    private JComboBox verboseLevelBox;
    private Vector verboseVector = new Vector();

    private Vector updateTimerVector = new Vector();
    private Vector updateTimerValues = new Vector(5);

    // extensions as well
    private JTextField textExt;
    private FileListChooser fileListChooser;

    private JPanel jpl = new JPanel();
    private JPanel jpr = new JPanel();
    private JPanel jpll = new JPanel();
    private JPanel jprl = new JPanel();
    private JPanel miniPanel = new JPanel();

    /**
     * Constructor
     */
    public DataPanel(GasProp gp) {

        verboseVector.addElement("No Verbose");
        verboseVector.addElement("DataModel Change");
        verboseVector.addElement("DataModel Events");
        verboseVector.addElement("External Commands");
        verboseVector.addElement("Basenames");
        verboseVector.addElement("Timing");
        verboseVector.addElement("Events");
        verboseVector.addElement("Expression Rules");
        verboseVector.addElement("Database");
        verboseVector.addElement("All verbose levels");

        updateTimerVector.addElement("No update");
        updateTimerVector.addElement("1 minute");
        updateTimerVector.addElement("2 minutes");
        updateTimerVector.addElement("5 minutes");
        updateTimerVector.addElement("30 minutes");

        updateTimerValues.addElement(new Integer(0));
        updateTimerValues.addElement(new Integer(1));
        updateTimerValues.addElement(new Integer(2));
        updateTimerValues.addElement(new Integer(5));
        updateTimerValues.addElement(new Integer(30));

        keyList = new JTextField(20);
        keyList.setToolTipText("Default directory for keyword list files.");
        scriptDir = new JTextField(20);
        scriptDir.setToolTipText("Default directory for script files.");
        filterRule = new JTextField(20);
        filterRule.setToolTipText("Location of filter rule file.");
        classificationRule = new JTextField(20);
        classificationRule.setToolTipText(
            "Location of classification rule file.");

        textExt = new JTextField(20);
        textExt.setToolTipText(
            "Semi-colon (;) separated list of text file extensions.");

        verboseLevelBox = new JComboBox(verboseVector);
        updateTimer = new JComboBox(updateTimerVector);
        shortenPaths =
            new JRadioButton("shorten data files paths for commands", true);
        loadProperties(gp);
        // TitledBorder tb = new TitledBorder(new EtchedBorder(), "Filesystem specifications" ,
        //                   0, 10 , Font. , Color.blue);
        TitledBorder tb = new TitledBorder("Filesystem specifications");
        // tb.setForeground(Color.blue);
        JPanel filesPanel = new JPanel(new BorderLayout());
        JPanel otherPanel = new JPanel(new BorderLayout());
        JPanel tmpPanel = new JPanel(new BorderLayout());

        setLayout(new BorderLayout(5, 5));
        filesPanel.setBorder(tb);

        // setup the file list chooser
        jpl.setLayout(new GridLayout(8, 1));
        jpl.setBorder(new EmptyBorder(3, 3, 3, 3));

        jpl.add(new JLabel("Keyword List directory:", SwingConstants.RIGHT));
        jpl.add(new JLabel("Scripts directory:", SwingConstants.RIGHT));
        jpl.add(new JLabel("Filter Rule file:", SwingConstants.RIGHT));
        jpl.add(new JLabel("Classification Rule file:", SwingConstants.RIGHT));
        jpl.add(new JLabel()); // separator
        jpl.add(new JLabel("Textfile Extensions:", SwingConstants.RIGHT));

        jpr.setLayout(new GridLayout(8, 1));
        jpr.setBorder(new EmptyBorder(3, 3, 3, 3));

        jpr.add(
            setupFileChooserPanel(
                keyList,
                JFileChooser.DIRECTORIES_ONLY));
        jpr.add(
            setupFileChooserPanel(
                scriptDir,
                JFileChooser.DIRECTORIES_ONLY));
        jpr.add(
            setupFileChooserPanel(
                filterRule,
                JFileChooser.FILES_ONLY));
        jpr.add(
            setupFileChooserPanel(
                classificationRule,
                JFileChooser.FILES_ONLY));
        jpr.add(new JLabel()); // separator
        jpr.add(textExt);

        jprl = new JPanel(new GridLayout(2, 1));
        jprl.setBorder(new EmptyBorder(3, 3, 3, 3));
        jpll = new JPanel(new GridLayout(2, 2));
        jpll.setBorder(new EmptyBorder(3, 3, 3, 3));
        otherPanel.setBorder(new EtchedBorder());

        jpll.add(
            new JLabel("Automatic Filesystem Refresh:", SwingConstants.RIGHT));
        jprl.add(updateTimer);
        jpll.add(new JLabel("Verbose Level:", SwingConstants.RIGHT));
        jprl.add(verboseLevelBox);

        tmpPanel.add("Center", jpll);
        tmpPanel.add("East", jprl);
        otherPanel.add("North", shortenPaths);
        otherPanel.add("West", tmpPanel);

        filesPanel.add("West", jpl);
        filesPanel.add("Center", jpr);

        add("North", fileListChooser);
        add("Center", filesPanel);
        add("South", otherPanel);
    }

    /*
     * Load the properties from the given GasProp
     */
    public void loadProperties(GasProp gp) {
        gpi = gp;

        if (fileListChooser == null) {
            fileListChooser = new FileListChooser();
        }
        Vector vec = null;

        vec = GasProp.stringScanner((String) gp.getProperty("DATA_FILES"), ';');
        if (vec != null) {
            fileListChooser.setFileList(vec);
        }
        keyList.setText(gp.getProperty("KEYWORDLIST_DIR"));
        scriptDir.setText(gp.getProperty("SCRIPTS_DIR"));
        filterRule.setText(gp.getProperty("FILTER_FILE"));
        classificationRule.setText(gp.getProperty("CLASSRULE_FILE"));
        textExt.setText(gp.getProperty("TEXTFILE_EXTENSIONS"));

        if (gp.getProperty("SHORTEN_FILES_PATH").equals("true")) {
            shortenPaths.setSelected(true);
        } else {
            shortenPaths.setSelected(false);
        }

        try {
            int pippo = Integer.parseInt(gp.getProperty("VERBOSE_LEVEL"));
            verboseLevelBox.setSelectedIndex(pippo);

            pippo =
                updateTimerValues.indexOf(
                    Integer.valueOf(gp.getProperty("UPDATE_TIMER")));
            if (pippo == -1) {
                updateTimer.setSelectedIndex(0);
            } else {
                updateTimer.setSelectedIndex(pippo);
            }

        } catch (NumberFormatException nfe) {
            verboseLevelBox.setSelectedIndex(0);
            updateTimer.setSelectedIndex(0);
        }
    }

    /*
     * Flush all properties to the properties set, returning an error message
     * on error.
     * @return String null if there is no error, else an error message.
     */
    public String saveProperties() {
        String rc = null;
        gpi.put(
            "DATA_FILES",
            GasProp.stringBuilder(fileListChooser.getFileList(), ';'));
        if (!keyList.getText().trim().equals("")) {
            gpi.put("KEYWORDLIST_DIR", keyList.getText());
        } else {
            rc = "Data Panel: Keyword List Directory field  empty";
        }
        if (!scriptDir.getText().trim().equals("")) {
            gpi.put("SCRIPTS_DIR", scriptDir.getText());
        } else {
            rc = "Data Panel: Scripts Directory field empty";
        }
        if (!filterRule.getText().trim().equals("")) {
            gpi.put("FILTER_FILE", filterRule.getText());
        } else {
            rc = "Data Panel: Filter File field  empty";
        }
        if (!classificationRule.getText().trim().equals("")) {
            gpi.put("CLASSRULE_FILE", classificationRule.getText());
        } else {
            rc = "Data Panel: Classification Rules File field  empty";
        }
        gpi.put("TEXTFILE_EXTENSIONS", textExt.getText());
        gpi.put(
            "VERBOSE_LEVEL",
            String.valueOf(verboseLevelBox.getSelectedIndex()));
        gpi.put(
            "UPDATE_TIMER",
            String.valueOf(
                updateTimerValues.elementAt(updateTimer.getSelectedIndex())));
        if (shortenPaths.isSelected()) {
            gpi.put("SHORTEN_FILES_PATH", "true");
        } else {
            gpi.put("SHORTEN_FILES_PATH", "false");
        }
        return rc;
    }
    
    public int restartRequired() {
        return NOACTION_ON_SAVE;
    }
}
