package org.eso.ohs.core.dbb.client;


import org.eso.ohs.core.docview.datatrans.ObjectAdaptor;
import org.eso.ohs.core.utilities.Convert;

/**
 * A special data type to be used in the database browsers built for
 * P2PP and the OT; and in general, by all browsers dealing with
 * coordinates stored in the OB repository. <BR>
 *
 * This was a private class of p2pp/DbaseWindow.java. Moved to
 * utilities for want of a better place; in fact the DBB package
 * should be moved up one level, from org/eso/ohs/dbb to org/eso/dbb,
 * and this class should then go into some OHS-specific dbb directory.
 *
 * <p>
 * $Id: DbaseAngle.java,v 1.5 2005/09/23 07:14:10 pnunes Exp $
 * <br>
 * @author Bob Kemp
 * @author Tim Canavan
 * @author A M Chavan, 03-Feb-01: moved class out of DbaseWindow.java */

public abstract class DbaseAngle implements DbbDataType {

    private ObjectAdaptor adaptor_;
    private final static String DEFAULT_ANGLE_ = "-00:00:00.000";

    public DbaseAngle() {
    }
      
    public DbaseAngle(ObjectAdaptor adaptor) {
	adaptor_ = adaptor;
    }
      
      
    public int getDataTypeLength() {
	return DEFAULT_ANGLE_.length();
    }

    public Class getDataType() {
	return String.class;
    }
      
    public Class getDatabaseType() {
	return Integer.class;
    }

    public String fromDatabaseData(Object value) {
    	
		// We can safely cast this to an Integer because
		// this renderer is supposed to only take integer
		// values -- see getDatabaseType().
		// amchavan & pnunes, 22-Sep-2005
		Integer i = (Integer) value;
		
		try {
			return (Convert.milliarcsecToDDMMSS(i.intValue()));
		} catch (IllegalArgumentException e) {
			return " 00:00:00.000";
		}
	}

    public Object toDatabaseData(String value) {
        int converted_value;

        try{
            converted_value = Convert.DDMMSSToMilliarcsec( value );
        }catch(IllegalArgumentException e){
            return new Integer(0);
        }
        return new Integer(converted_value);

    }
}





