package org.eso.ohs.core.dbb.client;

// $Id: DbbRightAscension.java,v 1.3 2005/04/11 13:53:44 tcanavan Exp $

import org.eso.ohs.core.utilities.Convert;

/**
 *  A transducer to convert set/get of String values into set/get
 *  on an int property.  The target property's name is given to the
 *  constructor. The renderer is applied to a RA obj
 *
 *  <p>
 *  For example, users will call it's
 *  setValue/getValue methods.  Those calls will be translated into
 *  calls to setX/getX on the object given to the constructor.
 *
 *  <p>
 */

public class DbbRightAscension extends Coordinate {

    private static int angleToHourFactor = 15;  
    //----------------------------------------
    // Public methods
    //----------------------------------------


    /**
     * Converts data to a String from the meta-level equivalent of the
     * actual property's type, i.e. from a java.lang.Long
     * to a long int. 
     *
     * @param value	value of the property.
     */
    public String fromDatabaseData(Object obj) {
	String decFormatString = null;
	if (obj == null) {
	    decFormatString = "00:00:00.0000";
	} else {
	    int milliSeconds = (((Integer)obj).intValue())/angleToHourFactor;
	    
	    decFormatString = Convert.milliarcsecToHHMMSS(milliSeconds);
	}
	return decFormatString;
    }


    /**
     * Converts data from a String to a meta-level equivalent of the
     * actual property's type, i.e. to a java.lang.Integer
     * representing an int.
     *
     * @param value the new value of the property.
     */
    public Object toDatabaseData(String value) {
	int milliarcSec = 0; 
	if (value != null ) {
	    milliarcSec = Convert.HHMMSSToMilliarcsec(  value ) ;
	}
    int millisecs = milliarcSec * angleToHourFactor;
	return new Integer(millisecs);
    }


}

