package org.eso.ohs.core.dbb.examples;

// $Id: ExampleBrowser0.java,v 1.4 2005/11/04 10:55:29 ddorigo Exp $

import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

import javax.swing.JFrame;

import org.apache.log4j.Logger;
import org.eso.ohs.core.dbb.client.DbbPanel;
import org.eso.ohs.core.dbb.client.DbbTextField;
import org.eso.ohs.core.dbb.client.DbbView;
import org.eso.ohs.core.dbb.server.DbbSession;
import org.eso.ohs.core.dbb.server.DbbSqlEngine;
import org.eso.ohs.core.dbb.server.DbbSqlEngineImpl;
import org.eso.ohs.core.dbb.sql.DbbSqlChunk;
import org.eso.ohs.core.dbb.sql.DbbSqlOperator;
import org.eso.ohs.core.dbb.sql.DbbSqlTable;

/**
 * This class provides a simple example that shows how to create a
 * database browser with a direct connection to the database.
 * <br>
 * @author A.M. Chavan, ESO 18-Feb-2000
 */
class ExampleBrowser0 extends JFrame {

private static final String rcsid = "$Id: ExampleBrowser0.java,v 1.4 2005/11/04 10:55:29 ddorigo Exp $";

 private static Logger stdlog_ = 
    Logger.getLogger( ExampleBrowser0.class );
  
    //----------------------------------------
    // Constructors
    //----------------------------------------

    /**
     * Create an object setting all its internal variable. The
     * constructor creates the associated sql engine (DbbSqlEngineImpl
     * implements the DbbSqlEngine interface) using a direct
     * connection to the database.  
     *
     * @param user_id		Database login
     * @param user_password	Database password
     */
    public ExampleBrowser0( String user_id, String user_password ) {

	DbbSqlTable    table = new DbbSqlTable( "opc70", "", "eso_users" );
	DbbView        view  = null;
	DbbPanel       panel = null;
	DbbSqlEngine   sqle  = null;
	DbbSqlChunk    c;
	DbbTextField   w;
	DbbSqlOperator o;
    
	String labName  = "Name";
	String labID    = "ID";

	/* Build and SQL engine which selects by default 
	 * columns id and surname_dl from our table
	 */

	/* Connect to the database (define a session) */
	DbbSession session =
	    new DbbSession("jdbc:sybase:Tds:acdbdev.hq.eso.org:6789",
			   "opc70", user_id, user_password);

	try {
            
	    // Build an SQL engine using the connection
	    sqle = new DbbSqlEngineImpl( session, false );

	    // Default column: user ID
	    c = new DbbSqlChunk( table, "id", labID );
	    sqle.addDefaultColumnsChunk( c );
      
	    // Default column: user name
	    c = new DbbSqlChunk( table, "surname_dl", labName );
	    sqle.addDefaultColumnsChunk( c );
       
	} catch( Exception e ) { 
	    stdlog_.warn("",e);
	    System.exit(0);
	}

    
	// Build a view for interfacing to the SQL engine
	view = new DbbView( sqle );
	view.hideSortButton();

	/* Build the rows selection panel. Add a widget to allow the
	 * user to search by name with 'LIKE' 
	 */
	panel = new DbbPanel( "Selection Criteria", 1 );

	try {
      
	    o = new DbbSqlOperator( "LIKE" );
	    c = new DbbSqlChunk( table, "surname_dl", labName, o );
	    w = new DbbTextField( labName , c );
	    panel.addWidget( w );
      
	} 
	catch( Exception e ) { 
	    stdlog_.warn("",e); 
	}
    
	// Add this panel to the view as the rows selection panel
	view.setWherePanel( panel, "North" );
	getContentPane().add(view);
    } 

  
    //----------------------------------------
    // Public Methods
    //----------------------------------------

    /** Main method, to run directly the example. */
    public static void main( String[] args ) {
    
	int len = args.length;
	String msg = 
	    "Usage: ExampleBrowser0 db_login db_password";
	if(len < 2) {
	    stdlog_.info(msg);
	    System.exit(1);
	}

	ExampleBrowser0 dbb = new ExampleBrowser0( args[0], args[1] );
	dbb.setTitle( "DBB Example 0" );
    
	WindowListener l = new WindowAdapter() {
		public void windowClosing(WindowEvent e) {
		    System.exit(0);
		}
	    };
    
	dbb.addWindowListener(l);
	dbb.pack();
	dbb.setVisible(true);
    }  
}
