package org.eso.ohs.core.docview.datatrans;

/**
 *  A transducer to convert set/get of String values into set/get
 *  on an int property.  The target property's name is given to the
 *  constructor.
 *
 *  <p>
 *  For example, users will call it's
 *  setValue/getValue methods.  Those calls will be translated into
 *  calls to setX/getX on the object given to the constructor.
 *
 * <p>
 * $Id: IntAdaptor.java,v 1.1 2004/02/17 12:37:59 tcanavan Exp $
 * <br>
 * @author Bob Kemp
 */
public class IntAdaptor extends ObjAdaptor {

    /**
     * Construct a transducer.
     */
    public IntAdaptor() {
	super(int.class);
    }

    /**
     * Construct a transducer to convert set/get of String values into
     * setX/getX on an int property of <code>object</code>.
     *
     */
    public IntAdaptor(Object obj, String fieldName) {
	super(int.class, obj, fieldName);
    }

    /**
     * Converts data from a String to a meta-level equivalent of the
     * actual property's type, i.e. to a java.lang.Integer
     * representing an int.  Used with reflection to set
     * the new property value.
     *
     * @param value the new value of the property.
     */
    public Object toMetaLevelObject(String value) {
	return new Integer(value);
    }

    /**
     * Converts data to a String from the meta-level equivalent of the
     * actual property's type, i.e. from a java.lang.Long
     * to a long int.  Used with reflection to get
     * the current property value.
     *
     * @param value	value of the property.
     */
    public String fromMetaLevelObject(Object obj) {
	return obj == null ? "0" : obj.toString();
    }

}

