package org.eso.ohs.core.gui.widgets;

import java.awt.Dimension;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.StringReader;
import java.util.Enumeration;
import java.util.Vector;

import javax.swing.JComponent;
import javax.swing.JToolTip;
import javax.swing.SwingUtilities;
import javax.swing.plaf.metal.MetalToolTipUI;

import org.apache.log4j.Logger;

/**
 * Swing 1.1 goes not support Multi Line tooltip text
 * so we had to override JToolTip in order to provide
 * this support
 */

public class MultiLineToolTip extends JToolTip {

    /** Standard log channel */
    private static Logger stdlog_ =
        Logger.getLogger(MultiLineToolTip.class);

    /**
     * Create a MultiLineToolTip
     */

    public MultiLineToolTip() {
        setUI(new MultiLineToolTipUI());
    }

    /**
     * MultiLineToolTipUI creates the actual tooltip
     * and provides it with the correct dimensions
     */

    private class MultiLineToolTipUI extends MetalToolTipUI {
        private String[] strs;
        private int maxWidth = 0;
        public void paint(Graphics g, JComponent c) {
            // Port to 1.4.1 FontMetrics metrics = Toolkit.getDefaultToolkit().getFontMetrics(g.getFont());
            FontMetrics metrics = g.getFontMetrics(g.getFont());
            Dimension size = c.getSize();
            g.setColor(c.getBackground());
            g.fillRect(0, 0, size.width, size.height);
            g.setColor(c.getForeground());
            if (strs != null) {
                for (int i = 0; i < strs.length; i++) {
                    g.drawString(strs[i], 3, (metrics.getHeight()) * (i + 1));
                }
            }
        }

        /**
         * Preferred size for the tooltip text to be displayed
         *
         * @param c component that the tooltip text will be displayed in
         *
         */

        public Dimension getPreferredSize(JComponent c) {
            // Port to 1.4.1 FontMetrics metrics = Toolkit.getDefaultToolkit().getFontMetrics(c.getFont());
            FontMetrics metrics = c.getFontMetrics(c.getFont());
            String tipText = ((JToolTip) c).getTipText();
            if (tipText == null) {
                tipText = "";
            }
            BufferedReader br = new BufferedReader(new StringReader(tipText));
            String line;
            int maxWidth = 0;
            Vector v = new Vector();
            try {
                while ((line = br.readLine()) != null) {
                    int width =
                        SwingUtilities.computeStringWidth(metrics, line);
                    maxWidth = (maxWidth < width) ? width : maxWidth;
                    v.addElement(line);
                }
            } catch (IOException ex) {
                stdlog_.error(ex);
            }
            int lines = v.size();
            if (lines < 1) {
                strs = null;
                lines = 1;
            } else {
                strs = new String[lines];
                int i = 0;
                for (Enumeration e = v.elements(); e.hasMoreElements(); i++) {
                    strs[i] = (String) e.nextElement();
                }
            }
            int height = metrics.getHeight() * lines;
            this.maxWidth = maxWidth;
            return new Dimension(maxWidth + 6, height + 4);
        }
    }

}
