package org.eso.ohs.core.gui.widgets;

import java.io.File;

import javax.swing.JFileChooser;
import javax.swing.JList;
import javax.swing.UIManager;
import javax.swing.plaf.FileChooserUI;
import javax.swing.plaf.basic.BasicFileChooserUI;
/**
 * A SwingFileChooser wraps a JFileChooser and actually returns multiple
 * file selections. This class had to written to get around the multiple
 * file selection problem in Swinf
 *
 * @version $Revision: 1.2 $
 * @author ???
 */
public class SwingFileChooser extends JFileChooser {

    /**
     * Set the UIManager to use Java14WorkAroundMotifFileChooserUI instead of
     * the standard MotifFileChooserUI. Please read comment in header of
     * Java14WorkAroundMotifFileChooserUI.java for details of why this is done.
     */
    static {
        UIManager.getDefaults().put("FileChooserUI", "org.eso.ohs.core.gui.widgets." + "Java14WorkAroundMotifFileChooserUI");
    }
   
    private static final String rcsid =
        "$Id: SwingFileChooser.java,v 1.2 2004/02/26 11:07:49 ddorigo Exp $";

    /**
     * Default Constructor
     */

    public SwingFileChooser() {
        super();
    }

    /**
     * Constructs a file chooser given a certaing directory
     * @param directory Directory for file Chooser
     */
    public SwingFileChooser(String directory) {
        super(directory);  
    }

    /**
     * Set the current Directory.
     * This had to be overridden to get around another bug in Swing.
     * in Directory only selection mode.
     * @param file Set the current directory
     */

    public void setCurrentDirectory(File file) {
        // Let the action take place, then override the 'Name' field.
        // This is equivalent to the user typing "." in the name field.
        // stdlog_.debug("CLASS SwingFileChooser - setCurrentDirectory: File = "+file);
        super.setCurrentDirectory(file);
        if (isDirectorySelectionEnabled()) {
            setFilename(".");
        }
    }  
    


    /**
     * Set filename
     * @param filename
     */

    public void setFilename(String filename) {
        FileChooserUI ui = getUI();
        if (ui instanceof BasicFileChooserUI) {
            ((BasicFileChooserUI) ui).setFileName(filename);
        }
    }

    /**
     * Clear the selected entry in the file list
     */
/*
 * Possibly no longer needed for Java1.4:
    public void clearSelection() {
        cancelSelection();
        // Java1.4 port JList outerFileList = getList();
        // Java1.4 port outerFileList.clearSelection();
        //setSelectedFile(null);
    }
*/


    /**
     * @return the JList component containing the file list,
     * or null if this file chooser does not use the
     * Java14WorkAroundMotifFileChooserUI class.
     */
    public JList getFileListComponent() {
        FileChooserUI ui = getUI();
        JList rc = null;
        if (ui instanceof Java14WorkAroundMotifFileChooserUI) {
            rc = ((Java14WorkAroundMotifFileChooserUI)ui).getFileListComponent();
        }
        return rc;
    }
    /**
     * Override the getSelectedFiles method to actually return the
     * selected files
     *
     * @return the array of selected files
     */
    public File[] getSelectedFiles() {      
        JList ourFileList = getFileListComponent();
        if (ourFileList == null) {
            return null;
        }

        Object[] entries = ourFileList.getSelectedValues();
        File[] ourFiles = new File[entries.length];

        for (int i=0 ; i<entries.length ; i++) {
            if (entries[i] instanceof File) {
               ourFiles[i] = (File)entries[i];
            }
        }
        if (this.isMultiSelectionEnabled()) {
            return ourFiles;
        }
        else {
            return null;
        }
    }
    
    /**
     * Yet another bug in the FileChooser where the Update button
     * would not work in Motif because the following code was not called
     */

    public void rescanCurrentDirectory() {
        super.rescanCurrentDirectory();
    
        if (UIManager.getLookAndFeel().getID().equals("Motif")) {
            BasicFileChooserUI ui = (BasicFileChooserUI) getUI();
            ui.getModel().validateFileCache();
        }
    }

}
