package org.eso.ohs.instruments;

import java.util.StringTokenizer;

/**
 * Instancaites a paramater class that represents a keyword. 
 * The string that contains the keyword value is checked for its validity.
 * <p>
 * $Id: KeywordListParameter.java,v 1.4 2001/07/25 12:15:11 tcanavan Exp $
 * @author Tim Canavan 
 */

public class KeywordListParameter extends KeywordParameter {

    static final long serialVersionUID = 8401427198565530989L;

    /** 
     * Constructor for keyword list
     */

    public KeywordListParameter(ParamMetaData pMeta) {
        super(pMeta);
    }

    /**
     * Return tooltip text for Parameter class
     */

    public String getToolTipText() {
        return "Select keywords from list";
    }

    /**
     * Append value to the parameters value attribute
     * @param value value to appendsed to base classes value attribute
     */

    public void appendValue(String value) {
        StringBuffer newValue = new StringBuffer(getValue());
        newValue.append(" ");
        newValue.append(value);
        setValue(newValue.toString());
    }

    public char getDBTypeCode() {
        return 'k';
    }

    /**
     * Return the list of valid keywords that can be chooses
     * @return List of valid keywords
     */

    public String[] getKeywords() {
        String value = getValue();
        if(value == null || value.equals("NODEFAULT"))
            return new String[0];
        StringTokenizer token = new StringTokenizer(value);
        String [] keywords =  new String[token.countTokens()];
        int i=0;
        while(token.hasMoreElements()) {
            keywords[i] = token.nextToken();         
            i++;
        }
        return keywords;
    }



    /**
     * This method verifies a string is a keyword value
     * @param value String representing keyword values
     * @return OK,Range of allowed values.
     */

    public boolean verifyValue(String value) {
        if(value == null || value.length() == 0) {
	    return false;
	}
        StringTokenizer token = new StringTokenizer(value);
        while(token.hasMoreElements()) {
            String item = token.nextToken();
            boolean status = super.verifyValue(item);
            if(!status) {
                return status;
            }
        }
        return true;
    }
    
}
