package org.eso.ohs.instruments;

import java.io.IOException;
import java.io.StringReader;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.PafData;
import org.eso.ohs.core.utilities.PafFile;


public class MaskInstrument extends Instrument {


       private String dummyAdpData_ = "PAF.HDR.START\nPAF.TYPE \"paramfile\" ;\nPAF.ID \"3277996668361\" ;\nPAF.NAME \"ThisIsADummyAdm.adm\" ;\nPAF.DESC \"ADM:\" ;\nPAF.CRTE.NAME \"gbr2adm.awk\" ;\nPAF.CRTE.DAYTIM \"2001-08-01T14:19:21.000\" ;\nPAF.LCHG.NAME \" ;\" ;\nPAF.LCHG.DAYTIM \" ;\" ;\nPAF.CHCK.NAME \" ;\" ;\nPAF.CHCK.DAYTIM\" ;\" ;\nPAF.CHCK.CHECKSUM \"00000\" ;\nPAF.HDR.END\nTPL.FILE.DIRNAME \"$INS_ROOT/$INS_USER/ADF\" ;\n";


   /** Standard log channel */
   private static Logger stdlog_ =
       Logger.getLogger( MaskInstrument.class );
    

     /** 
      * The number of masks in a given instrument 
      */
     private int cabinetSize_ = 15;
 

     /**
      * The number of cabinets associated with a give instrument
      */
     private int cabinetCount_ = 4;


     private double TEMP_COEFF =  2.0e-05;


     ////////////////////////////////////////////////////////////////
     // Constructor(s)                                              /
     ////////////////////////////////////////////////////////////////

   
    /**
     * This constructor call the PafFile base class to initially parse the Instrument file into Identifier and 
     * and data pairs. Then it constructs a list of TemplateSignatureSummary classes associated with the 
     * instrument
     *
     * @param instrumentFileName location of instrument file
     * @param tsfDirectory       location of tsf files
     * @param indexFilename      name of index file
     * @exception IOException IO error occurs
     * @exception ClassNotFoundException Class not in index files
     */

    public MaskInstrument(String tsfDirectory,String instrumentFile) throws IOException,IllegalArgumentException {        
        super(tsfDirectory,instrumentFile);
        initMaskData();
    }


    public boolean hasMasks() {
        if(cabinetCount_ != -1 && cabinetSize_ != -1)  
           return true;
        return false;
    } 



    public String getDummyAdm() {
        return dummyAdpData_;
    }

    public String getDummyNaid() {
        PafFile f = null;
        try {
            f = new PafFile(new StringReader(dummyAdpData_));           
        }
        catch(IOException e) {
            stdlog_.error(e);
        }
        return f.getPafIdentifier();
    }



    public void setCabinetSize(int cabinetSize) {
        cabinetSize_ = cabinetSize;
    }

    public void setCabinetCount(int cabinetCount) {
        cabinetCount_ = cabinetCount;
    }

    public int getCabinetCount() {
        return cabinetCount_;
    }

    public int getCabinetSize() {
        return cabinetSize_;
    }


    public double getTempCoeff() {
        return TEMP_COEFF;
    }

 
    public int getMaxMaskCount() {
        return cabinetCount_ * cabinetSize_;
    }



    /** 
     *  Set the data in the instrument class that can be obtained directly 
     *  from the identifier list in the base PafFile class
     */
    protected void initMaskData() {
        stdlog_.debug("Sifetting identifier List for VIMOS");
        PafData[] identifierList = pafFile_.getIdentifierList();         
        for(int i = 0;i<identifierList.length;i++) {  
             String id = identifierList[i].getIdentifier();              
             String value = identifierList[i].getValue();           
             if(id.equals("CABINET.COUNT")) {
                setCabinetCount(new Integer(value).intValue());
             }
             if(id.equals("CABINET.SIZE"))
                 setCabinetSize(new Integer(identifierList[i].getValue()).intValue()); 
             if(id.equals("DUMMY.ADM")) 
                 dummyAdpData_ = new String(identifierList[i].getValue());             
             if(id.equals("TEMP.COEFF")) {  
                 TEMP_COEFF = new Double(identifierList[i].getValue()).doubleValue();
             }
        }
    }
}
