/* $Id: DbaseHandlerCommentCard.java,v 1.1 2005/03/30 09:56:11 tcanavan Exp $ */

package org.eso.ohs.persistence.dbase.phase1;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.core.utilities.TextUtils;
import org.eso.ohs.dfs.CommentCard;
import org.eso.ohs.dfs.StorableObject;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.persistence.StorageManager.UnpackedStorableObject;
import org.eso.ohs.persistence.dbase.DbaseIOHandler;

/**
 * Sql handler to retrieve, to get and to update ReportCard objects
 * on the database.  
 * <br>
 *
 *
 * @author Carlo Boarotto, ESO 21-Mar-2000 
 */
public class DbaseHandlerCommentCard extends DbaseIOHandler {

	private final static int MAX_PHASE1_USER = 1000000;

	/**
	 * Managed class
	 * @return ReportCard class
	 */
	public Class getManagedClass() {
		return CommentCard.class;
	}

	public static long runIdFromId(long id) {
		stdlog_.debug("original run id: " + id);
		long runId = (long) (id / (MAX_PHASE1_USER));
		return runId;
	}

	public static long userIdFromId(long id) {
		stdlog_.debug("original user id: "+ id);
		long userId = (long) (id % (MAX_PHASE1_USER));
		return userId;
	}

	public static long idForPhase1Comment(long runId, long userId) {
		return runId * MAX_PHASE1_USER + userId;
	}

	/** Standard log channel */
	private static Logger stdlog_ =
		Logger.getLogger(DbaseHandlerCommentCard.class);

	/** A reference to the singleton ReportCardSql instance. */
	private static DbaseHandlerCommentCard singleton_ =
		new DbaseHandlerCommentCard();

	public static final String RUN_ID = "run_id";
	public static final String USER_ID = "user_id";
	public static final String COMMENT_SHORT = "comment_short";
	public static final String COMMENT_LONG = "comment_long";
	public static final String COMMENT_FLAG = "comment_flag";
	

	/** Sql select statement to retrieve ReportCard entries. */

	protected final static String comment_fields =
		Phase1SelectStmt.commentsTable.get()
			+ "."
			+ USER_ID
			+ ","
			+ Phase1SelectStmt.commentsTable.get()
			+ "."
			+ RUN_ID
			+ ","
			+ Phase1SelectStmt.commentsTable.get()
			+ "."
			+ COMMENT_SHORT
			+ ","
			+ Phase1SelectStmt.commentsTable.get()
			+ "."
			+ COMMENT_LONG
			+ ","
			+ Phase1SelectStmt.commentsTable.get()
			+ "."
			+ COMMENT_FLAG;
			
			

	/** Sql select statement to retrieve ReportCard entries. */
	protected final static String comment_sql =
		"SELECT "
			+ comment_fields
			+ " FROM "
			+ Phase1SelectStmt.commentsTable.get()
			+ " ";

	/** 
	 * Private constructor: it is a singleton class.
	 */
	public DbaseHandlerCommentCard() {
		super();
	}

	/**
	    * Read  opcReferee data
	    * @param sql statement
	    * @param Object id to be read
	    * @exception SQLException
	    *		    an error occurred during the dbase operation
	    * @exception ObjectNotFoundException
	    *		    the requested object was not found in the database   
	    * @exception ObjectIOException IO Eror occured while reading from the database
	    */

	public UnpackedStorableObject read(Statement stmt, long objId)
		throws ObjectIOException, ObjectNotFoundException, SQLException {
		CommentCard comment = (CommentCard) newInstance();

		UnpackedStorableObject uso = new UnpackedStorableObject(comment);
		String sql = comment_sql + " WHERE " + splitObjIdWhereCondition(objId);

		stdlog_.debug(sql);
		ResultSet rs = stmt.executeQuery(sql);

		if (!rs.next()) { // Only one with this id !!
			throw new ObjectNotFoundException("Object not found in database");
		}
		
		comment.setId(objId);
		setCommentFields(comment, rs);
		if (rs.next()) {
					throw new ObjectIOException("More than one row retrieved.");
				}
		rs.close();

		return new UnpackedStorableObject(comment);
	}

	/**
	  * Read  data
	  * @param sql statement
	  * @param Object id to be read
	  * @exception SQLException
	  *		    an error occurred during the dbase operation
	  * @exception ObjectNotFoundException
	  *		    the requested object was not found in the database   
	  * @exception ObjectIOException IO Eror occured while reading from the database
	  */

	public UnpackedStorableObject[] read(
		Statement stmt,
		String whereClause,
		String otherTables)
		throws ObjectIOException, ObjectNotFoundException, SQLException {

		UnpackedStorableObject[] comment_array = null;
		CommentCard comment = null;
		if (otherTables == null) {
			otherTables = "";
		}
		String sql = comment_sql + otherTables + " WHERE " + whereClause;
		stdlog_.debug(sql);
		ResultSet rs = stmt.executeQuery(sql);

		try {
			Vector comment_vector = new Vector();
			Vector commentIdList = new Vector();
			while (rs.next()) {
				long commentId =
					idForPhase1Comment(rs.getLong(RUN_ID), rs.getLong(USER_ID));

				Long commentIdObj = new Long(commentId);
				// if the id is already selected, skip the next
				if (!commentIdList.contains(commentIdObj)) {
					commentIdList.addElement(commentIdObj);
					comment = (CommentCard) newInstance();
					// New element
					comment_vector.addElement(comment);

					// Set only when attaching the object as a new
					// element
					setCommentFields(comment, rs);

					comment.setId(
						Config.getCfg().tableToUniqueId(
							commentId,
							CommentCard.class));
				} else {
					stdlog_.warn(
						"The comment card "
							+ commentId
							+ " already retrieved. The SQL '"
							+ sql
							+ "' returns duplicate rows.");
				}

			}
			rs.close();

			comment_array = new UnpackedStorableObject[comment_vector.size()];
			// add the affiliation to the user and insert them in array
			for (int i = 0; i < comment_vector.size(); i++) {
				comment = (CommentCard) comment_vector.elementAt(i);
				comment_array[i] = new UnpackedStorableObject(comment);
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}

		return comment_array;
	}

	/**
	 * Throws an ObjectIOException -- the phase 1 database cannot be
	 * changed from within P2PP.
	 * 
	 * @param stmt  JDBC Statement for contacting the dbase
	 * @param id    per-table id of the object to be written.
	 * @param obj   an array: the first element is the BO to be stored.
	 * 
	 * @exception ObjectIOException	operation is not permitted exception
	 * 
	 * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
	 * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
	 */
	public void write(Statement stmt, long id, UnpackedStorableObject obj)
		throws ObjectIOException {

		CommentCard comment = (CommentCard) obj.getObject();
		StringBuffer sb = new StringBuffer();

		sb.append(userIdFromId(id));
		sb.append(",");
		sb.append(runIdFromId(id));
		sb.append(",NULL,\'");
		sb.append(TextUtils.quoteSqlString(comment.getBody()));
		sb.append("\', 1 ");

		String sqlText =
			Phase1SelectStmt.beginTransaction
				+ " \n "
				+ "INSERT INTO "
				+ Phase1SelectStmt.commentsTable.get()
				+ " ("
				+ comment_fields
				+ " ) "
				+ " VALUES ("
				+ sb.toString()
				+ " ) ";

		stdlog_.debug(sqlText);
		try {

			stmt.executeUpdate(sqlText);
		} catch (SQLException e) {
			stdlog_.error("update error", e);
			throw new ObjectIOException("Error executing write: " + sqlText);
		}

	}

	public void update(Statement stmt, StorableObject obj)
		throws ObjectIOException, SQLException {
		CommentCard comment = (CommentCard) obj;

		StringBuffer sb = new StringBuffer();
		sb.append(COMMENT_LONG);
		sb.append(" = \'");
		sb.append(TextUtils.quoteSqlString(comment.getBody()));
		sb.append("\',");
		sb.append(COMMENT_SHORT);
		sb.append(" = NULL ");

		String sql =
			Phase1SelectStmt.beginTransaction
				+ " \n "
				+ "UPDATE  "
				+ Phase1SelectStmt.commentsTable.get()
				+ " SET "
				+ sb.toString()
				+ " WHERE "
				+ splitObjIdWhereCondition(
					Config.getCfg().uniqueToTableId(comment.getId()));
		stdlog_.debug(sql);
		stmt.executeUpdate(sql);

	}

	/**
	 * Throws an ObjectIOException -- the phase 1 database cannot be
	 * changed from within P2PP.
	 * 
	 * @param stmt  JDBC Statement for contacting the dbase
	 * @param id    per-table id of the object to be written.
	 * @param obj   an array: the first element is the BO to be stored.
	 * 
	 * @exception ObjectIOException	operation is not permitted exception
	 * 
	 * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
	 * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
	 */
	public void update(Statement stmt, long id, UnpackedStorableObject obj)
		throws ObjectIOException {

		CommentCard comment = (CommentCard) obj.getObject();

		StringBuffer sb = new StringBuffer();
		sb.append(COMMENT_LONG);
		sb.append(" = \'");
		sb.append(TextUtils.quoteSqlString(comment.getBody()));
		sb.append("\',");
		sb.append(COMMENT_SHORT);
		sb.append(" = NULL ");

		String sqlText =
			Phase1SelectStmt.beginTransaction
				+ " \n "
				+ "UPDATE  "
				+ Phase1SelectStmt.commentsTable.get()
				+ " SET "
				+ sb.toString()
				+ " WHERE "
				+ splitObjIdWhereCondition(id);

		stdlog_.debug(sqlText);
		try {

			stmt.executeUpdate(sqlText);
		} catch (SQLException e) {
			stdlog_.error("update error", e);
			throw new ObjectIOException("Error executing write: " + sqlText);
		}

	}

	/**
	 * Throws an ObjectIOException -- the phase 1 database cannot be
	 * changed from within P2PP.
	 * 
	 * @param stmt  JDBC Statement for contacting the dbase
	 * @param id    per-table id of the object to be written.
	 * 
	 * @exception ObjectIOException	operation is not permitted exception
	 */
	public void delete(Statement stmt, long id) throws ObjectIOException {

		String sqlText =
			Phase1SelectStmt.beginTransaction
				+ " \n "
				+ " \n DELETE  FROM "
				+ Phase1SelectStmt.commentsTable.get()
				+ " WHERE "
				+ splitObjIdWhereCondition(id);

		stdlog_.debug(sqlText);
		try {

			stmt.executeUpdate(sqlText);
		} catch (SQLException e) {
			stdlog_.error("delete error", e);
			throw new ObjectIOException("Error executing delete: " + sqlText);
		}
	}

	private String splitObjIdWhereCondition(long id) {
		return RUN_ID
			+ " = "
			+ runIdFromId(id)
			+ " AND    "
			+ USER_ID
			+ "  = "
			+ userIdFromId(id)
			+ " AND    "
			+ COMMENT_FLAG
			+ "  = 1 ";
	}

	/**
	 * Set the user fields from the result set.
	 * Only for avoiding duplicated code
	 *
	 */
	protected void setCommentFields(CommentCard comment, ResultSet rs)
		throws SQLException {

		comment.setRunId(rs.getLong(RUN_ID));
		comment.setUserId(rs.getLong(USER_ID));
		String commentBody = rs.getString(COMMENT_SHORT);
		if (commentBody == null) {
			commentBody = rs.getString(COMMENT_LONG);
		}
		
		stdlog_.debug("-------------------------------------- >Set comment fields(body) with " + commentBody);
		comment.setBody(commentBody);
	}

}
