package org.eso.ohs.persistence.dbase.phase2;

import java.io.IOException;
import java.io.StringReader;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.HashMap;
import java.util.List;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.Convert;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.CommentDetails;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.dfs.ObservingRun;
import org.eso.ohs.dfs.Readme;
import org.eso.ohs.dfs.ReadmeHistory;
import org.eso.ohs.instruments.AttribComparison;
import org.eso.ohs.instruments.Instrument;
import org.eso.ohs.instruments.InstrumentList;
import org.eso.ohs.instruments.ReadmeAttribComparison;
import org.eso.ohs.instruments.ReadmeAttribute;
import org.eso.ohs.instruments.ReadmeAttributesFactory;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.OHSApplication;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.persistence.dbase.DbaseIOHandler;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import org.xml.sax.InputSource;

/**
 * Class ReadmeFileDBIO This class deals with reading and writing og the readme
 * file to the database. The readmefile is converted from the ReadmeAttribute data
 * structure to an XML file in the database. With each change that is made to the system
 * it records a history of changes that occurs. TODO Javadoc
 * 
 * @version $Revision: 1.1 $
 * @author tcanavan, Nov 8, 2004
 */

// $Id: ReadmeFileDBIO.java,v 1.1 2005/03/30 09:56:12 tcanavan Exp $
public class ReadmeFileDBIO {

    /** Standard log channel */
    private static Logger stdlog_ = Logger.getLogger(ReadmeFileDBIO.class);
    


    private static String obrepDbName = Config.getCfg().getDbName(
            ObservationBlock.getSuffix());
            
            
    public static String readmeFile = obrepDbName + "..readme";
    
    private static String readmeStatusHistory = obrepDbName + "..readme_status_history";
    
    private static String readmeHistory = obrepDbName + "..readme_history";
    
    
    public static String RP = obrepDbName + "..readme_replicate";
    
    private static String OB = obrepDbName + "..obs_blocks";

    
    public static String usgFile = obrepDbName + "..usg_file";
    
    
   
   
   

    
    /**
     * Read readme file from the database.
     * 
     * @param conn
     * @param boId
     * @throws ObjectIOException
     * @throws SQLException
     * @throws IOException
     * @throws JDOMException
     * @throws IOException
     * @throws ObjectNotFoundException
     */

    public static Readme read(Connection conn,ObservingRun obsRun)
            throws SQLException,ObjectIOException, JDOMException, IOException
             {
        setTextSize(conn);
        
        Readme readme = null;
        ResultSet rs;
        long runId = Config.getCfg().uniqueToTableId(obsRun.getId());
        String sqlText = "SELECT " + obrepDbName + "..readme.readme_data, "
                + obrepDbName + "..readme.last_mod_date, "
                + obrepDbName + "..readme.status, "
                + obrepDbName + "..readme.version, " 
                + obrepDbName + "..readme.ip_version from "  
                + obrepDbName + "..readme where " + obrepDbName
                + "..readme.run_id = ?";
        stdlog_.debug(sqlText);
        PreparedStatement pstmt = conn.prepareStatement(sqlText);
        pstmt.setLong(1, runId);
        rs = pstmt.executeQuery();
        boolean found = false;
        String readmeText=null;
        String status=null;
        float ipVersion=-1;
        long last_mod_date;
        int version=-1;
        while (rs.next()) {
            readmeText = rs.getString("readme_data");
            status = rs.getString("status");
            ipVersion = rs.getFloat("ip_version");
            last_mod_date = rs.getLong("last_mod_date");
            version = rs.getInt("version");

            found = true;
        }
        rs.close();
        pstmt.close();
        if(!found) {
            throw new ObjectIOException("No readme file has been checked in for this ObservingRun");
        }
        Readme readmefile = new Readme(ipVersion);
        readmefile.setStatus(status);   
        readmefile.setVersion(version);    
        Instrument inst = InstrumentList.getInstance().getInstrument(obsRun.getInstCode(),ipVersion);
        if(inst == null) {
            throw new ObjectIOException("Instrument package not available for this observing run " + ipVersion);            
        }
        readmefile.setRedmeAttributes(ReadmeAttributesFactory.creteReadmeAttribFromXMLString(inst,readmeText));
        processReadmeHistoryAttributes(conn,readmefile,runId);
        readmefile.setCommentArray(getCommentDetails(conn,runId));
        readmefile.setUsdReadmeFileDetails(USDFileDBIO.read(conn,runId));
        return readmefile;
    }
    
    
    /**
     * Read readme file from the database.
     * 
     * @param conn
     * @param boId
     * @throws ObjectIOException
     * @throws SQLException
     * @throws IOException
     * @throws JDOMException
     * @throws IOException
     * @throws ObjectNotFoundException
     */

    public static Readme readReadmeReplicate(Connection conn,ObservingRun obsRun)
            throws SQLException,ObjectIOException, JDOMException, IOException
             {
        setTextSize(conn);
        
        Readme readme = null;
        ResultSet rs;
        long runId = Config.getCfg().uniqueToTableId(obsRun.getId());
        String sqlText = "SELECT " + RP + ".readme_data, "
                + RP  + ".ip_version "
                + " FROM " + RP +
                " WHERE " + RP +  ".run_id = ?";
        stdlog_.debug(sqlText);
        PreparedStatement pstmt = conn.prepareStatement(sqlText);
        pstmt.setLong(1, runId);
        rs = pstmt.executeQuery();
        boolean found = false;
        String readmeText=null;
        String status=null;
        float ipVersion=-1;
        long last_mod_date;
        int version=-1;
        while (rs.next()) {
            readmeText = rs.getString("readme_data");
            ipVersion = rs.getFloat("ip_version");
            found = true;
        }
        rs.close();
        pstmt.close();
        if(!found) {
            throw new ObjectIOException("Accepted Readme file not available");
        }
        Readme readmefile = new Readme(ipVersion);
        Instrument inst = InstrumentList.getInstance().getInstrument(obsRun.getInstCode(),ipVersion);
        if(inst == null) {
            throw new ObjectIOException("Instrument package not available for this observing run " + ipVersion);            
        }
        readmefile.setRedmeAttributes(ReadmeAttributesFactory.creteReadmeAttribFromXMLString(inst,readmeText));
        readmefile.setCommentArray(getCommentDetails(conn,runId));
        readmefile.setUsdReadmeFileDetails(USDFileDBIO.readReplicate(conn,runId));
        return readmefile;
    }




    private static void setTextSize(Connection conn) throws SQLException {
        Statement stmt = conn.createStatement();
        stmt.execute("set TEXTSIZE " + 1000000);
        stmt.close();
    }

    
    
    private static void processReadmeHistoryAttributes(Connection conn,Readme readme,long runId) throws SQLException {
        ResultSet rs;
        String sqlText = "SELECT " 
        + obrepDbName + "..readme_history.event_time, " 
        + obrepDbName + "..readme_history.version,"  
        + obrepDbName + "..readme_history.change_status," 
        + obrepDbName + "..readme_history.data_value_old," 
        + obrepDbName + "..readme_history.data_value_new,"
        + obrepDbName + "..readme_history.data_type, "
        + obrepDbName + "..readme_attribute_name.readme_attribute_name,"
        + obrepDbName + "..readme_attribute_name.readme_attribute_label"
        + " FROM " + obrepDbName + "..readme_history,"
        + obrepDbName + "..readme_attribute_name"
        + " WHERE " + obrepDbName + "..readme_history.run_id = ?"
        + " AND " + obrepDbName + "..readme_history.readme_attribute_name_id = "
        + obrepDbName + "..readme_attribute_name.readme_attribute_name_id";
        stdlog_.debug(sqlText);
        PreparedStatement pstmt = conn.prepareStatement(sqlText);
        pstmt.setLong(1, runId);
        rs = pstmt.executeQuery();
        Vector historyList = new Vector();
        while (rs.next()) {
            ReadmeHistory history = new ReadmeHistory();
            String dataValueOld = rs.getString("data_value_old");
            if(dataValueOld == null)
                dataValueOld="";
            history.setOldValue(dataValueOld.trim());
            String dataValueNew = rs.getString("data_value_new");
            if(dataValueNew == null)
                dataValueNew="";
            history.setNewValue(dataValueNew.trim());
            history.setStatus(rs.getString("change_status").trim());
            history.setVersion(rs.getInt("version"));
            history.setName(rs.getString("readme_attribute_name").trim());
            history.setLabel(rs.getString("readme_attribute_label").trim());            
            history.setEventTime(rs.getLong("event_time"));
            historyList.addElement(history);
        }
        rs.close();
        pstmt.close();
        stdlog_.debug("Historyt list " + historyList.size());
        ReadmeHistory [] readmeHistoryArray = new ReadmeHistory[historyList.size()];
        historyList.copyInto(readmeHistoryArray);
        readme.setReadmeHistoryItems(readmeHistoryArray);
    }
    
    

    /**
     * Read readme file from the database.
     * 
     * @param conn
     * @param runId
     * @throws ObjectIOException
     * @throws SQLException
     * @throws IOException
     * @throws ObjectNotFoundException
     */

    public static String checkOut(Connection conn, long runId)
            throws SQLException,ObjectIOException
             {
        long newId=DbaseIOHandler.getDbId(conn);
        setTextSize(conn);
        ResultSet rs;
        String readme="";
        int version=-1;
        String sqlText = "SELECT " + obrepDbName + 
                 "..readme.readme_data,"   
                + obrepDbName + "..readme.version from " 
                + obrepDbName + "..readme where " + obrepDbName
                + "..readme.run_id = ?";
        stdlog_.debug(sqlText);
        PreparedStatement pstmt = conn.prepareStatement(sqlText);
        pstmt.setLong(1, runId);
        rs = pstmt.executeQuery();
        boolean found = false;
        while (rs.next()) {
            readme = rs.getString(1);
            version = rs.getInt(2);
            found = true;
        }
        rs.close();
        pstmt.close();
        if(!found) {
            throw new ObjectIOException("No readme file has been checked in for this ObservingRun");
        }
        updateStatusHistory(conn,runId, Readme.CHECKEDOUT_STATUS);
        
        insertStatusHistory(conn,runId, Readme.CHECKEDOUT_STATUS,newId,version);
        return readme;
    }
    
    /**
     * Get count from the database
     * 
     * @param conn
     * @param boId
     * @throws ObjectIOException
     * @throws SQLException
     * @throws IOException
     * @throws ObjectNotFoundException
     */

    public static int getReadmeExists(Connection conn, long runId)
            throws SQLException {
        String sqlText = "SELECT count(*) from "
                + obrepDbName + "..readme where " + obrepDbName
                + "..readme.run_id = ?";
        PreparedStatement pstmt = conn.prepareStatement(sqlText);
        pstmt.setLong(1,runId);
        ResultSet rs = pstmt.executeQuery();
        int count = -1;
        while (rs.next()) {
            count = rs.getInt(1);
        }
        rs.close();
        pstmt.close();
        return count;
    }
    
    /**
     * Get count from the database
     * 
     * @param conn
     * @param boId
     * @throws SQLException
     */

    public static boolean notAcceptedAndCheckedOut(Connection conn, long runId)
            throws SQLException {
        boolean accepted = hasBeenAccepted(conn, runId);
        boolean isCheckedOut = isCheckedOut(conn,runId);
        if(!accepted && isCheckedOut) 
            return true;
        return false;     
    }



    public static boolean hasBeenAccepted(Connection conn, long runId)
        throws SQLException {
        String sqlText = "SELECT count(*) from "
                + readmeStatusHistory + " where "
                + readmeStatusHistory + ".readme_status = ?" 
                + " AND " + readmeStatusHistory + ".run_id = ?";
        PreparedStatement pstmt = conn.prepareStatement(sqlText);
        pstmt.setString(1,Readme.ACCEPTED_STATUS);
        pstmt.setLong(2,runId);
        ResultSet rs = pstmt.executeQuery();
        int count = 0;
        while (rs.next()) {
            count = rs.getInt(1);
        }
        rs.close();
        pstmt.close();
        if(count == 0)
           return false;
        return true;   
    }
    
    public static boolean isCheckedOut(Connection conn, long runId)
        throws SQLException {
        String sqlText = "SELECT " + readmeFile +  ".status from "
                + readmeFile + " where "
                + readmeFile + ".run_id = ?";
        PreparedStatement pstmt = conn.prepareStatement(sqlText);
        pstmt.setLong(1,runId);
        ResultSet rs = pstmt.executeQuery();
        String status=null;
        while (rs.next()) {
            status = rs.getString(1);
        }
        rs.close();
        pstmt.close();
        if(status.equals(Readme.CHECKEDOUT_STATUS))
           return true;
        return false;   
    }


    
    /**
     * Get count from the database
     * 
     * @param conn
     * @param boId
     * @throws ObjectIOException
     * @throws SQLException
     * @throws IOException
     * @throws ObjectNotFoundException
     */

    public static int getHasObs(Connection conn, long runId)
            throws SQLException {
        String sqlText = "SELECT count(*) from "
                + obrepDbName + "..obs_blocks where " + obrepDbName
                + "..obs_blocks.run_id = ?";
        PreparedStatement pstmt = conn.prepareStatement(sqlText);
        pstmt.setLong(1, runId);
        ResultSet rs = pstmt.executeQuery();
        int count = -1;
        while (rs.next()) {
            count = rs.getInt(1);
        }
        rs.close();
        pstmt.close();
        return count;
    }

    

    
    
    

    /**
     * Read readme file from the database.
     * 
     * @param conn
     * @param boId
     * @throws ObjectIOException
     * @throws SQLException
     * @throws IOException
     * @throws ObjectNotFoundException
     */

    public static String write(Connection conn, ObservingRun obsRun)
            throws ObjectIOException, SQLException, IOException,
            ObjectNotFoundException {
        stdlog_.debug("CLASS ReadmeFileDBIO - write:");
        String emailaddress = null;
        checkIPPackage(obsRun);
        StringBuffer sb = new StringBuffer();
        Statement stmt = conn.createStatement();
        long readmeId = -1;
        long runDbId = Config.getCfg().uniqueToTableId(obsRun.getId());
        String readmeData = null;
        int version = -1;
        float latestIPVersion = InstrumentPackageDBIO.getIPVFromObsRun(stmt,runDbId);
        stmt.close();
        
        stdlog_.debug("Latest DB IPVersion " + latestIPVersion);
        
        stdlog_.debug("Observing Run IPVersion " + obsRun.getIPVersion());

        if(latestIPVersion != obsRun.getIPVersion()) {
            throw new ObjectIOException("The instrument package you are using is out of date\n Please select download refresh observing runs and try checking in the readme again");
        }
        setTextSize(conn);
        stmt = conn.createStatement();        
        String sql = "SELECT " + obrepDbName + "..readme.run_id," + obrepDbName
                + "..readme.readme_data," + obrepDbName + "..readme.version,"
                + obrepDbName + "..readme.status" + " from " + obrepDbName
                + "..readme" + " WHERE " + obrepDbName + "..readme.run_id = "
                + runDbId;
        ResultSet rs = stmt.executeQuery(sql);
        while (rs.next()) {
            readmeId = rs.getLong("run_id");
            readmeData = rs.getString("readme_data");
            version = rs.getInt("version");
        }
        rs.close();
        stmt.close();
        Readme readme = obsRun.getReadme();
        stdlog_.debug("Readme IP VERSION " + readme.getIpVersion());

        if (readme == null)
            throw new ObjectIOException(
                    "Readme Not Defined for this Observing Run");
        String isPreImagingStr = "false";
        boolean isPreImaging = false;
        ReadmeAttribute[] readmeAttributes = readme.getRedmeAttributes();
        for (int i = 0; i < readmeAttributes.length; i++) {
            if (readmeAttributes[i].getName().equals("emailaddress"))
                emailaddress = readmeAttributes[i].getValue();
            if (readmeAttributes[i].getName().equals("IsPreImaging"))
                isPreImagingStr = readmeAttributes[i].getValue();
        }
        if (isPreImagingStr.equals("true")) {
            isPreImaging = true;
        }
        String xmlString = readme.toAttributesXMLString();
        long newId=DbaseIOHandler.getDbId(conn);
        if (readmeId == -1) {
            conn.setAutoCommit(false);
            insertReadme(conn, emailaddress, runDbId, readme, isPreImaging, xmlString, newId);
            
        } else {

            SAXBuilder builder = new SAXBuilder();
            StringReader rdr = new StringReader(readmeData.trim());
            Document doc;
            try {
                doc = builder.build(new InputSource(rdr));
                Element element = doc.getRootElement();
                List list = element.getChildren();
                AttribComparison[] comps = ReadmeAttribComparison
                        .compareAttributes(readme.getRedmeAttributes(), list);
                if(comps.length == 0) {
                    conn.setAutoCommit(false);
                    updateStatusHistory(conn,runDbId,readme.getStatus());
                    insertStatusHistory(conn, runDbId,readme.getStatus(),newId,version);
                    return getUSDEmailAddress(conn, runDbId);
                }
                long [] compIds = new long[comps.length];
                for(int i=0;i<comps.length;i++) {
                    compIds[i] = DbaseIOHandler.getDbId(conn);
                }
                HashMap attribIds = extractIds(conn, comps);
                updateReadme(conn, emailaddress, runDbId, version, readme, isPreImaging, xmlString, newId, comps, compIds, attribIds);
            } catch (JDOMException e) {
                throw new ObjectIOException("Error Reading Readme File");
            }
        }
        return getUSDEmailAddress(conn, runDbId);
    }
    
    
    /**
     * @param conn
     * @param emailaddress
     * @param runDbId
     * @param version
     * @param readme
     * @param isPreImaging
     * @param xmlString
     * @param newId
     * @param comps
     * @param compIds
     * @param attribIds
     * @throws SQLException
     */
    private static void updateReadme(Connection conn, String emailaddress, long runDbId, int version, Readme readme, boolean isPreImaging, String xmlString, long newId, AttribComparison[] comps, long[] compIds, HashMap attribIds) throws SQLException {
        conn.setAutoCommit(false);
        addCompHistory(conn, comps, runDbId, version,compIds,attribIds);
        String sqlText = "UPDATE " + obrepDbName + "..readme SET "
                + "status = ? , version = ? , readme_data = ? , email_address = ?,ip_version = ?,last_mod_date = ?"
                + " WHERE run_id=?";

        stdlog_.debug(sqlText);
        PreparedStatement pstmt = conn.prepareStatement(sqlText);
        pstmt.setString(1, Readme.DEFINED_STATUS);
        pstmt.setInt(2, ++version);
        pstmt.setString(3, xmlString);
        pstmt.setString(4, emailaddress);
        pstmt.setFloat(5,readme.getIpVersion());
        pstmt.setLong(6, Convert.getCurrentTime());
        pstmt.setLong(7, runDbId);
        pstmt.execute();
        pstmt.close();
        sqlText = "UPDATE " + obrepDbName + "..usg_file SET pre_imaging = ? "
                + " WHERE run_id = ?";
        pstmt = conn.prepareStatement(sqlText);
        pstmt.setBoolean(1,isPreImaging);
        pstmt.setLong(2, runDbId);
        pstmt.execute();
        pstmt.close();
        insertStatusHistory(conn, runDbId, Readme.DEFINED_STATUS,newId,version);
    }



    /**
     * @param conn
     * @param emailaddress
     * @param runDbId
     * @param readme
     * @param isPreImaging
     * @param xmlString
     * @param newId
     * @throws SQLException
     */
    private static void insertReadme(Connection conn, String emailaddress, long runDbId, Readme readme, boolean isPreImaging, String xmlString, long newId) throws SQLException {
        String sqlText = "INSERT INTO " + obrepDbName + "..readme "
                + " (run_id,status,version,readme_data,email_address,ip_version,last_mod_date)"
                + " VALUES (?,?,?,?,?,?,?)";

        stdlog_.debug(sqlText);
        PreparedStatement pstmt = conn.prepareStatement(sqlText);
        pstmt.setLong(1, runDbId);
        pstmt.setString(2, Readme.DEFINED_STATUS);
        pstmt.setInt(3, 1);
        pstmt.setString(4, xmlString);
        pstmt.setString(5, emailaddress);
        pstmt.setFloat(6,readme.getIpVersion());
        pstmt.setLong(7, Convert.getCurrentTime());
        pstmt.execute();
        pstmt.close();

        sqlText = "INSERT INTO " + obrepDbName + "..usg_file (run_id,pre_imaging)"
                + " VALUES (?,?)";
        pstmt = conn.prepareStatement(sqlText);
        pstmt.setLong(1, runDbId);
        pstmt.setBoolean(2, isPreImaging);
        pstmt.execute();
        pstmt.close();
        insertStatusHistory(conn, runDbId, Readme.DEFINED_STATUS,newId,1);
    }



    /**
     * @param conn
     * @param runDbId
     * @throws SQLException
     * @throws ObjectIOException
     */
    public static void updateStatusHistory(Connection conn, long runDbId,
            String status) throws SQLException {
        String sqlText;
        PreparedStatement pstmt;
        sqlText = "UPDATE " + obrepDbName + "..readme SET status = ?,last_mod_date = ?"
                  + " WHERE run_id = ?";
        stdlog_.debug(sqlText);
        pstmt = conn.prepareStatement(sqlText);
        pstmt.setString(1, status);
        pstmt.setLong(2, Convert.getCurrentTime());
        pstmt.setLong(3,runDbId);
        pstmt.execute();
        pstmt.close();
    }


    /**
     * @param conn
     * @param runDbId
     * @throws SQLException
     * @throws ObjectIOException
     */
    public static void insertStatusHistory(Connection conn, long runDbId,
            String status,long newId,int version) throws SQLException {
        String sqlText;
        PreparedStatement pstmt;
        sqlText = "INSERT INTO " + obrepDbName + "..readme_status_history (run_id,readme_status_history_id,readme_status,event_time,version,application_name) "
                + " VALUES (?,?,?,?,?,?)";
        pstmt = conn.prepareStatement(sqlText);
        pstmt.setLong(1, runDbId);
        pstmt.setLong(2, newId);
        pstmt.setString(3, status);
        pstmt.setLong(4, Convert.getCurrentTime());
        pstmt.setLong(5, version);
        pstmt.setString(6,OHSApplication.getShortDesc());
        pstmt.execute();
        pstmt.close();
    }

    private static void addCompHistory(Connection conn,
            AttribComparison[] comps, long runDbId,int version,long [] compIds,HashMap attribIds)
            throws SQLException {
        String sqlText=null;
        PreparedStatement pstmt=null;
        for(int i=0;i<comps.length;i++) {
            Long attribId = (Long)attribIds.get(comps[i].getName());
            sqlText = "INSERT INTO " + obrepDbName + "..readme_history "
                    + " (run_id,readme_history_id,readme_attribute_name_id,event_time, "
                    + " version,data_type,change_status, "
                    + " data_value_new,data_value_old)"
                    + " VALUES (?,?,?,?,?,?,?,?,?)";
            pstmt = conn.prepareStatement(sqlText);
            pstmt.setLong(1, runDbId);
            pstmt.setLong(2, compIds[i]);
            pstmt.setLong(3, attribId.longValue());
            pstmt.setLong(4, Convert.getCurrentTime());
            pstmt.setInt(5, version);
            pstmt.setString(6, comps[i].getDBType());
            pstmt.setString(7, comps[i].getStatus());
            pstmt.setString(8, comps[i].getNewValue());
            pstmt.setString(9, comps[i].getOldValue());
            pstmt.execute();
            pstmt.close();

        }
    }

    /**
     * Creates a Hashmap of names and dbids keyed on name.
     * @param conn
     * @param comps
     * @return A hashmap of names and dbids keyed on name. 
     * @throws SQLException
     * @throws ObjectIOException
     */
    private static HashMap extractIds(Connection conn, AttribComparison[] comps) throws SQLException, ObjectIOException {
        HashMap attribIds = new HashMap();
        String sqlText=null;
        PreparedStatement pstmt=null;
        for (int i = 0; i < comps.length; i++) {
            long readAttribId = -1;
            sqlText = "SELECT " + obrepDbName
                    + "..readme_attribute_name.readme_attribute_name_id "
                    + " from " + obrepDbName + "..readme_attribute_name "
                    + " WHERE " + obrepDbName
                    + "..readme_attribute_name.readme_attribute_name = ?";
            stdlog_.debug(sqlText);
            pstmt = conn.prepareStatement(sqlText);
            pstmt.setString(1, comps[i].getName());
            stdlog_.debug(comps[i].getName());
            ResultSet rs = pstmt.executeQuery();
            while (rs.next()) {
                readAttribId = rs.getLong(1);
            }
            rs.close();
            pstmt.close();
            if (readAttribId == -1) {
                readAttribId = DbaseIOHandler.getDbId();
                sqlText = "INSERT INTO " + obrepDbName + "..readme_attribute_name (readme_attribute_name_id,readme_attribute_name,readme_attribute_label) VALUES (?,?,?)";
                pstmt = conn.prepareStatement(sqlText);
                pstmt.setLong(1, readAttribId);
                pstmt.setString(2, comps[i].getName());
                pstmt.setString(3, comps[i].getLabel());
                pstmt.execute();
            }
            attribIds.put(comps[i].getName(),new Long(readAttribId));

        }
        return attribIds;
    }



    /**
     * @param obsRun
     * @throws IOException
     */
    private static void checkIPPackage(ObservingRun obsRun) throws IOException {
        Instrument inst = InstrumentList.getInstance().getInstrument(
                obsRun.getInstCode(), obsRun.getIPVersion());
        // Try and find the instrument again.
        inst = InstrumentList.getInstance().getInstrument(obsRun.getInstCode(),
                obsRun.getIPVersion());
        if (inst == null) {
            stdlog_.debug("IP Template Not Found: " + obsRun.getInstCode()
                    + " version: " + obsRun.getIPVersion());

            String versionMessage = InstrumentList.isInSingleIPMode() ? ""
                    : " version " + obsRun.getIPVersion();
            throw new IOException("Instrument Package for "
                    + obsRun.getInstCode() + versionMessage
                    + " could not be found.");
        }
    }
    

    public static String getUSDEmailAddress(Connection conn, long dbRunId)
            throws SQLException {

        String sql = "SELECT " + obrepDbName + "..usg_staff.email " + " from "
                + obrepDbName + "..run_states," + obrepDbName + "..usg_staff"
                + " WHERE " + obrepDbName + "..run_states.run_id = ?" + " AND "
                + obrepDbName + "..run_states.cs_acronym" + " = " + obrepDbName
                + "..usg_staff.cs_acronym";
        stdlog_.debug(sql);
        PreparedStatement pstmt = conn.prepareStatement(sql);
        pstmt.setLong(1, dbRunId);
        ResultSet rs = pstmt.executeQuery();
        String emailaddress = null;
        while (rs.next()) {
            emailaddress = rs.getString(1);
        }
        rs.close();
        pstmt.close();
        return emailaddress;
    }
    
    public static void setReadmeFileStatus(Connection con,String status,long runId) throws SQLException, ObjectIOException {
        long runDbId = Config.getCfg().uniqueToTableId(runId);
        long newId=DbaseIOHandler.getDbId(con);
        setTextSize(con);
        String sql = "SELECT " 
        + readmeFile + 
        ".readme_data," 
        + readmeFile 
        + ".version,"
        + readmeFile 
        + ".status,"
        + readmeFile 
        + ".ip_version " 
        + " from " 
        + readmeFile +
        " WHERE " 
        + readmeFile + ".run_id = ?";
        PreparedStatement pstmt = con.prepareStatement(sql);
        pstmt.setLong(1, runDbId);
        ResultSet rs = pstmt.executeQuery();
        int count = -1;
        String readmeData=null;
        String dbStatus=null;
        boolean hasReadme=false;
        int version=-1;
        float ip_version=-1;
        while(rs.next()) {
           readmeData=rs.getString("readme_data");
           dbStatus=rs.getString("status");
           version=rs.getInt("version");
           ip_version=rs.getFloat("ip_version");
           hasReadme=true;         
        }
        rs.close();
        pstmt.close();
        if(!hasReadme) {
            throw new SQLException("No Readme file available for this Observong Run");
        }
        if(status.equals(dbStatus)) {
            throw new SQLException("Readme has already status " + dbStatus);
        }
        con.setAutoCommit(false);
        updateStatusHistory(con,runDbId,status);
        insertStatusHistory(con,runDbId,status,newId,version);
        if(status.equals(Readme.ACCEPTED_STATUS)) {
            sql = "SELECT count(*) from " + RP +
                    " WHERE " + RP + ".run_id = ?";
            pstmt = con.prepareStatement(sql);
            pstmt.setLong(1, runDbId);
            rs = pstmt.executeQuery();
            count = -1;
            while(rs.next()) {
               count = rs.getInt(1);         
            }
            rs.close();
            pstmt.close();
            if(count == 0) {
                sql = "INSERT INTO " + RP + " (run_id,readme_data,ip_version) " + " VALUES (?,?,?)";
                pstmt = con.prepareStatement(sql);
                pstmt.setLong(1, runDbId);
                pstmt.setString(2,readmeData);
                pstmt.setFloat(3,ip_version);
                pstmt.execute();
                pstmt.close();
            } else {
                sql = "UPDATE " + RP + " SET readme_data = ?,ip_version = ? WHERE run_id = ?";
                pstmt = con.prepareStatement(sql);
                pstmt.setString(1,readmeData);
                pstmt.setFloat(2,ip_version);
                pstmt.setLong(3, runDbId);
                pstmt.execute();
                rs.close();
                pstmt.close();
            }
        }        
    }
    
    public static void delete(Connection con,long runId) throws SQLException  {
        Statement stmt = con.createStatement();
        long dbId = Config.getCfg().uniqueToTableId(runId);
        String sql = "delete " + readmeStatusHistory + " where " + readmeStatusHistory + ".run_id = " + dbId;
        stmt.executeUpdate(sql);
        sql = "delete  " + readmeHistory + " where " + readmeHistory + ".run_id = " + dbId;
        stmt.executeUpdate(sql);
        sql = "delete  " + RP + " where " + RP + ".run_id = " + dbId;
        stmt.executeUpdate(sql);
        sql = "delete  " + USDFileDBIO.usgSupportNotes + " where " + USDFileDBIO.usgSupportNotes + ".run_id = " + dbId;
        stmt.executeUpdate(sql);
        sql = "delete  " + usgFile + " where " + usgFile + ".run_id = " + dbId;
        stmt.executeUpdate(sql);
        sql = "delete  " + readmeFile + " where " + readmeFile + ".run_id = " + dbId;
        stmt.executeUpdate(sql);
        stmt.close();      
    }
    
    
    public static CommentDetails [] getCommentDetails(Connection con,long dbId) throws SQLException {
        String sql = "SELECT " +
                      OB + ".ob_id," +
                      OB + ".item_name," +
                      OB + ".inst_comments," +
                      OB + ".userComments," + 
                      OB + ".userPriority," +
                      OB + ".status FROM " +
                      OB +  " WHERE " + OB + ".run_id = ? " +
                      " order by " + OB + ".ob_id";
        stdlog_.debug(sql);
        PreparedStatement pstmt = con.prepareStatement(sql);
        pstmt.setLong(1,dbId);
        ResultSet rs = pstmt.executeQuery();
        Vector vec = new Vector();
        while(rs.next()) {
            boolean hasComments=false;
            CommentDetails commentDetails = new CommentDetails();
            commentDetails.setObId(rs.getLong("ob_id"));
            commentDetails.setName(rs.getString("item_name"));
            String instComments = rs.getString("inst_comments");
            if(instComments != null && instComments.length() > 0) {
                commentDetails.setInstSpecificComment(instComments.trim());
                hasComments=true;
            }
            String userObComment= rs.getString("userComments").trim();
            if(userObComment != null && userObComment.length() > 0) {
                commentDetails.setUserComment(rs.getString("userComments").trim());
                hasComments=true;
            }
            commentDetails.setUserComment(rs.getString("userComments").trim());
            commentDetails.setUserPriority(rs.getString("userPriority").trim());
            if(hasComments)
                vec.addElement(commentDetails);        
        }
        CommentDetails [] commentDetails = new CommentDetails[vec.size()];
        vec.copyInto(commentDetails);
        return commentDetails;
    }
    
    
}
