package org.eso.ohs.phase2.apps.masktracker;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JCheckBox;
import javax.swing.JPopupMenu;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.apache.log4j.Logger;
import org.eso.ohs.core.dbb.client.DbbPanel;
import org.eso.ohs.core.dbb.client.DbbSelectionList;
import org.eso.ohs.core.dbb.sql.DbbSqlChunk;
import org.eso.ohs.core.dbb.sql.DbbSqlOperator;
import org.eso.ohs.core.utilities.TextUtils;
import org.eso.ohs.dfs.Mask;
import org.eso.ohs.dfs.MaskQueue;
import org.eso.ohs.phase2.apps.masktracker.actions.ManufacturingReportAction;


/**
 * The QueueItemsBView class is a DB search engine used for 
 * displaying queues of OBs according to a user-definable set of 
 * properties
 * @author G Passera
 * @author Tim Canavan 
 * @author cmavros Add call to unsubscribe execution action.
 * @author cmavros 1709/OT Call ExecutionSequenceAction 
 *               class to implement progress bar
 *
 * @version $Id: MaskManExecSequence.java,v 1.13 2005/12/04 23:50:02 tcanavan Exp $
 *
 * @author cmavros Add call to unsubscribe execution action.
 * @author cmavros 1709/OT Call ExecutionSequenceAction 
 *               class to implement progress bar
 *
 * @version $Id: MaskManExecSequence.java,v 1.13 2005/12/04 23:50:02 tcanavan Exp $
 *
 */

public class MaskManExecSequence extends MaskExecSequence {

    private static Logger stdlog_ =
	Logger.getLogger(MaskManExecSequence.class );
   
    private JCheckBox selected_ ;


   private SelectedManListener selectedManListener_ = null;
   
  

    /*
   * Default constructor
   * @author Giorgio Passera <DMD/ESO> <gpassera@eso.org>
   *
   */
    public MaskManExecSequence(MaskQueue newQueue,MaskQueueView mkView) {
        super(newQueue,null,mkView);
        selectedManListener_ = new SelectedManListener();
        view_.addListSelectionListener(selectedManListener_);
    }


    public void buildRows() {
        super.buildRows();       
        DbbSqlOperator op_in = null;
        try {
            op_in = new DbbSqlOperator("in");
        }
        catch( Exception e ) { 
            stdlog_.error( e ); 
        }
        String label_name = "Status";
        String column_name = "mask_status";
        DbbPanel rowPanel = getRowPanel();
        DbbSqlChunk c = new DbbSqlChunk(masks_table, column_name, label_name, op_in, String.class);
        Object[] statusValues = {Mask.NO_MASK,Mask.PENDING_MANUFACTURED,Mask.MASK_REMANUFACTURE};
        String[] statusDispValues = {Mask.getDisplayStatusValue(Mask.NO_MASK),
                                     Mask.getDisplayStatusValue(Mask.PENDING_MANUFACTURED),
                                     Mask.getDisplayStatusValue(Mask.MASK_REMANUFACTURE)};
        DbbSelectionList status = new DbbSelectionList(label_name, c,statusValues, statusDispValues);
        rowPanel.addWidget(status);
        status.set("(" + "'" + TextUtils.quoteSqlString(Mask.NO_MASK) + "'"+ ",'" + 
	               TextUtils.quoteSqlString(Mask.PENDING_MANUFACTURED) + "'" + 
	               ",'" + TextUtils.quoteSqlString(Mask.MASK_REMANUFACTURE) + "'"  + ")");
        //status.getList().addSelectionInterval(0,2);
        status.fireDbbWidgetAction();
	    view_.setWherePanel(rowPanel, "North");
    } 


     public JPopupMenu createPopupMenu() {
        JPopupMenu popupMenu = new JPopupMenu();       
        popupMenu.add(new ManufacturingReportAction(view_,true,"Mask Manufacturing Statistics"));
        MaskManufacturing.getInstance().setSelectedNaidList(getSelectedMaskNaids());          
        return popupMenu;
    }

    public void buildToolBar() {
         super.buildToolBar();
         selected_ = new JCheckBox("Selected");
         toolBar_.add(selected_);
         //selected_.setSelected(true);
         selected_.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                MaskManufacturing.getInstance().setSingleSelect(selected_.isSelected());                   
                //MaskManufacturing.getInstance().generateOrder(5);
                //MaskManufacturing.getInstance().setSelectedNaidList(getSelectedMaskNaids());
            } 
         });
    } 


   private class SelectedManListener implements ListSelectionListener {
        
        public SelectedManListener() { }
        
        public void valueChanged(ListSelectionEvent ev) {    
            if (ev.getValueIsAdjusting()) {
                return;
            }
            ListSelectionModel lsm = (ListSelectionModel)ev.getSource();
            currentRow_ = lsm.getMinSelectionIndex();
            if(!lsm.isSelectionEmpty() && currentRow_ >= 0) {
                  stdlog_.debug("Setting Mask List" + getSelectedMaskNaids().length);
                  MaskManufacturing.getInstance().setSelectedNaidList(getSelectedMaskNaids());                 
            }
                     
        }
    } 
}
