package org.eso.ohs.phase2.apps.masktracker;

import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;

import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.KeyStroke;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.eso.ohs.core.utilities.OHSKeys;
import org.eso.ohs.dfs.MaskQueue;
import org.eso.ohs.persistence.AppConfig;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.OHSApplication;
import org.eso.ohs.phase2.apps.masktracker.actions.ProcessReportAction;
import org.eso.ohs.phase2.apps.masktracker.actions.ShowMaskDbaseAction;
import org.eso.ohs.phase2.apps.ot.actions.QuitAction;
import org.eso.ohs.phase2.apps.ot.actions.ShowDbaseAction;
import org.eso.ohs.phase2.apps.ot.gui.ExecutionSequenceBView;
import org.eso.ohs.phase2.apps.ot.gui.OTMainView;
import org.eso.ohs.phase2.apps.ot.gui.OTViewManager;
import org.eso.ohs.phase2.apps.ot.gui.QueueView;

/**
 * This class provides the main view for the OT application.
 *
 * @version $Revision: 1.21 $
 *
 * @author Giorgio Passera <ESO\DMD> <gpassera@eso.org>
 * @author cmavros 19/02/02, 1640/OT,  Add Refresh Menu option
 */

public class MaskTrackerView extends OTMainView {
    
    public void buildFileMenu() {
        fileMenu = new JMenu("Queues");
        menuBar.add(fileMenu);
        newMenuItem = fileMenu.add(newQueueAction);
        openMenuItem = fileMenu.add(openQueueAction);
        reloadMenuItem = fileMenu.add(reloadQueueAction);
        refreshMenuItem = fileMenu.add(refreshQueueAction);
        closeMenuItem = fileMenu.add(closeQueueAction);
        fileMenu.addSeparator();
        saveMenuItem = fileMenu.add(saveQueueAction);
        //dumpMenuItem = fileMenu.add(dumpQueueAction);
        //JMenuItem exportMenuItem = new JMenuItem("Export...");
        //exportMenuItem.setEnabled(false);
        //fileMenu.add(exportMenuItem);
        deleteMenuItem = fileMenu.add(deleteQueueAction);
        fileMenu.addSeparator();
        repositoryBrowserMenuItem = fileMenu.add(new ShowDbaseAction(this));
        fileMenu.add(OBHistoryAction);
            maskBrowserMenuItem = fileMenu.add(new ShowMaskDbaseAction(this));
        //fileMenu.addSeparator();
        quitMenuItem = fileMenu.add(new QuitAction(this));
            processReportMenuItem = new JMenuItem("Process Report");
            fileMenu.add(new ProcessReportAction(this, "Process Reports"));
        fileMenu.add(repositoryBrowserMenuItem);
        fileMenu.addSeparator();
        fileMenu.add(quitMenuItem);
        newMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N,
                ActionEvent.CTRL_MASK));
        newMenuItem.setMnemonic(KeyEvent.VK_N + KeyEvent.CTRL_MASK);

        openMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O,
                ActionEvent.CTRL_MASK));
        openMenuItem.setMnemonic(KeyEvent.VK_O + KeyEvent.CTRL_MASK);

        repositoryBrowserMenuItem.setAccelerator(KeyStroke.getKeyStroke(
                KeyEvent.VK_B, ActionEvent.CTRL_MASK));
        repositoryBrowserMenuItem.setMnemonic(KeyEvent.VK_B
                + KeyEvent.CTRL_MASK);

            maskBrowserMenuItem.setAccelerator(KeyStroke.getKeyStroke(
                    KeyEvent.VK_M, ActionEvent.CTRL_MASK));
            maskBrowserMenuItem.setMnemonic(KeyEvent.VK_M + KeyEvent.CTRL_MASK);

        quitMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Q,
                ActionEvent.CTRL_MASK));
        quitMenuItem.setMnemonic(KeyEvent.VK_Q + KeyEvent.CTRL_MASK);

        /** cmavros 25-10-01 [1506] Add Reports to Execution Sequence */
        createReportsMenu(ExecutionSequenceBView.getInstance().getView());
        createFCPrintMenu(ExecutionSequenceBView.getInstance().getView());
        setEnabledReports(false);
        setEnabledFCPrint(false);

    }

    
    
    public void addTableListeners() {
        addListSelectionListener(new QueueListListener(), queueTable_);
        addListSelectionListener(new OpenQueueListListener(), openQueueTable_);
    }

    
    public class OpenQueueListListener implements ListSelectionListener {

        public OpenQueueListListener() {
        }

        public void valueChanged(ListSelectionEvent ev) {
            if (ev.getValueIsAdjusting()) {
                return;
            }
            ListSelectionModel lsm = (ListSelectionModel) ev.getSource();
            int selectedRow = lsm.getMinSelectionIndex();
            if (!lsm.isSelectionEmpty() && selectedRow >= 0
                    && tabPane_.getSelectedIndex() == TAB_PANE_OPEN_QUEUE_ID) {
                setEnabledOpenQueueList(true);
                long id = openQueueModel_.getQueueId(selectedRow);
                OTViewManager manager = OTViewManager.getOTViewMgr();
                QueueView queueView = manager.getQueueView(id);
                if (queueView != null) {
                    queueView.setVisible(true);
                    queueView.setState(JFrame.NORMAL);
                    if (AppConfig.getAppConfig().getBoolean(
                            OHSKeys.MaskTrackerEnabledKey)) {
                        MaskQueueView mkView = (MaskQueueView) queueView;
                        MaskListener mskLnr = MaskListener.getInstance();
                        if (mkView.getTabPaneIndex() == MaskQueueView.TAB_MAN) {
                            mskLnr.setType(MaskManufacturing.getInstance());
                        }
                        if (mkView.getTabPaneIndex() == MaskQueueView.TAB_INSERTED) {
                            mskLnr.setType(MaskInsertion.getInstance());
                        }
                        if (mkView.getTabPaneIndex() == MaskQueueView.TAB_DISCARDED) {
                            mskLnr.setType(MaskDiscarding.getInstance());
                        }
                    }
                }
            } else {
                setEnabledOpenQueueList(false);
            }

        }

    }

    /**
     * Constructor.
     */
    public MaskTrackerView() {
        super();
        setTitle(
            "Masktracker V."
                + OHSApplication.getVersion()
                + " User="
                + Config.getCfg().getUserName()
                + " Server="
                + AppConfig.getAppConfig().getDbServer());
        MTViewManager.setInstance(MTViewManager.getInstance());
        MTViewManager.getOTViewMgr().setOTMainView(this);
        // We do not need the execution sequnece for tha MaskTracker
        tabPane_.removeTabAt(TAB_PANE_EXEC_ID);
    }

    public Class getDisplayedObjectClass() {
        return MaskQueue.class;
    }   
}
