package org.eso.ohs.phase2.apps.ot.actions;

import java.awt.event.ActionEvent;

import javax.swing.JOptionPane;
import javax.swing.JTable;

import org.apache.log4j.Logger;
import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.gui.baseaction.IncrementalAction;
import org.eso.ohs.core.gui.baseaction.IncrementalActionAutomaton;
import org.eso.ohs.core.utilities.ObjUtils;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.Queue;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.ObjectManager;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.phase2.apps.ot.gui.OTMainView;
import org.eso.ohs.phase2.apps.ot.gui.OTViewManager;
import org.eso.ohs.phase2.apps.ot.gui.QueueSummaryModel;
import org.eso.ohs.phase2.apps.ot.gui.QueueView;


/**
 * Action class to delete a queue.
 *
 * <p>
 *
 * $Id: DeleteQueueAction.java,v 1.9 2005/02/11 10:31:49 tcanavan Exp $
 * <br>
 * Author: Bob Kemp
 */
public class DeleteQueueAction extends ActionSuperclass {


    private JTable table_;

   /** Standard log channel */
   private static Logger stdlog_ =
       Logger.getLogger( DeleteQueueAction.class );

  //private QueueSummaryModel openQueueModel_;
     
    private QueueSummaryModel queueModel_;

    private OTMainView view_;

    IncrementalActionAutomaton actionAutomaton_;

    public DeleteQueueAction(OTMainView view,JTable table, String name) {
	super(view,name);
        view_ = view;
        table_ = table;
        queueModel_ = (QueueSummaryModel)table_.getModel();      
    }


    private long [] getSelectedQueueIds() {           
        int [] rows = table_.getSelectedRows();
        long [] queueIds = new long[rows.length];
        QueueSummaryModel model = (QueueSummaryModel)table_.getModel();
        for(int i=0;i<rows.length;i++) {
             queueIds[i] = model.getQueueId(rows[i]);
	}
        return queueIds;
   }


    public void actionPerformedImpl(ActionEvent ev) {         
	stdlog_.debug("@@@@@@@@@@@@@@ Delete Action Event " + ev);
        QueueSummaryModel model = (QueueSummaryModel)table_.getModel();
        int [] rows = table_.getSelectedRows();
        long [] queueIds = getSelectedQueueIds();
        	
	actionAutomaton_ = new IncrementalActionAutomaton(
                    new IncrementalDeleteQueue(queueIds,rows),
		    view_,
                    "Deleting...."
		);
        	       
    }




 /**
     * Utility class to delete a number o queues, one at a time.
     * The user is also given the opportunity to stop the process using
     * a dialog box.
     */
    public class IncrementalDeleteQueue implements IncrementalAction {


	/** array of ids of objects to be duplicated */
	private long [] queueIds_;

        private int [] rows_;
  
        private int objCount_;

        private int currrentRow_;   


	/**
	 * Constructor.
	 * 
	 * @param frm	    window on which to base any dialog boxes.
	 * @param device    media containing the objects to be duplicated
	 * @param deepCopy  should we duplicate attached objects?
	 * @param dir	    directory containing the ObsBlocks
	 * @param objType   type of object to be duplicated
	 * @param idLongs   array of ids of objects to be duplicated
	 */
	public IncrementalDeleteQueue(long [] queueIds,int [] rows) {	   
	    queueIds_  = queueIds;         
	    objCount_ = queueIds.length;
            rows_ = rows;	   		
                
	}

       /**
        * Number of steps in the action that are still remaining to be done.
        * 
        * @return number of steps remaining (eg objects to acted upon)
        */
        public int getNumStepsRemaining(int counter) {
            Long [] queueObjs = new Long[queueIds_.length];
	    for(int i=0;i<queueIds_.length;i++) {
                queueObjs[i] = new Long(queueIds_[i]);
            }
            return ObjUtils.howManyNotNull(queueObjs, counter);
        }

        /**
         * Method to perform one incremental step of a larger action.
         * 
         * @param stepNo	Counter to indicate the n'th step of a larger action
         */
        public void performStep(int counter) {           
	    doDeletion(queueIds_[counter]);            

        }

        /**
         * Hook for initialisation of action object.
         */
        public void startOfAction() {
        }

        /**
         * Hook for cleanup of action object.
         */
        public void endOfAction() {             
            //table_.setRowSelectionInterval(0,0);
        }


	/**
	 * TDB
	 * 
	 * @param id	    TDB
	 */
	public void doDeletion(long id) {	  
	    //long queueId  = Config.getCfg().tableToUniqueId(id,Queue.class);
            OTViewManager manager = OTViewManager.getOTViewMgr();
            QueueView queueManager = manager.getQueueView(id);
	    String message = "Do you really want to remove queue " + queueModel_.getQueueName(id) + " ? ";
	    int n = JOptionPane.showConfirmDialog(table_,message,"Delete Queue",JOptionPane.YES_NO_OPTION);                
	    if (n == JOptionPane.YES_OPTION) {
	        try {
                    ObjectManager objMgr = ObjectManager.getObjectManager();
                    if(queueManager != null) {                         
			 OTViewManager otMgr = OTViewManager.getOTViewMgr();
                         int index = manager.getQueueViews().indexOf(queueManager);
			 //queueManager.closeQueue();
			 queueManager.closeQueueWithoutSaving();
                         if(actionAutomaton_.getProgressDialog()!=null)
                             actionAutomaton_.getProgressDialog().setVisible(true);                          
                    }
                    Class c = Config.getCfg().getClassFromQueueId(id);        
		    Queue queue = (Queue)objMgr.getBusObj(Media.DBASE,id,c);
	            objMgr.deleteBusObj(Media.DBASE,id,c);
                }
		catch (ObjectNotFoundException ex) {
                    /** cmavros 11-MAR-02 OT V2_3 beta1 comment out stacktrace do to Karmin's comment 35.
		     * ex.printStackTrace();
                     */
		}
		catch (ObjectIOException ex) {
		    ex.printStackTrace();
		}	       
	    }
        }
    }
}

