package org.eso.ohs.phase2.apps.ot.actions;

import java.awt.event.ActionEvent;
import java.lang.reflect.InvocationTargetException;
import java.sql.SQLException;

import javax.swing.JOptionPane;

import org.apache.log4j.Logger;
import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.gui.widgets.ErrorMessages;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.MaskQueue;
import org.eso.ohs.dfs.Queue;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.ObjectManager;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.phase2.apps.ot.gui.OTViewManager;
import org.eso.ohs.phase2.apps.ot.gui.QueueView;

/**
 * Action class to save a queue as another name and aabandon the 
 * edits of the old one.
 * <p>
 *
 * $Id: SaveAsQueueAction.java,v 1.13 2005/10/12 11:47:07 tcanavan Exp $
 * <br>
 * Author: Bob Kemp
 */
public class SaveAsQueueAction extends ActionSuperclass {

   /** Standard log channel */
   private static Logger stdlog_ =
       Logger.getLogger( SaveAsQueueAction.class );


    private QueueView queueView_;
 

    public SaveAsQueueAction(QueueView queueView, String name) {
	super(queueView,name);
        queueView_ = queueView;
    }


  
   public void actionPerformedImpl(ActionEvent ev) {
       String inputValue =  JOptionPane.showInputDialog( "Please enter a new queue name" );
       if(inputValue != null) {
           try {
               ObjectManager objMgr = ObjectManager.getObjectManager();
               queueView_.updateQueue();
               Queue oldQueue = queueView_.getQueue();
            
               Queue newQueue;
               if(oldQueue instanceof MaskQueue)
                    newQueue  = new MaskQueue();
               else
                    newQueue = new Queue();
               objMgr.register(Media.DBASE,objMgr.getRoot(Media.DBASE),newQueue); 
               oldQueue.copyProperties(oldQueue.simpleProperties(),newQueue);              
               newQueue.setName(inputValue);              
               objMgr.putBusObj(newQueue);
               newQueue.setUpdate(false);

               Class c = Config.getCfg().getClassFromQueueId(oldQueue.getId());
               objMgr.refreshBusObj(Media.DBASE,newQueue.getId(),c);
   	           //objMgr.refreshBusObj(Media.DBASE,oldQueue.getId(),c);
               newQueue.setUpdate(false);
               // Force setSize to fire an event that allows the summary model to update
               newQueue.setSize(oldQueue.getSize());
               //stdlog_.debug("Old Queue " + oldQueue.getSize());
               OTViewManager otMgr = OTViewManager.getOTViewMgr();
               QueueView mgr = otMgr.getQueueView(oldQueue.getId());              
               objMgr.forgetBusObj(Media.DBASE,oldQueue.getId(),c);
               mgr.setQueue(newQueue);
               mgr.getQueue().addPropertyChangeListener(mgr);
               newQueue.setIsNew(false);
               mgr.updateTitle(); 
               
           }
           catch(SQLException e) {
               e.printStackTrace();
	       stdlog_.error(e);
	   }
           catch(ObjectIOException e) {
               e.printStackTrace();
	       ErrorMessages.announceIOError(null,e);
               stdlog_.error(e);
           } 
           catch(ObjectNotFoundException e) {
               e.printStackTrace();
               ErrorMessages.announceNoObject(null,e);
           }
           catch(InvocationTargetException e) {
               stdlog_.error(e);
           }
           catch(IllegalAccessException e) {
               stdlog_.error(e);
           }
           catch(NoSuchMethodException e) {
               stdlog_.error(e);
           }
       }                 
   }
}

