package org.eso.ohs.phase2.apps.ot.gui;

import java.awt.Container;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

import org.eso.ohs.core.gui.models.LimitedCharsModel;
import org.eso.ohs.dfs.CalibrationBlock;

/** The CommentDialog dailog is used to modify the following OB property
 *  observer comments , status , obs grade
 *
 *  @version $Revision: 1.7 $
 *  @author  G.Passera, DMD/ESO, <gpassera@eso.org>
 */
public class CommentDialog extends JDialog {

    private JLabel obsCommentsLabel = new JLabel("Observer comments");
    private JLabel statusLabel = new JLabel("Status");
    private JLabel qcGradeLabel = new JLabel("QC grade");
    private JComboBox statusComboBox = new JComboBox(CalibrationBlock.markLabels);
    private JComboBox qcGradeComboBox;
    private JTextArea obsCommentsTextField =
        new JTextArea(new LimitedCharsModel(), "", 5, 10);

    private JButton cancelButton_ = new JButton("Cancel");
    private JButton okButton_ = new JButton("OK");

    private JPanel contentPanel = new JPanel();

    private int returnValue_ = JOptionPane.CANCEL_OPTION;

    /** Default constructor*/
    public CommentDialog() {
        super((java.awt.Frame) null, "Set Observer Parameters", true);

        Vector vec = new Vector();
        vec.addElement(new String(" "));
        for (int i = 0; i < CalibrationBlock.QC_GRADE_VALUES.length; i++) {
            vec.addElement(CalibrationBlock.QC_GRADE_VALUES[i]);
        }
        qcGradeComboBox = new JComboBox(vec);

        //builds the panel layout
        JPanel content = new JPanel();
        content.setLayout(new GridBagLayout());
        Container c = (Container) content;
        JScrollPane pane = new JScrollPane(obsCommentsTextField);

        GridBagAdder.add(
            c,
            statusLabel,
            0,
            0,
            2,
            1,
            0,
            0,
            GridBagConstraints.HORIZONTAL,
            GridBagConstraints.CENTER);

        GridBagAdder.add(
            c,
            qcGradeLabel,
            0,
            1,
            2,
            1,
            0,
            0,
            GridBagConstraints.HORIZONTAL,
            GridBagConstraints.CENTER);

        GridBagAdder.add(
            c,
            obsCommentsLabel,
            0,
            2,
            2,
            1,
            0,
            0,
            GridBagConstraints.HORIZONTAL,
            GridBagConstraints.CENTER);

        GridBagAdder.add(
            c,
            statusComboBox,
            2,
            0,
            2,
            1,
            1,
            0,
            GridBagConstraints.HORIZONTAL,
            GridBagConstraints.CENTER);

        GridBagAdder.add(
            c,
            qcGradeComboBox,
            2,
            1,
            2,
            1,
            1,
            0,
            GridBagConstraints.HORIZONTAL,
            GridBagConstraints.CENTER);

        GridBagAdder.add(
            c,
            pane,
            2,
            2,
            2,
            4,
            1,
            0,
            GridBagConstraints.BOTH,
            GridBagConstraints.CENTER);

        GridBagAdder.add(
            c,
            okButton_,
            0,
            10,
            1,
            1,
            0,
            0,
            GridBagConstraints.BOTH,
            GridBagConstraints.CENTER);

        GridBagAdder.add(
            c,
            cancelButton_,
            3,
            10,
            1,
            1,
            0,
            0,
            GridBagConstraints.VERTICAL,
            GridBagConstraints.EAST);

        getContentPane().add(c);

        okButton_.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                returnValue_ = JOptionPane.OK_OPTION;
                setVisible(false);
            }
        });
        cancelButton_.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                returnValue_ = JOptionPane.CANCEL_OPTION;
                setVisible(false);
            }
        });
        this.pack();
    }

    public int getUserChoice() {
        return returnValue_;
    }

    public void setQCVisible(boolean visible) {
        qcGradeLabel.setVisible(visible);
        qcGradeComboBox.setVisible(visible);
    }

    /**Returns the selected status*/
    public String getStatus() {
        return CalibrationBlock.getStatusCode(statusComboBox.getSelectedIndex());
    }

    /**Returns  the selected QCGrade*/
    public String getQcGrade() {
        return (String) qcGradeComboBox.getSelectedItem();
    }

    /**Retruns the observer comment*/
    public String getObsComment() {
        if (obsCommentsTextField.getText() != null)
            return obsCommentsTextField.getText();
        else
            return " ";
    }

    /**Returns the selected status*/
    public void setStatus(String status) {
        statusComboBox.setSelectedItem(status);
    }

    /**Returns  the selected QCGrade*/
    public void setQcGrade(String grade) {
        if(grade != null && grade.length() > 0)
           qcGradeComboBox.setSelectedItem(grade);
        else   
           qcGradeComboBox.setSelectedIndex(0);   
    }

    /**Retruns the observer comment*/
    public void setObsComment(String comments) {
        obsCommentsTextField.setText(comments);
    }

}
