package org.eso.ohs.phase2.apps.ot.gui;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.text.ParseException;

import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;

import org.eso.ohs.core.docview.gui.DocView;
import org.eso.ohs.core.gui.models.BoNameModel;
import org.eso.ohs.core.gui.models.ISODateModel;
import org.eso.ohs.core.gui.models.LimitedCharsModel;
import org.eso.ohs.core.gui.models.TimeDurationModel;
import org.eso.ohs.core.utilities.Convert;
import org.eso.ohs.dfs.Queue;
import org.eso.ohs.persistence.Config;


/** This view displays the following information regarding a Queue:
 *  description, start, duration and telescope code 
 *  
 *  @version $Revision: 1.12 $
 *  @author  G.Passera, DMD/ESO, <gpassera@eso.org>
 *  <br>ChangeLog: cmavros 24-10-01 Added calls to ISODateModel() 
 *    and TimeDurationModel() 
 */
public class QueuePropertiesView extends JPanel implements PropertyChangeListener {

    /* Displays properties that belongs to this Queue objects */
    private Queue queue_; 
       
    /* GUI component for the startTime field */
    private DocView startDate_;

    /* GUI component for the duration field */
    private DocView duration_;

    /* GUI component for the queueName field */
    private DocView queueName_;

    /* GUI component for the description field */
    private DocView description_;

    /* GUI component for the telescope field */
    // nkornwei: replaced telescope_ JTextField with a DocView.
    //private JTextField telescope_;
    private DocView telescope_;

    /*
    private SimpleDateFormat stFormatter;
    private SimpleDateFormat dFormatter;
    */


    /**
     * 
     */
    public QueuePropertiesView(Queue newQueue) {
        super();
	/**
	 *  <br>ChangeLog: amchavan, 03-Jun-2002: removed dependency from persistence
        stFormatter = new SimpleDateFormat( Convert.ISO_DATE_FORMAT );
        dFormatter = new SimpleDateFormat( Convert.DURATION_FORMAT );
        stFormatter.setLenient(false);
        dFormatter.setLenient(false);
	*/
        setLayout(new BorderLayout());
        queue_ = newQueue;
        init();
        buildLayout();
        setObject(queue_);
    }

    /**
     * 
     */
    public void setObject(Queue newQueue) {
	// stdlog_.debug("CLASS ot.gui.QueuePropertiesView  setObject: " + newQueue);
        if(queue_ != null)
            queue_.removePropertyChangeListener(this);
        queue_ = newQueue;
        if(queue_ != null)
            queue_.removePropertyChangeListener(this);
        description_.setObjectProperty(queue_,"Description");
        duration_.setObjectProperty(queue_,"Duration"); 
        System.out.println("duration data Data " + ((JTextField)duration_.getComponent()).getText());
        // nkornwei - replaced telescope_ JTextField with a DocView.
        //telescope_.setText(queue_.getTelescope());
        //telescope_.setEnabled(false);
        telescope_.setObjectProperty(queue_,"Telescope");

        queueName_.setObjectProperty(queue_,"Name");
        startDate_.setObjectProperty(queue_,"StartTime"); 
       
    }
    
    /**
     * 
     */
    private void init() {
        JTextField tf;
        JTextArea texta;   

        /* Queue Name */
	tf = new JTextField(Config.getCfg().getMaxNameLength());
	tf.setDocument(new BoNameModel());
        queueName_ = new DocView(tf);

        /* Start Date */
        tf = new JTextField();
	tf.setDocument(new ISODateModel());
	startDate_ = new DocView(tf);	

        /* Duration */
        tf = new JTextField();	
	tf.setDocument(new TimeDurationModel());
	duration_ = new DocView(tf);

        /* Description */
        texta = new JTextArea();   
        texta.setDocument(new LimitedCharsModel());
        description_ = new DocView(texta);

        // nkornwei - replaced telescope_ JTextField with a DocView.
        //telescope_ = new JTextField(); 
        /* Telescope */
        JComboBox telescopeCombo = new JComboBox( new Object[] {
            "UT1", "UT2", "UT3", "UT4","VLTI", "NTT", "2.2", "3.6" });
        telescopeCombo.setSelectedIndex(0);

        telescope_ = new DocView(telescopeCombo);
    }

    /**
     * 
     */
    private void buildLayout() {
        GridLayout outerGrid  = new GridLayout(1,0);
        JPanel outerPanel = new JPanel(outerGrid);
        add(outerPanel,BorderLayout.NORTH);

        JPanel leftPanel = new JPanel();
        JPanel rightPanel = new JPanel();
        leftPanel.setLayout(new BorderLayout());
        rightPanel.setLayout(new BorderLayout());
        outerPanel.add(leftPanel);
        outerPanel.add(rightPanel);
       
        JPanel leftCol1 = new JPanel();
        JPanel leftCol2 = new JPanel();
        leftCol1.setLayout(new GridLayout(0,1));
        leftCol2.setLayout(new GridLayout(0,1));
        leftPanel.add(leftCol1,BorderLayout.WEST);
        leftPanel.add(leftCol2,BorderLayout.CENTER);

        leftCol1.add(new JLabel("Name"));
        leftCol1.add(new JLabel("Start Time"));
        leftCol1.add(new JLabel("Duration"));
        leftCol1.add(new JLabel("Telescope"));

        leftCol2.add(queueName_);
        leftCol2.add(startDate_);
        leftCol2.add(duration_);
        leftCol2.add(telescope_);

        rightPanel.add(new JLabel("Description"),BorderLayout.NORTH);
        rightPanel.add(new JScrollPane(description_),BorderLayout.CENTER);
    }
      
    /**
     * 
     */
    public void propertyChange(PropertyChangeEvent ev) {
        if (queue_ == null) {
		return;
	}
    }
    public void updateQueue() {
        String name = ((JTextField)queueName_.getComponent()).getText();
        String startDate = ((JTextField)startDate_.getComponent()).getText();
        String duration = ((JTextField)duration_.getComponent()).getText();
        String description = ((JTextArea)description_.getComponent()).getText();
        String telescope = (String)((JComboBox)telescope_.getComponent()).getSelectedItem();
        if(name != null)
            queue_.setName(name);
        if(startDate != null) {
            try {
                queue_.setStartTime(Convert.ISOdateToLong(startDate));
            }
            catch(ParseException e) {
            }
        }
        if(duration != null)
           queue_.setDuration(((Long)duration_.getObjectAdaptor().
           toMetaLevelObject(duration)).longValue());
        if(telescope != null)
           queue_.setTelescope(telescope);
        if(description != null)
            queue_.setDescription(description);   
    }

   
}
