package org.eso.ohs.phase2.apps.p2pp.actions;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.HeadlessException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.apache.log4j.Logger;
import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.dfs.FindingChart;
import org.eso.ohs.dfs.FindingChartSession;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.ObjectSelector;

/**
 * This allows for the detatching of FindingCharts from a SINGLE OB.
 * The user is presented with a list of finding charts which he can 
 * multi select on. The finding charts will then be disconnected 
 * from the OB.
 * 
 */

public class DetatchFindingCharts extends ActionSuperclass {

    private static Logger stdlog_ =
        Logger.getLogger(DetatchFindingCharts.class);

    private JFrame frm_;

    private ObjectSelector objSelector_;

    private Media device_;

    private ObservationBlock ob_;

    /**
     * Constructor for DetatchFindingCharts.
     * @param objSelector 
     * @param name
     * @param device
     */
    public DetatchFindingCharts(
        ObjectSelector objSelector,
        JFrame frm,
        Media device) {
        super(frm, "Detach Finding Charts");
        frm_ = frm;
        objSelector_ = objSelector;
        device_ = device;
    }

    /**
     * @see org.eso.ohs.core.gui.baseaction.ActionSuperclass#actionPerformedImpl(java.awt.event.ActionEvent)
     */
    public void actionPerformedImpl(ActionEvent ev) {
        ob_ = (ObservationBlock) objSelector_.getDisplayedObject();        
        FindingChart [] fcCharts = ob_.getFindingCharts();
        if(fcCharts == null || fcCharts.length == 0) {
            JOptionPane.showMessageDialog(
                frm_,
                new Object[] {
                    "There are no finding charts to be detached."                       
                    },
                "Finding Charts not found",
                JOptionPane.ERROR_MESSAGE);
            return;    

        }
        if(!Config.getCfg().isInOperationsMode()) {
            if(ob_.isCheckedIn()) {
                JOptionPane.showMessageDialog(
                    frm_,
                    new Object[] {
                        "OB is checked in. Cannot detach finding charts"                       
                        },
                    "Cannot detach finding charts",
                    JOptionPane.WARNING_MESSAGE);
                return;    
            
            }
            
        }   
        FindingChartDeSelectDialog dialog =
            new FindingChartDeSelectDialog(frm_,"Detatch Finding Chart for OB : " + ob_.getName());
        dialog.show();
    }


    private class FindingChartDeSelectDialog extends JDialog {

        private JList fcList;

        private JButton ok;

        private JButton cancel;
        
        
        private FindingChart [] findingCharts;
        
        
        private FindingChartListModel model;
        
        private OKActionListener okActionListener;
        
        
        private FindingChartListener findingChartListener;

        /**
         * Constructor for FindingChartDelectDialog.
         * @param arg0
         * @param arg1
         * @throws HeadlessException
         */
        public FindingChartDeSelectDialog(JFrame frm, String title)
            throws HeadlessException {
            super(frm, title, true);
            JPanel outerPanel = new JPanel(new BorderLayout(1, 1));
            JPanel leftPanel = new JPanel(new BorderLayout(1, 1));
            fcList = new JList();
            fcList.setVisibleRowCount(10);
            FindingChart[] FCs = ob_.getFindingCharts();
            fcList = new JList();
            model = new FindingChartListModel();
            for (int i = 0; i < FCs.length; i++) {
                model.addElement(FCs[i]);
            }
            fcList.setModel(model);
            fcList.setVisibleRowCount(10);

            fcList.setPrototypeCellValue("ZZZZZZZZZZZZZXXXXXXXXXXXXXXXXX");
            fcList.setFixedCellWidth(200);
            fcList.setSelectedIndex(0);
            fcList.addListSelectionListener(new FindingChartListener(fcList));
            //leftPanel.add(fcList,BorderLayout.CENTER);
            leftPanel.add(fcList, BorderLayout.CENTER);

            ok = new JButton("OK");
            ok.addActionListener(new OKActionListener());
            cancel = new JButton("Cancel");
            cancel.addActionListener(new CancelActionListener());
            JPanel rightPanel = new JPanel(new BorderLayout());
            JPanel northPanel = new JPanel(new GridLayout(2, 1));
            northPanel.add(ok);
            northPanel.add(cancel);
            rightPanel.add(northPanel, BorderLayout.NORTH);
            JPanel southPanel = new JPanel(new BorderLayout());
            rightPanel.add(southPanel, BorderLayout.SOUTH);
            outerPanel.add(leftPanel, BorderLayout.CENTER);
            leftPanel.add(fcList, BorderLayout.CENTER);
            outerPanel.add(rightPanel, BorderLayout.EAST);
            getContentPane().add(outerPanel);
            
            pack();
        }

        private class OKActionListener implements ActionListener {
            /**
             * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
             */
            public void actionPerformed(ActionEvent e) {
                stdlog_.debug("Action Event " + e);
                int [] selectedIndexes = fcList.getSelectedIndices();
                FindingChart [] detatchFCs = new FindingChart[selectedIndexes.length];
                for(int i=0;i<detatchFCs.length;i++) {
                    detatchFCs[i] = (FindingChart) model.elementAt(selectedIndexes[i]);
                }
                FindingChartSession.detatchFindingCharts(ob_,detatchFCs);            
                dispose();  
                
            }

        }

        private class CancelActionListener implements ActionListener {
            /**
             * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
             */
            public void actionPerformed(ActionEvent arg0) {
                dispose();
            }

        }
        
        private class FindingChartListener implements ListSelectionListener {
            
            private JList list_;
            
            public FindingChartListener(JList list) {
                 list_ = list;
            }
            
            
            /**
             * @see javax.swing.event.ListSelectionListener#valueChanged(javax.swing.event.ListSelectionEvent)
             */
            public void valueChanged(ListSelectionEvent ls) {
                if(ls.getValueIsAdjusting())
                     return;
                if(list_.getSelectionModel().isSelectionEmpty()) {     
                    ok.setEnabled(false);
                } else {
                    ok.setEnabled(true);
                }
            }
             
        }
        

        private class FindingChartListModel extends DefaultListModel {
            /**
              * Return a reference to the template name in the list
              * @param index index
              */

            public Object getElementAt(int index) {
                FindingChart fc = (FindingChart) super.getElementAt(index);
                return fc.getFcName();
            }

            public Object elementAt(int index) {
                return super.elementAt(index);
            }

        }

    }


}
