package org.eso.ohs.phase2.apps.p2pp.actions;

import java.awt.event.ActionEvent;
import java.io.File;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;

import javax.swing.JComponent;
import javax.swing.JOptionPane;

import org.apache.log4j.Logger;
import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.gui.baseaction.IncrementalAction;
import org.eso.ohs.core.gui.widgets.ErrorMessages;
import org.eso.ohs.core.gui.widgets.ExtensionFileFilter;
import org.eso.ohs.core.gui.widgets.SwingFileChooser;
import org.eso.ohs.core.gui.widgets.TextDisplayWidget;
import org.eso.ohs.core.utilities.ObjUtils;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.dfs.ObservingRun;
import org.eso.ohs.dfs.Target;
import org.eso.ohs.p2pp.Folder;
import org.eso.ohs.persistence.DirectoryNode;
import org.eso.ohs.persistence.ImportTargets;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.persistence.TargetImportException;
import org.eso.ohs.phase2.apps.p2pp.FolderGridView;
import org.eso.ohs.phase2.apps.p2pp.MenuGroups;
import org.eso.ohs.phase2.apps.p2pp.Persistence;

/**
 * Class to import a target list file in accordance to the simba format into
 * P2PP
 * 
 * <p>
 * $Id: ImportTargetListAction.java,v 1.7 2005/11/10 12:36:26 pnunes Exp $
 */

public class ImportTargetListAction extends ActionSuperclass {

	/** Standard log channel */
	private static Logger stdlog_ = Logger.getLogger(ImportBOAction.class);

	private FolderGridView folderView_;

	private static TextDisplayWidget log_ = new TextDisplayWidget(
			"Import Target List Report");

	/**
	 * Static reference to file chooser dialog
	 */

	private static SwingFileChooser chooser = new SwingFileChooser();

	/**
	 * Static refrences to file chooser filters
	 */

	private static ExtensionFileFilter tifFilter = new ExtensionFileFilter(
			"tif", "Taget Import Files");

	/**
	 * Constructs a ImportTargetLiust action
	 * 
	 * @param folderView
	 * 
	 * 
	 */

	public ImportTargetListAction(FolderGridView folderView) {
		super((JComponent) folderView, "Import Target List");
		folderView_ = folderView;
	}

	/**
	 * Event handler import a list of targets from a catalog.
	 * 
	 * @param ev
	 *            user-initiated ActionEvent to create an OD.
	 */

	public void actionPerformedImpl(ActionEvent ev) {
		Class type = folderView_.getDisplayedObjectClass();
		Folder folder = folderView_.getFolder();

		Long[] idLongs = folderView_.getAllSelected();

		if (idLongs == null || idLongs.length == 0) {
			JOptionPane.showMessageDialog(folderView_.getTopLevelAncestor(),
					new Object[] { "There are no selected observation blocks.",
							"Which observation blocks should be updated?", },
					"Nothing selected", JOptionPane.ERROR_MESSAGE);
			return; // nothing selected?
		}

		if (idLongs.length > 1) {
			String msg = "Please note that only the first selected observation block will be\n"
					+ "used to create new OB from the target list file.\n\n"
					+ "Are you sure you want to continue?\n";

			int option = JOptionPane.showConfirmDialog(folderView_.getTopLevelAncestor(), new Object[] {
					"There are " + idLongs.length
							+ " observation blocks selected.", msg },
					"Continue with Import?", JOptionPane.OK_CANCEL_OPTION);
			switch (option) {
			case JOptionPane.CLOSED_OPTION:
				return;
			case JOptionPane.CANCEL_OPTION:
				return;
			}
		}
		
	    Persistence objMgr = Persistence.getInstance();

	    ObservationBlock ob=null;
		try {
			ob = (ObservationBlock)objMgr.getBusObj(
						Media.LOCAL,
						idLongs[0].longValue(),
						ObservationBlock.class
					    );
		} catch (ObjectNotFoundException e1) {
			ErrorMessages.announceNoObject(getWdwComponent(),e1);
			return;
		} catch (ObjectIOException e1) {
			announceIOError(e1);
			return;
		}
	    if (objMgr.isAllowedToModify(ob)) {
			chooser.cancelSelection();
			chooser.setMultiSelectionEnabled(false);
			int response = chooser.showDialog(null, "Select Target List Files");
			if (response == JOptionPane.CLOSED_OPTION
					|| response == JOptionPane.CANCEL_OPTION)
				return;

			File f = chooser.getSelectedFile();
			// Parse file and create a list of targets. Assign
			// this to the targets variable below and and pass it to
			// Incremental action.

			ImportTargets importer = new ImportTargets();

			Target[] targets = new Target[0];
			try {
				targets = importer.importTargets(f);
				new P2PPIncrementalActionAutomaton(new IncrementalImport(targets,
						ob,
						folder.getDirNode(), folder.findObsRun()),
						(MenuGroups) folderView_.getTopLevelAncestor(),
						"Importing...");


			} catch (IOException e) {

				JOptionPane.showMessageDialog(folderView_.getTopLevelAncestor(),
						new Object[] { e.getMessage() }, "IO File error",
						JOptionPane.ERROR_MESSAGE);

				e.printStackTrace();
			} catch (TargetImportException e) {
				JOptionPane.showMessageDialog(folderView_.getTopLevelAncestor(),
						new Object[] { e.toString() }, "Import error",
						JOptionPane.ERROR_MESSAGE);
				e.printStackTrace();
			}
	    } else {
	    	 JOptionPane.showMessageDialog(
	    			 folderView_.getTopLevelAncestor(),
	 			    new Object[] {
	 			      "The observation block is locked",
	 			      "and may not be updated with target data."
	 			    },
	 			    "OB is locked",
	 			    JOptionPane.ERROR_MESSAGE
	 		        );
	 		        return;
	    }
         
		// Java1.4 port chooser.clearSelection();

	}

	/**
	 * Utility class to impoer and create a target list
	 */

	public class IncrementalImport implements IncrementalAction {

		/** Directory node the business objects will end up in */

		private DirectoryNode node_;

		/**
		 * List of targets ti import
		 */

		private Target[] targets_;
		
        private ObservationBlock ob_;
		 
		
		/**
		 * Observing ffr the new ob
		 */

		private ObservingRun obsRun_;

		/**
		 * Constructor
		 * 
		 * @param targets
		 * @param node
		 * @param obsRun
		 */
		public IncrementalImport(Target[] targets,ObservationBlock ob, DirectoryNode node,
				ObservingRun obsRun) {
			targets_ = targets;
			node_ = node;
			obsRun_ = obsRun;
			ob_ = ob;
		}

		public int getNumStepsRemaining(int counter) {
			return ObjUtils.howManyNotNull(targets_, counter);
		}

		public void startOfAction() {
		}

		public void endOfAction() {
		}

		/**
		 * Method to create a new OB and set the target of that ob.
		 * 
		 * @param counter
		 *            counter to show which element of obs to check in.
		 * @return true if the duplicating should continue and false otherwise.
		 */
		public void performStep(int counter) {
			if (targets_[counter] != null) {
				// Main body of the "loop" + fall thru
				Persistence objMgr = Persistence.getInstance();
				try {
					if (counter > 0) {
					    ob_ = (ObservationBlock)objMgr.deepCopy(
										Media.LOCAL,
										node_,
										ob_
									    );
					}

					Target destTG = ob_.getTarget();
					String[] props = targets_[counter].simpleProperties();
					try {
						targets_[counter].copyProperties(props, destTG);
					} catch (NoSuchMethodException e) {
						JOptionPane.showMessageDialog(folderView_.getTopLevelAncestor(),
								new Object[] { e.getMessage() }, "Reflection error",
								JOptionPane.ERROR_MESSAGE);

					} catch (IllegalAccessException e) {
						JOptionPane.showMessageDialog(folderView_.getTopLevelAncestor(),
								new Object[] { e.getMessage() }, "Reflection error",
								JOptionPane.ERROR_MESSAGE);

					} catch (InvocationTargetException e) {
						JOptionPane.showMessageDialog(folderView_.getTopLevelAncestor(),
								new Object[] { e.getMessage() }, "Reflection error",
								JOptionPane.ERROR_MESSAGE);

					}

				} catch (ObjectIOException e) {
					announceIOError(e);
				}

			}
		}

	}
}
