package org.eso.ohs.phase2.icdVcs;

import java.util.StringTokenizer;

import javax.swing.event.EventListenerList;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.TextUtils;
import org.eso.vlt.base.Ccs.CcsCommand;
import org.eso.vlt.base.Ccs.CcsCommandMonitor;
import org.eso.vlt.base.Ccs.CcsDebug;
import org.eso.vlt.base.Ccs.CcsErrorReplyEvent;
import org.eso.vlt.base.Ccs.CcsException;
import org.eso.vlt.base.Ccs.CcsReplyEvent;
import org.eso.vlt.base.Ccs.CcsSeqListener;

/**
 * Listener for events from the BOB interface
 *
 * <p>
 * <b>WARNING:</b>
 * This cannot be merged with the ScriptCmdBroadcaster currently,
 * due to limitations in the implementeation of the Java/Ccs library.
 * If you do merge them, one or the other will stop working due to a
 * Hashtable in the superclass.
 * 
 * <p>
 * NOTE: This class must be public in order for Java/Ccs to work.
 *       I'm not sure of the reason but it's probably to do with
 *       their use of the Reflection API.
 * 
 * <p>
 * $Id: EventCmdBroadcaster.java,v 1.5 2004/04/08 12:09:53 tcanavan Exp $
 * <br>
 * @author Bob Kemp
 */

public class EventCmdBroadcaster extends CcsSeqListener {

   /** Standard log channel */
   private static Logger stdlog_ =
       Logger.getLogger( EventCmdBroadcaster.class );

    static final int TEMPLATE_STATUS_EVENT  = 1;
    static final int OBSBLK_STATUS_EVENT    = 2;
    static final int GETPAF_EVENT    	    = 3;

    /** Support for handling listeners */
    protected EventListenerList listenerList_ = new EventListenerList();


    /**
     * Default constructor.
     * @exception CcsException CCS throws an exception
     */
    public EventCmdBroadcaster() throws CcsException {
	initialise();
    }

    /**
     * Initialisation method.  We keep this as a separate method, so
     * that the test routines can override it and avoid starting up CCS.
     * 
     * @exception CcsException CCS throws an exception
     */
    protected void initialise()
			    throws CcsException {
	CcsCommandMonitor evtMonitor = new CcsCommandMonitor("EVENT");
	stdlog_.debug("CommandMonitor created for EVENT");

	evtMonitor.addCommandListener(this);
	stdlog_.debug("script listener added");
    }


   /**
     * Handler method for ObsBlockStatus messages from BOB.
     * 
     * @param msg	    status message as a tcl array
     * @return	    null string
     *
     * <br>ChangeLog: A M Chavan, 19-Jan-2000, [0582] now uses 
     *                                  convertBobEventToStatus() 
     */
    public void ObsBlockStatus(String msg) {

	stdlog_.debug(">>> ObsBlockStatus( \"" + msg + "\" )" );

	String[] args = TextUtils.splitTclArray(msg);

	fireStatusEvent(OBSBLK_STATUS_EVENT, args);
    }


    /**
     * Handler method for TemplateStatus messages from BOB.
     * 
     * @param msg	    status message as a tcl array
     * @return	    null string
     */
    public void TemplateStatus(String msg) {
	stdlog_.debug(">>> TemplateStatus(" + msg + ")");
	String[] args = TextUtils.splitTclArray(msg);

	fireStatusEvent(TEMPLATE_STATUS_EVENT, args);
    }


    public void PAFReady(String pars) {
	CcsDebug.println(5, "Got PAFReady event: " + pars); 
	StringTokenizer st = new StringTokenizer(pars);
	String  envName = st.nextToken();
	String  procName = st.nextToken();
	String  path = st.nextToken();
	String  trash = st.nextToken();

	try {
	    // Added as Ccs listener as a side-effect in the constructor.
	    new GetPAFCommand (envName, procName, path, trash);
	}
	catch (CcsException e) {
	    e.printStackTrace();
	    CcsDebug.println(5,"PAFReady: Error: " +e);
	}
    }


    protected void fireStatusEvent(int eventType, String[] args) {
	Object[] listeners = listenerList_.getListenerList();

	for (int i = listeners.length-2; i>=0; i-=2) {

	    if (listeners[i] == EventCmdListener.class) {
		EventCmdListener lnr = (EventCmdListener)listeners[i+1];

		if (eventType == TEMPLATE_STATUS_EVENT) {
		    lnr.TemplateStatus(args[0], args[1], args[2],
					args.length >= 4 ? args[3] : "");
		}
		else if (eventType == OBSBLK_STATUS_EVENT){
		    lnr.ObsBlockStatus(args[0], args[1], args[2],
					args.length >= 4 ? args[3] : "");
		}
		else if (eventType == GETPAF_EVENT) {
		    lnr.GetPAFResult(args[0]);
		}
		else {
		    stdlog_.debug( "fireStatusEvent: unknown event type: " + eventType );
		    // throw( new UnknownEventTypeException("type: "+eventType));
		}
	    }
	}
    }


    public void addEventCmdListener(EventCmdListener lnr) {
	// org.eso.ohs.utilities.DebugUtils.stackTrace();
	listenerList_.add(EventCmdListener.class, lnr);
    }

    public void removeEventCmdListener(EventCmdListener lnr) {
	listenerList_.remove(EventCmdListener.class, lnr);
    }


    /**
     * Class to handle the "SCRIPT GetPAF" command and its replies.
     **/
    public class GetPAFCommand extends CcsCommand {

	private StringBuffer reply_ = new StringBuffer();


	public GetPAFCommand(String  envName,
			     String  procName,
			     String  path,
			     String  trash) throws CcsException {
	    super(envName, procName, "SCRIPT", "GetPAF " + path + " " + trash);
	}

	public void replyReceived(CcsReplyEvent reply) {
	    reply_.append(reply.getBuffer());	// concatenate
	    
	    if (reply.isLastReply()) {
		String fullReply = reply_.toString();

		CcsDebug.println(5, "Final reply on GetPAF in; fire event");
		CcsDebug.println(6, "Final reply is: " + fullReply);

		fireStatusEvent(GETPAF_EVENT, new String[]{ fullReply });
	    }
	    else {
		CcsDebug.println(5, "Partial reply on GetPAF"); 
	    }
	}

	public void errorReplyReceived(CcsErrorReplyEvent reply) {
	    CcsDebug.println(5, "Received error reply on GetPAF: >" 
			     +  reply.getBuffer() + "<");
	}
    }
}


