package org.eso.ohs.phase2.visibility;

/**
 * A PCFPlotterFilter is used to manipulate (round, truncate, etc) or reject
 * data values prior to their storage in a PCFPlotter.
 *
 * It a base class that is intended to be extended and passed to the
 * contructor of the PCFPlotter class.
 *
 * This base class is transparent and accepts all PCFIntervals.
 *
 * @author Nick Kornweibel for ESO.
 */

public class PCFPlotterFilter {

    /**
     * The input() method is called with every value which is to be set in
     * the PCFPlotter. The return code is the new value to be set instead
     * of the value issued in the PCFInterval.
     * If the value is to be rejected, the ValueRejectedException should be
     * thrown. This will return a failure code to the code attempting to
     * store the value.
     *
     * @param the PCFInterval that is requested to be stored in the PCFPlotter
     * @return the (possibly) modified value that will ne stored.
     * @throws PCFPlotterFilter.ValueRejectedException is the value is not
     * to be stored.
     */

    public double input(PCFInterval interval)
            throws PCFPlotterFilter.ValueRejectedException {

        // the base class does nothing than return the value from the interval.
        return interval.getValue();
    }

    /**
     * Throwing this exception indicates that the value is not to be
     * stored in the PCFPlotter.
     */

    public static class ValueRejectedException extends Exception {
        public ValueRejectedException() {};
    }
}
