package org.eso.ohs.phase2.visiplot;

/**
 * Extension of the BarChart bean used to display binary constraint data, 
 * such as FLI, moon angle, sun, etc.
 *
 * @author Nick Kornweibel for ESO
 */

import java.awt.Color;

import javachart.beans.chart.BarChart;
import javachart.chart.Bar;
import javachart.chart.Dataset;

import org.eso.ohs.core.utilities.MsgManager;
import org.eso.ohs.phase2.visibility.PCF;
import org.eso.ohs.phase2.visibility.PCF2Dataset;
import org.eso.ohs.phase2.visibility.PCFDataFeed;

public class ConstraintChart extends BarChart {

    /**
     * Create a chart from the given PCF. This means creatig a dataFeed
     * data model and extracting the data from the model.
     * The colorPcf is used to set the color - gray when the colorPcf is
     * 0.0 and darkGray when > 0.0.
     * @param pcf
     * @param colorPcf
     * @param precision
     * @param label
     */
  
    public ConstraintChart(
        PCF pcf,
        PCF colorPcf,
        int precision,
        String label, ChartColorItem colorItem) {

    	assert (pcf != null): MsgManager.errPrecondition("PCF is null");

    	
        PCFDataFeed data = new PCFDataFeed(pcf, precision);
        Dataset colorData = null;
        if (colorPcf != null) {
            colorData = PCF2Dataset.convertPCF2Dataset(colorPcf, precision);
        } 
        
        init(data, colorData, label, colorItem);
    }

    /**
     * Create a chart from the given PCF. This means creating a dataFeed
     * data model and extracting the data from the model.
     * @param pcf The PCF containing the data used to create the chart.
     * @param label The label that is to be displayed in the Y axis.
     */

    public ConstraintChart(PCF pcf, int precision, String label, ChartColorItem item) {
        PCFDataFeed data = new PCFDataFeed(pcf, precision);
        //Questa label e' il titolo del chart. va parametrizzato
        init(data, null, label, item);
    }

    /**
     * Initialize all graph settings - called from constructor.
     */

    private void init(
        PCFDataFeed dataFeed,
        Dataset colorDataSet,
        String label,
        ChartColorItem colorItem) {
        
    	dataFeed.addDataFeedListener(this);
        dataFeed.update();
        javachart.chart.BarChart chart = (javachart.chart.BarChart) getChart();
        
        System.out.println("Chart: " + label + 
        					" Area: " + 
        					colorItem.getAreaColor().getRed() + ", " + 
        					colorItem.getAreaColor().getGreen() + ", " + 
        					colorItem.getAreaColor().getBlue() +  
        					" Perimeter: " + 
        					colorItem.getPerimeterColor().getRed() + ", " + 
        					colorItem.getPerimeterColor().getGreen() + ", " + 
        					colorItem.getPerimeterColor().getBlue());
        					
     
       
        Dataset[] datasets = chart.getDatasets();
   	 	Dataset data = datasets[0];
   	 	data.getGc().setFillColor(colorItem.getAreaColor());
        data.getGc().setLineColor(colorItem.getPerimeterColor());
        Bar b = new ColorChangeBar(chart.getDatasets(), data,
                chart.getXAxis(), chart.getYAxis(), chart.getPlotarea());
           chart.setBar(b);

        if (colorDataSet != null) {
            // we want to indicate some bars in a different colour. (darkGray)
        	colorDataSet.getGc().setFillColor(colorItem.getPerimeterColor());

            // over-ride the standard Bar with our ColorChangeBar.
            Bar b1 = new ColorChangeBar(chart.getDatasets(), colorDataSet,
                chart.getXAxis(), chart.getYAxis(), chart.getPlotarea());
            chart.setBar(b1);
        }
        // remove the title
        chart.getBackground().setTitleString("");

        // Y Axis - original
        // set the axis title.
        chart.getYAxis().setTitleString(label);

        // Special Note: we want an axis title, but not the axis labels
        // by making the axis labels not visible, the title disapears as well
        // I have worked around this by making the axis labels visible
        // (so we can see the title), but have set their color to white
        // (so we can not see them against a white background.
        chart.getYAxis().setLabelVis(true);
     //   chart.getYAxis().setLabelColor(colorItem.getLabelsColor());
        chart.getYAxis().setLabelColor(Color.white);

        // make all tick marks invisible.
        chart.getYAxis().setMajTickVis(false);
        chart.getYAxis().setMinTickVis(false);

        // X Axis - ticks and labels are not displayed.
        chart.getXAxis().setLabelVis(false);
        chart.getXAxis().setLabelColor(colorItem.getLabelsColor());
        chart.getXAxis().setMajTickVis(false);
        chart.getXAxis().setMinTickVis(false);

        chart.getBar().setClusterWidth(1.0);
      //  chart.getDatasets()[0].getGc().setFillColor(Color.pink);
    }
}
