package org.eso.ohs.scripting;

/**
 * This is an adaptor class accepting a TimeInterval object and presenting
 * the TimeIntervalAPI interface.
 *
 * @author Nick Kornweibel
 */

import org.eso.ohs.dfs.TimeInterval;
import org.eso.ohs.persistence.Config;

public class TimeIntervalAdaptor implements TimeIntervalAPI {

    private TimeInterval timeInt_;

    /**
     * The constructor accepts a TimeInterval object, assumed non-null
     * @param the time interval
     */

    public TimeIntervalAdaptor(TimeInterval ti) {
        timeInt_ = ti;
    }

    /**
     * Get the name of this time interval
     * @return string containing name of time interval
     */

    public String getName() {
        return ScriptingUtilities.nullToEmptyString(TimeInterval.getDisplayName());
    }

    /**
     * Get the id of this time interval
     * @return string containing the id of this time interval
     */

    public String getId() {
        String rc;
        rc = ""+Config.getCfg().uniqueToTableId(timeInt_.getId());
        return rc;
    }


    /**
     * Get the start time
     * @return the start time
     */

    public long getStartTime() {
        return timeInt_.getStartTime();
    }

    /**
     * Get the end time
     * @return end time
     */

    public long getEndTime() {
        return timeInt_.getEndTime();
    }

    /**
     * Get the preference
     * @return preference
     */

    public int getPreference() {
        return timeInt_.getPreference();
    }

}
